<?php
/**
 * Debug functions.
 *
 * @package flo-sync
 */

namespace NMFS;

use Closure;

/**
 * Debug functions.
 */
class Debug {

	/**
	 * Setup debug functions.
	 */
	public static function setup(): void {
		add_action( 'query_vars', array( __CLASS__, 'add_query_var' ) );
	}

	/**
	 * Check if debug mode is enabled.
	 *
	 * @return bool
	 */
	public static function is_on(): bool {

		/**
		 * FLO_DEBUG can be set in wp-config.php to enable debug mode.
		 *
		 * @noinspection PhpUndefinedConstantInspection
		 */
		if (
			wp_get_environment_type() === 'local' ||
			( defined( 'FLO_DEBUG' ) && true === FLO_DEBUG )
		) {
			return true;
		} elseif ( self::check_cached_debug_status() ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Add flo-debug as allowed query var.
	 *
	 * @param array $vars Query vars.
	 */
	public static function add_query_var( array $vars ): array {
		$vars[] = 'flo-debug';
		return $vars;
	}

	/**
	 * Check if debug mode is cached to be enabled.
	 *
	 * @return bool
	 */
	private static function check_cached_debug_status(): bool {
		if ( current_user_can( 'manage_options' ) ) {
			if (
				get_transient( 'flo_debug' ) ||
				( isset( $_GET['flo-debug'] ) && 'on' === $_GET['flo-debug'] ) // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			) {
				set_transient( 'flo_debug', 'on', MINUTE_IN_SECONDS * 5 ); // Cache for 5 minutes.
				return true;
			}
		}

		return false;
	}

	/**
	 * Display the hooks registered for a given hook name.
	 *
	 * This method iterates over the global `$wp_filter` variable
	 * to find the hooks registered for the specified hook name
	 * and displays them with their priorities and associated functions/methods.
	 *
	 * @param string $hook_name The name of the hook.
	 *
	 * @return void
	 */
	public static function display_hooks( string $hook_name ): void {
		global $wp_filter;
		if ( isset( $wp_filter[ $hook_name ] ) ) {
			$hooks = $wp_filter[ $hook_name ]->callbacks;
			if ( ! empty( $hooks ) ) {
				foreach ( $hooks as $priority => $hooked ) {
					foreach ( $hooked as $hook ) {
						if ( is_string( $hook['function'] ) ) {
							printf(
								'<p>%s: %s - %s: %s</p>',
								esc_html__( 'Priority', 'flo-sync' ),
								esc_html( $priority ),
								esc_html__( 'Function', 'flo-sync' ),
								esc_html( $hook['function'] )
							);
						} elseif ( is_array( $hook['function'] ) ) {
							if ( is_object( $hook['function'][0] ) ) {
								printf(
									'<p>%s: %s - %s: %s</p>',
									esc_html__( 'Priority', 'flo-sync' ),
									esc_html( $priority ),
									esc_html__( 'Method', 'flo-sync' ),
									esc_html( get_class( $hook['function'][0] ) . '::' . $hook['function'][1] )
								);
							} else {
								printf(
									'<p>%s: %s - %s: %s</p>',
									esc_html__( 'Priority', 'flo-sync' ),
									esc_html( $priority ),
									esc_html__( 'Function', 'flo-sync' ),
									esc_html( $hook['function'][0] . '::' . $hook['function'][1] )
								);
							}
						} elseif ( $hook['function'] instanceof Closure ) {
							printf(
								'<p>%s: %s - %s</p>',
								esc_html__( 'Priority', 'flo-sync' ),
								esc_html( $priority ),
								esc_html__( 'Function: Closure', 'flo-sync' ),
							);
						}
					}
				}
			} else {
				printf(
					'<p>%s</p>',
					esc_html(
						sprintf(
							// translators: %s: hook name.
							__( 'No hooks found for %s.', 'flo-sync' ),
							$hook_name
						)
					)
				);
			}
		} else {
			printf(
				'<p>%s</p>',
				esc_html(
					sprintf(
					// translators: %s: hook name.
						__( 'No hooks registered for %s.', 'flo-sync' ),
						$hook_name
					)
				)
			);
		}
	}
}
