<?php
/**
 * Various helper functions.
 *
 * @package flo-sync
 */

namespace NMFS;

use DateTime;
use DateTimeZone;
use Exception;
use Throwable;
use WC_Log_Handler_File;

/**
 * Helper class.
 */
class Helpers {

	/**
	 * Truncate string to a certain length.
	 * Based on https://github.com/delight-im/PHP-Str/blob/8fd0c608d5496d43adaa899642c1cce047e076dc/src/Str.php#L599
	 *
	 * @param string $input_string String to truncate.
	 * @param int    $max_length   Maximum length of the string.
	 * @param string $ellipsis     Ellipsis to append to the string.
	 * @param bool   $safe         Whether to truncate at a word boundary.
	 *
	 * @return string
	 */
	public static function truncate_string(
		string $input_string,
		int $max_length = 30,
		string $ellipsis = '...',
		bool $safe = true
	): string {
		// If the string doesn't need truncating.
		if ( mb_strlen( $input_string ) <= $max_length ) {
			return $input_string;

		} else { // If the string needs truncating.
			// Calculate the actual maximum length without the ellipsis.
			$max_length -= mb_strlen( $ellipsis );

			// Truncate the string to the desired length.
			$raw_string = mb_substr( $input_string, 0, $max_length );

			// If we don't want to break words.
			if ( $safe ) {
				// If the truncated string *does* end _within_ a word.
				if ( ! preg_match( '/\\W/u', mb_substr( $input_string, $max_length - 1, 2 ) ) ) {
					// If there's some word boundary before.
					if ( preg_match( '/.*\\W/u', $raw_string, $matches ) ) {
						// Truncate there instead.
						$raw_string = $matches[0];
					}
				}
			}

			// Return the correctly truncated string together with the ellipsis.
			return $raw_string . $ellipsis;
		}
	}

	/**
	 * Get the current time in the format that the API expects.
	 *
	 * @return string Requested time.
	 */
	public static function get_current_time(): string {
		try {
			$date = new DateTime( 'now', new DateTimeZone( 'Europe/Stockholm' ) );

			return $date->format( 'Y-m-d\TH:i:s' );
		} catch ( Exception $e ) {
			Logger::error( $e->getMessage() );
			return '';
		}
	}

	/**
	 * Get age of a timestamp in a human-readable format.
	 *
	 * @param int $timestamp Timestamp to get age of.
	 *
	 * @return string
	 */
	public static function get_timestamp_age( int $timestamp ): string {
		$now = new DateTime();
		try {
			$then = new DateTime( '@' . $timestamp ); // '@' is used to pass a Unix timestamp to DateTime constructor
		} catch ( Throwable $t ) {
			Logger::error( $t->getMessage() );
			return '';
		}

		$diff = $now->diff( $then );

		if ( $diff->y ) {
			return $diff->y . ' ' . _n( 'year', 'years', $diff->y, 'flo-sync' );
		} elseif ( $diff->m ) {
			return $diff->m . ' ' . _n( 'month', 'months', $diff->m, 'flo-sync' );
		} elseif ( $diff->d ) {
			return $diff->d . ' ' . _n( 'day', 'days', $diff->d, 'flo-sync' );
		} elseif ( $diff->h ) {
			return $diff->h . ' ' . _n( 'hour', 'hours', $diff->h, 'flo-sync' );
		} elseif ( $diff->i ) {
			return $diff->i . ' ' . _n( 'minute', 'minutes', $diff->i, 'flo-sync' );
		} else {
			return $diff->s . ' ' . _n( 'second', 'seconds', $diff->s, 'flo-sync' );
		}
	}

	/**
	 * Get a WooCommerce log file URL.
	 *
	 * @param string $handle Log handle.
	 *
	 * @return string
	 */
	public static function get_wc_log_file_url( string $handle = 'flo-sync' ): string {
		if ( defined( 'WC_LOG_DIR' ) ) {
			return add_query_arg(
				array(
					'page'     => 'wc-status',
					'tab'      => 'logs',
					'log_file' => WC_Log_Handler_File::get_log_file_name( $handle ),
				),
				admin_url( 'admin.php' )
			);
		}

		return '';
	}

	/**
	 * Get a link tag to the WooCommerce log file.
	 *
	 * @param string|null $text   Text to display for link.
	 * @param string      $handle Log handle.
	 *
	 * @return string Link HTML tag.
	 */
	public static function get_wc_log_a_tag( ?string $text = null, string $handle = 'flo-sync' ): string {
		$url = self::get_wc_log_file_url( $handle );
		if ( empty( $url ) ) {
			return '';
		}
		$text = $text ?? __( 'View log', 'flo-sync' );
		return '<a href="' . esc_url( self::get_wc_log_file_url( $handle ) ) . '" target="_blank">' . esc_html( $text ) . '</a>';
	}
}
