<?php
/**
 * Handles plugin logging.
 *
 * @package flo-sync
 */

namespace NMFS;

use Throwable;

defined( 'ABSPATH' ) || exit;

/**
 * Class for logging.
 */
class Logger {

	/**
	 * Handle for log entries/files.
	 *
	 * @var string $log_handle
	 */
	private static string $log_handle = 'flo-sync';

	/**
	 * Add a log entry.
	 *
	 * @param string $level   One of the following:
	 *                        'emergency': System is unusable.
	 *                        'alert': Action must be taken immediately.
	 *                        'critical': Critical conditions.
	 *                        'error': Error conditions.
	 *                        'warning': Warning conditions.
	 *                        'notice': Normal but significant condition.
	 *                        'info': Informational messages.
	 *                        'debug': Debug-level messages.
	 * @param mixed  $message Log message.
	 */
	private static function log( string $level, mixed $message ): void {
		if ( ! function_exists( 'wc_get_logger' ) ) {
			return;
		}

		self::sanitize_log_level( $level );

		if ( ! self::is_log_level_enabled( $level ) ) {
			return;
		}

		if ( is_array( $message ) || is_object( $message ) ) {
			$message = var_export( $message, true ); // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_var_export
		} elseif ( is_null( $message ) ) {
			$message = '[NULL_VALUE]';
		}

		$source = self::$log_handle . '-' . $level;

		$mf_logger = wc_get_logger();
		$mf_logger->log( $level, $message, array( 'source' => $source ) );
	}

	/**
	 * Add an emergency log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function emergency( mixed $message ): void {
		self::log( 'emergency', $message );
	}

	/**
	 * Add a critical log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function critical( mixed $message ): void {
		self::log( 'critical', $message );
	}

	/**
	 * Add an error log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function error( mixed $message ): void {
		self::log( 'error', $message );
	}

	/**
	 * Add an alert log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function alert( mixed $message ): void {
		self::log( 'alert', $message );
	}

	/**
	 * Add a warning log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function warning( mixed $message ): void {
		self::log( 'warning', $message );
	}

	/**
	 * Add a notice log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function notice( mixed $message ): void {
		self::log( 'notice', $message );
	}

	/**
	 * Add an info log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function info( mixed $message ): void {
		self::log( 'info', $message );
	}

	/**
	 * Add a debug log entry.
	 *
	 * @param mixed $message Log message.
	 */
	public static function debug( mixed $message ): void {
		self::log( 'debug', $message );
	}

	/**
	 * Get all available log levels.
	 *
	 * @return array
	 */
	public static function get_log_levels(): array {
		return array(
			'emergency' => __( 'Emergency', 'flo-sync' ),
			'critical'  => __( 'Critical', 'flo-sync' ),
			'error'     => __( 'Error', 'flo-sync' ),
			'alert'     => __( 'Alert', 'flo-sync' ),
			'warning'   => __( 'Warning', 'flo-sync' ),
			'notice'    => __( 'Notice', 'flo-sync' ),
			'info'      => __( 'Info', 'flo-sync' ),
			'debug'     => __( 'Debug', 'flo-sync' ),
		);
	}

	/**
	 * Get the default log level.
	 *
	 * @return string
	 */
	public static function get_default_log_level(): string {
		return 'error';
	}

	/**
	 * Get the enabled log level stored in settings.
	 *
	 * @return string
	 */
	public static function get_enabled_log_level(): string {
		static $enabled_level;
		if ( ! isset( $enabled_level ) ) {
			try {
				$log_level = Settings::get( 'log_level', self::get_default_log_level() );
				self::sanitize_log_level( $log_level );
				$enabled_level = $log_level;
			} catch ( Throwable $t ) {
				$enabled_level = self::get_default_log_level();
				self::error( 'Failed to get log level: ' . $t->getMessage() );
			}
		}
		return $enabled_level;
	}

	/**
	 * Sanitize a log level.
	 *
	 * @param string $level Log level to sanitize. Passed by reference.
	 */
	private static function sanitize_log_level( string &$level ): void {
		$level = trim( strtolower( $level ) );
	}

	/**
	 * Get all enabled log levels.
	 *
	 * @return array
	 */
	private static function get_enabled_levels(): array {
		$enabled_level = self::get_enabled_log_level();

		if ( 'disabled' === $enabled_level ) {
			return array();
		}

		$levels = self::get_log_levels();

		// Get the numeric index of the search key.
		$index = array_search( $enabled_level, array_keys( $levels ), true );
		// Slice from 0 up to index + 1. Plus 1 to include the search key.
		return array_slice( $levels, 0, $index + 1, true );
	}

	/**
	 * Check if a log level is enabled.
	 *
	 * @param string $level Log level to check.
	 *
	 * @return bool True if enabled, false otherwise.
	 */
	private static function is_log_level_enabled( string $level ): bool {
		$enabled_levels = self::get_enabled_levels();

		return isset( $enabled_levels[ $level ] );
	}
}
