<?php
/**
 * Functions for plugin updates and settings upgrade.
 *
 * @package flo-sync
 */

namespace NMFS;

use stdClass;
use WP_Upgrader;

/**
 * Plugin update class.
 */
class Plugin_Update {

	/**
	 * Registers filters and actions.
	 */
	public function __construct() {
		if ( is_admin() ) {
			add_filter(
				'plugins_api',
				array(
					&$this,
					'get_plugin_info',
				),
				20,
				3
			);

			add_filter(
				'site_transient_update_plugins',
				array(
					&$this,
					'push_plugin_updates',
				)
			);

			add_filter(
				'upgrader_process_complete',
				array(
					&$this,
					'mf_after_update',
				),
				10,
				2
			);
		}
	}

	/**
	 * Filter callback for plugins_api. Gets plugin info from Northmill Server.
	 *
	 * @param object|false|array $res    Response object or array. Default false.
	 * @param string             $action The type of information being requested from the Plugin Install API.
	 * @param object             $args   Plugin API arguments.
	 *
	 * @return false|object|array Plugin info.
	 */
	public function get_plugin_info( object|false|array $res, string $action, object $args ): object|false|array {
		if ( ( 'plugin_information' !== $action ) || ( NMF_PLUGIN_SLUG !== $args->slug ) ) {
			return $res;
		}

		$remote_error = get_transient( NMF_PLUGIN_UPGRADE_ERROR_TRANS );

		if ( $remote_error ) {
			return $res;
		}

		$remote = $this->get_remote_plugin_info();

		if ( $remote ) {
			$remote              = json_decode( $remote['body'] );
			$res                 = new stdClass();
			$res->name           = $remote->name;
			$res->slug           = NMF_PLUGIN_SLUG;
			$res->version        = $remote->version;
			$res->tested         = $remote->tested;
			$res->requires       = $remote->requires;
			$res->author         = '<a href="https://www.northmill.com">Northmill</a>';
			$res->author_profile = 'https://www.northmill.com';
			$res->download_link  = $remote->download_url;
			$res->trunk          = $remote->download_url;
			$res->last_updated   = $remote->last_updated;
			$res->sections       = array(
				'changelog' => $remote->sections->changelog,
			);

			if ( ! empty( $remote->sections->screenshots ) ) {
				$res->sections['screenshots'] = $remote->sections->screenshots;
			}

			$res->banners = array(
				'low'  => 'https://downloadsflo.northmill.com/wp-content/uploads/2017/04/moreflo-system-1024x532.png',
				'high' => 'https://downloadsflo.northmill.com/wp-content/uploads/2017/04/moreflo-system-1024x532.png',
			);
		}

		return $res;
	}

	/**
	 * Filters update_plugins transient to maybe add our plugin to the array of available updates.
	 *
	 * @param mixed $transient Transient data.
	 *
	 * @return mixed
	 */
	public function push_plugin_updates( mixed $transient ): mixed {
		if ( empty( $transient->checked ) ) {
			return $transient;
		}

		if ( get_transient( NMF_PLUGIN_UPGRADE_ERROR_TRANS ) ) {
			return $transient;
		}

		$remote = $this->get_remote_plugin_info();

		$plugin_data    = get_plugin_data( NMF_PLUGIN_DIR . '/flo-sync.php' );
		$plugin_version = $plugin_data['Version'];

		if ( $remote ) {
			$remote = json_decode( $remote['body'] );
			if (
				$remote &&
				version_compare( $plugin_version, $remote->version, '<' ) &&
				version_compare( $remote->requires, get_bloginfo( 'version' ), '<' )
			) {
				$res                                 = new stdClass();
				$res->slug                           = NMF_PLUGIN_SLUG;
				$res->plugin                         = plugin_basename( __DIR__ ) . '/flo-sync.php';
				$res->new_version                    = $remote->version;
				$res->tested                         = $remote->tested;
				$res->package                        = $remote->download_url;
				$res->compatibility                  = new stdClass();
				$transient->response[ $res->plugin ] = $res;
			}
		}

		return $transient;
	}

	/**
	 * Get plugin info from Northmill server.
	 *
	 * @return array|false
	 */
	private function get_remote_plugin_info(): false|array {
		$remote = get_transient( NMF_PLUGIN_UPGRADE_TRANS );
		if ( ! $remote ) {
			$remote = wp_remote_get(
				'https://downloadsflo.northmill.com/plugin/flo-sync.json',
				array(
					'timeout' => 10,
					'headers' => array(
						'Accept' => 'application/json',
					),
				)
			);

			if (
				! is_wp_error( $remote ) &&
				isset( $remote[ NMF_RESPONSE ]['code'] ) &&
				200 === $remote[ NMF_RESPONSE ]['code'] &&
				! empty( $remote['body'] )
			) {
				set_transient( NMF_PLUGIN_UPGRADE_TRANS, $remote, HOUR_IN_SECONDS * 12 ); // 12-hour cache.
			} else {
				$remote = false;
				set_transient( NMF_PLUGIN_UPGRADE_ERROR_TRANS, true, HOUR_IN_SECONDS );
			}
		}

		return $remote;
	}

	/**
	 * After update callback.
	 *
	 * @param WP_Upgrader $upgrader_object Upgrader object.
	 * @param array       $options Options.
	 *
	 * @noinspection PhpUnusedParameterInspection WP action/filter function.
	 */
	public function mf_after_update( WP_Upgrader $upgrader_object, array $options ): void {
		if ( 'update' === $options['action'] && 'plugin' === $options['type'] ) {
			delete_transient( NMF_PLUGIN_UPGRADE_TRANS );
		}
	}
}
