<?php
/**
 * Plugin upgrade class.
 *
 * @package flo-sync
 */

namespace NMFS;

use NMFS\Sync\API\Backoffice_Accounts\Factory as Backoffice_Account_Factory;
use NMFS\Sync\API\Backoffice_Accounts\Sync as Backoffice_Account_Sync;
use NMFS\Sync\Cron\Cron;
use NMFS\Sync\Items\Bookkeeping\Article_Account_Codes\Article_Account_Codes;
use NMFS\Sync\Items\Bookkeeping\Article_Account_Codes\Factory as Article_Account_Code_Factory;
use NMFS\Sync\Items\Bookkeeping\Income_Account_Codes\Models\Income_Account_Code_Model;

/**
 * Plugin upgrade class.
 */
class Plugin_Upgrade {

	/**
	 * Registers filters and actions
	 *
	 * @return void
	 */
	public static function setup(): void {
		add_action(
			'admin_init',
			array(
				__CLASS__,
				'upgrade',
			)
		);
	}

	/**
	 * Upgrade plugin settings from previous versions
	 *
	 * @return void
	 */
	public static function upgrade(): void {
		if ( ! function_exists( 'get_plugin_data' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		$plugin_data    = get_plugin_data( NMF_PLUGIN_DIR . '/flo-sync.php' );
		$plugin_version = $plugin_data['Version'] ?? '';

		$plugin_version_option_name = 'nmf_version';
		$stored_version             = get_option( $plugin_version_option_name, '0.0.0' );

		// The plugin has been updated.
		if ( version_compare( $stored_version, $plugin_version, '<' ) ) {

			/**
			 * Upgrade to 3.0.0
			 * - Store Hub accounts as CPT
			 * - Store Hub account settings as CPT meta
			 * - Rename setting for current environment from 'is_active' to 'mf_sync_env', and replace value 'production' with 'prod'
			 * - Store shipping settings as Model and store to Hub Account CPT. Also save options for backwards compatibility.
			 * - Store mf_account_id (Article Account Codes) as models, and rename option to 'nmf_article_account_codes'. Also store with Hub Account CPT.
			 * - Store bookkeeping_account_payments (Income Account Codes) as models, and store with Hub Account CPT. Also save options for backwards compatibility.
			 * - Removes the old cron job, and sets up a new cron job.
			 */
			if ( version_compare( $stored_version, '3.0.0', '<' ) ) {
				$nmf_settings = get_option( 'moreflo_api_url' );

				// If old settings don't exist, this is most likely a new installation and no upgrade is needed.
				if ( $nmf_settings ) {
					/**
					 * Backup old settings.
					 */
					update_option( 'nmf_settings_backup_pre.3.0.0', $nmf_settings );

					// Remove company name and number from settings. Stored directly in Hub Account CPT instead.
					unset( $nmf_settings['mf_company_name'] );
					unset( $nmf_settings['mf_company_corp_no'] );

					/**
					 * Rename setting for current environment from 'is_active' to 'mf_sync_env'.
					 */
					if ( ! empty( $nmf_settings['is_active'] ) ) {
						$nmf_settings[ NMF_SYNC_ENV ] = $nmf_settings['is_active'];

						if ( 'production' === $nmf_settings[ NMF_SYNC_ENV ] ) {
							$nmf_settings[ NMF_SYNC_ENV ] = 'prod';
						}

						unset( $nmf_settings['is_active'] );
					}

					/**
					 * Migrate old receipt shipping settings.
					 */
					$old_shipping_product_sku          = $nmf_settings['mf_shipping_product_id'] ?? '';
					$old_shipping_article_account_code = $nmf_settings['mf_bookkeeping_account_shipping'] ?? '';
					$old_shipping_settings             = array(
						'article_sku'          => $old_shipping_product_sku,
						'article_account_code' => $old_shipping_article_account_code,
					);

					$curr_env = $nmf_settings[ NMF_SYNC_ENV ];

					if ( $old_shipping_product_sku ) {
						update_option( "nmf_shipping_product_sku_{$curr_env}_from_pre_3.0.0", $old_shipping_product_sku );
					}

					if ( $old_shipping_article_account_code ) {
						update_option( "nmf_shipping_article_account_code_{$curr_env}_from_pre_3.0.0", $old_shipping_article_account_code );
					}
					unset( $nmf_settings['mf_shipping_product_id'] );
					unset( $nmf_settings['mf_bookkeeping_account_shipping'] );

					/**
					 * Migrate old income account code settings.
					 */
					$payment_gateways = array();
					if ( function_exists( 'WC' ) ) {
						foreach ( WC()->payment_gateways()->payment_gateways() as $gateway ) {
							if ( 'cod' === $gateway->id ) {
								continue;
							}
							$payment_gateways[ $gateway->id ] = $gateway->get_title();
						}
					}
					$old_income_account_codes   = $nmf_settings['bookkeeping_account_payments'] ?? false;
					$income_account_code_models = array();
					if ( ! empty( $payment_gateways ) && ! empty( $old_income_account_codes ) && is_array( $old_income_account_codes ) ) {
						foreach ( $payment_gateways as $gateway_id => $gateway_label ) {
							$income_account_code = $old_income_account_codes[ $gateway_id ];
							if ( ! empty( $income_account_code ) ) {
								update_option( "nmf_income_account_code_{$gateway_id}_{$curr_env}_from_pre_3.0.0", $income_account_code );
								$income_account_code_models[ $gateway_id ] = new Income_Account_Code_Model(
									array(
										'payment_method_id' => $gateway_id,
										'account_code' => $income_account_code,
									)
								);
							}
						}
					}
					unset( $nmf_settings['bookkeeping_account_payments'] );

					/**
					 * Migrate old article account code settings.
					 */
					$old_account_codes = get_option( 'mf_account_ids' );
					$nmf_tax_classes   = Article_Account_Code_Factory::create_array_from_api( $old_account_codes );
					Article_Account_Codes::update( $nmf_tax_classes );
					delete_option( 'mf_account_ids' );
					update_option( 'nmf_article_account_codes_from_pre_3.0.0', $nmf_tax_classes );

					/**
					 * Migrate old test API settings.
					 */
					if ( isset( $nmf_settings['api_test_url'] ) ) {
						$account_data =
							array(
								'url'  => $nmf_settings['api_test_url'],
								'user' => $nmf_settings['api_test_user'],
								'pass' => $nmf_settings['api_test_pw'],
								'env'  => 'test',
								'type' => 'main',
							);

						if ( 'test' === $curr_env ) {
							$account_data['article_account_codes'] = $nmf_tax_classes;
							$account_data['shipping_settings']     = $old_shipping_settings;
							$account_data['income_account_codes']  = $income_account_code_models;
						}

						$test_account = Backoffice_Account_Factory::create(
							$account_data
						);

						Backoffice_Account_Sync::sync( $test_account, 'create' );
						unset( $nmf_settings['api_test_url'] );
						unset( $nmf_settings['api_test_user'] );
						unset( $nmf_settings['api_test_pw'] );
					}

					/**
					 * Migrate old prod API settings.
					 */
					if ( isset( $nmf_settings['api_production_url'] ) ) {
						$account_data =
							array(
								'url'  => $nmf_settings['api_production_url'],
								'user' => $nmf_settings['api_production_user'],
								'pass' => $nmf_settings['api_production_pw'],
								'env'  => 'prod',
								'type' => 'main',
							);

						if ( 'prod' === $curr_env ) {
							$account_data['article_account_codes'] = $nmf_tax_classes;
							$account_data['shipping_settings']     = $old_shipping_settings;
							$account_data['income_account_codes']  = $income_account_code_models;
						}

						$prod_account = Backoffice_Account_Factory::create(
							$account_data
						);
						Backoffice_Account_Sync::sync( $prod_account, 'create' );
						unset( $nmf_settings['api_production_url'] );
						unset( $nmf_settings['api_production_user'] );
						unset( $nmf_settings['api_production_pw'] );
					}

					/**
					 * Store settings.
					 */
					Settings::update_all( $nmf_settings );

					/**
					 * Removing the old cron job if it exists.
					 */
					if ( wp_next_scheduled( 'mf_sync_cron_schedule' ) ) {
						wp_clear_scheduled_hook( 'mf_sync_cron_schedule' );
					}
					if ( wp_next_scheduled( 'mf_sync_orders' ) ) {
						wp_clear_scheduled_hook( 'mf_sync_orders' );
					}
					if ( wp_next_scheduled( 'mf_sync_articles' ) ) {
						wp_clear_scheduled_hook( 'mf_sync_articles' );
					}
				}
			}

			$nmf_settings = Settings::get_all();

			// If old settings don't exist, this is most likely a new installation and no upgrade is needed in most cases.
			$is_nmf_settings_empty = empty( $nmf_settings );

			/**
			 * Upgrade to 3.0.6
			 * - Change log_enabled to log_level, enabling logging different levels.
			 */
			if ( version_compare( $stored_version, '3.0.6', '<' ) && ! $is_nmf_settings_empty ) {
				/**
				 * Migrate old log_enabled setting.
				 */
				if ( isset( $nmf_settings['log_enabled'] ) ) {
					if ( Settings::is_setting_enabled( $nmf_settings['log_enabled'], false, true ) ) {
						$nmf_settings['log_level'] = 'debug';
					} else {
						$nmf_settings['log_level'] = Logger::get_default_log_level();
					}
					unset( $nmf_settings['log_enabled'] );
				}
			}

			/**
			 * Upgrade to 3.0.30
			 * - Change log_enabled to log_level, enabling logging different levels.
			 */
			if ( version_compare( $stored_version, '3.0.30', '<' ) && ! $is_nmf_settings_empty ) {

				/**
				 * Migrate old log_enabled setting.
				 */
				if ( isset( $nmf_settings['adjust_stock_if_order_open_state'] ) ) {
					if ( Settings::is_setting_enabled( $nmf_settings['adjust_stock_if_order_open_state'], false, true ) ) {
						$nmf_settings['sync_order_stock_adjustment'] = 'order_opened';
					}
					unset( $nmf_settings['adjust_stock_if_order_open_state'] );
				}

				if ( isset( $nmf_settings['receipts_manages_stock'] ) ) {
					unset( $nmf_settings['receipts_manages_stock'] );
				}
			}

			/**
			 * Upgrade to 3.0.32
			 * - Delete old store of invalid Hub accounts.
			 */
			if ( version_compare( $stored_version, '3.0.32', '<' ) ) {
				delete_option( 'mf_invalid_bo_accounts' );
			}

			/**
			 * Upgrade to 3.1.0
			 * - Delete old sync per page setting (moved to manual sync page).
			 */
			if ( version_compare( $stored_version, '3.1.0', '<' ) && ! $is_nmf_settings_empty ) {
				if ( isset( $nmf_settings['mf_sync_items_per_page'] ) ) {
					unset( $nmf_settings['mf_sync_items_per_page'] );
				}
			}

			/**
			 * Upgrade to 3.2.18
			 * - Delete deprecated cron check email sent option.
			 */
			if ( version_compare( $stored_version, '3.2.18', '<' ) ) {
				delete_option( 'mf_sync_cron_check_email_sent' );
				Cron::get_cron_event( 'sync_checker' )->schedule();
			}

			/**
			 * Store settings.
			 */
			Settings::update_all( $nmf_settings );

			/**
			 * Update the stored plugin version.
			 */
			update_option( $plugin_version_option_name, $plugin_version );
		}
	}
}
