<?php
/**
 * Handles plugins settings.
 *
 * @package flo-sync
 */

namespace NMFS;

use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Envs;
use NMFS\Sync\Cron\Cron;
use NMFS\Sync\Items\Orders\Receipts\Helpers as Receipts_Helpers;
use NMFS\Sync\Items\Orders\Receipts\Models\Payment_Gateway_Collection;
use NMFS\Sync\Items\Orders\Receipts\Receipts;

/**
 * Class Settings
 */
class Settings {

	/**
	 * Settings array.
	 *
	 * @var array
	 */
	private static array $settings;

	/**
	 * Saves settings on form submission.
	 *
	 * @phpcs:disable WordPress.Security.NonceVerification.Missing
	 */
	public static function maybe_update_from_post(): void {
		if ( ! isset( $_POST['mf_submit_options'] ) ) {
			return;
		}

		$settings = self::get_all();

		/**
		 * Connection settings.
		 */
		$settings[ NMF_SYNC_ENV ] = sanitize_text_field( $_POST['mf_env_select'] );

		Backoffice_Accounts::update_from_settings( $_POST['bo_accounts'] );

		/**
		 * Sync settings.
		 */
		$settings[ NMF_SYNC_STATUS ]                   = self::is_post_enabled( NMF_SYNC_STATUS );
		$settings['sync_new_orders_to_apps']           = self::is_post_enabled( 'sync_new_orders_to_apps', 'enabled' );
		$settings['sync_publish_products_status']      = self::is_post_enabled( 'sync_publish_products_status' );
		$settings['sync_only_product_stock_amount']    = self::is_post_enabled( 'sync_only_product_stock_amount' );
		$settings[ NMF_IS_PRICE_ACTIVE ]               = self::is_post_enabled( 'sync_price_select' );
		$settings['sync_article_description']          = self::is_post_enabled( 'sync_article_description' );
		$settings['is_pic_active']                     = self::is_post_enabled( 'sync_pic_select' );
		$settings['is_product_title_active']           = self::is_post_enabled( 'sync_product_title_select' );
		$settings['sync_product_display_order_status'] = self::is_post_enabled( 'sync_product_display_order_select' );
		$settings['is_tax_class_active']               = self::is_post_enabled( 'sync_product_tax_class' );
		$settings['is_campaigns_active']               = self::is_post_enabled( 'sync_campaigns' );
		$settings['is_product_categories_active']      = self::is_post_enabled( 'sync_product_categories' );

		$settings['sync_order_stock_adjustment']  = sanitize_text_field( $_POST['mf_order_stock_adjustment'] ?? '' );
		$settings[ NMF_CRON_SYNC_EVENT_INTERVAL ] = sanitize_text_field( $_POST[ NMF_CRON_SYNC_EVENT_INTERVAL ] ?? '' );

		/**
		 * Receipt settings.
		 */
		$settings['is_receipt_active'] = self::is_post_enabled( 'sync_receipt_active' );

		/**
		 * Other settings.
		 */
		$settings['log_level'] = sanitize_text_field( $_POST['mf_log_level'] ?? '' );

		$settings['hub_order_number_prefix'] = sanitize_text_field( $_POST['hub_order_number_prefix'] ?? '' );

		self::update_all( $settings );

		wp_safe_redirect( get_admin_url( null, 'admin.php?page=' . NMF_SYNC_SETTINGS_SLUG ) );
		exit;
	}
	// phpcs:enable

	/**
	 * Check if a post checkbox is checked.
	 *
	 * @param string $key      Key to check.
	 * @param string $fallback Fallback value, if key is not set.
	 *
	 * @return string
	 *
	 * @phpcs:disable WordPress.Security.NonceVerification.Missing
	 */
	private static function is_post_enabled( string $key, string $fallback = 'disabled' ): string {
		if ( ! isset( $_POST[ $key ] ) ) {
			return $fallback;
		}

		if ( 'enabled' === $_POST[ $key ] || 'checked' === $_POST[ $key ] ) {
			return $_POST[ $key ];
		}

		return 'disabled';
	}
	// phpcs:enable

	/**
	 * Get all stored settings.
	 *
	 * @return array
	 */
	public static function get_all(): array {
		if ( ! isset( self::$settings ) ) {
			self::$settings = get_option( NMF_SETTINGS, array() );
		}
		return self::$settings;
	}

	/**
	 * Update all plugin settings.
	 *
	 * @param array $settings Settings to store.
	 *
	 * @return void
	 */
	public static function update_all( array $settings ): void {
		self::$settings = $settings;
		update_option( NMF_SETTINGS, $settings );

		if (
			isset( $settings[ NMF_SYNC_STATUS ] ) &&
			isset( $settings[ NMF_CRON_SYNC_EVENT_INTERVAL ] )
		) {
			$cron_event = Cron::get_cron_event( 'sync' );

			$cron_event->update_settings(
				self::is_setting_enabled(
					$settings[ NMF_SYNC_STATUS ],
					false,
					true
				),
				$settings[ NMF_CRON_SYNC_EVENT_INTERVAL ]
			);
		}
	}

	/**
	 * Get a specific setting.
	 *
	 * @param string     $key           Setting key.
	 * @param mixed|null $default_value Value to default to if $key isn't set.
	 *
	 * @return mixed
	 */
	public static function get( string $key, mixed $default_value = null ): mixed {
		$settings = self::get_all();

		return $settings[ $key ] ?? $default_value;
	}

	/**
	 * Check if a setting is enabled.
	 *
	 * @param string $key           Setting key.
	 * @param bool   $default_value Value to default to if $key isn't set.
	 * @param bool   $key_is_value  If the key is the value to check.
	 *
	 * @return bool
	 */
	public static function is_setting_enabled(
		string $key,
		bool $default_value = false,
		bool $key_is_value = false
	): bool {
		$setting = $key_is_value ? $key : self::get( $key );

		if ( null === $setting ) {
			return $default_value;
		}

		return 'enabled' === $setting || 'checked' === $setting;
	}

	/**
	 * Check if WooCommerce decimals are set up correct.
	 *
	 * @return bool
	 */
	public static function is_wc_decimals_correct(): bool {
		return function_exists( 'wc_get_price_decimals' ) && 2 === wc_get_price_decimals();
	}

	/**
	 * Print disabled status for receipt select input.
	 *
	 * @param ?Payment_Gateway_Collection[] $invalid_gateway_collections Array filled with invalid payment gateways. Passed by reference.
	 * @param ?array                        $invalid_accounts            Array filled with invalid Hub accounts. Passed by reference.
	 *
	 * @return bool Whether the field is disabled
	 */
	public static function get_receipt_select_input_disabled_status( ?array &$invalid_gateway_collections = array(), ?array &$invalid_accounts = array() ): bool {
		// If receipt creation is enabled, don't disable the field.
		// You should always be able to disable receipt creation if it's enabled.
		if ( Receipts::is_enabled() ) {
			return false;
		}

		$return = false;
		if (
			! self::is_wc_decimals_correct() ) {
			$return = true;
		}

		if (
			! Receipts_Helpers::is_required_income_account_codes_set(
				null,
				$invalid_gateway_collections,
				false
			)
		) {
			$return = true;
		}

		if (
			! Receipts_Helpers::is_required_vat_account_codes_set( $invalid_accounts, false )
		) {
			$return = true;
		}

		return $return;
	}

	/**
	 * Get extra HTML for receipts enabled field.
	 *
	 * @param ?Payment_Gateway_Collection[] $invalid_gateway_collections Array filled with invalid payment gateways.
	 * @param ?array                        $invalid_vat_bo_accounts     Array filled with Hub accounts with invalid VAT accounts.
	 *
	 * @return string
	 */
	public static function get_receipts_enabled_field_extra_html(
		?array $invalid_gateway_collections,
		?array $invalid_vat_bo_accounts
	): string {
		ob_start();
		if ( ! self::is_wc_decimals_correct() ) {
			echo '<p class="description" style="color: red;">';
				echo wp_kses_post(
					sprintf(
						/* translators: %1$s: Start of WooCommerce settings page a tag. %2$s: end of a tag. */
						__( 'Number of decimals must be set to 2 in WooCommerce, set it %1$shere%2$s.', 'flo-sync' ),
						'<a href="' . esc_url( admin_url( 'admin.php?page=wc-settings&tab=general' ) ) . '" target="_blank">',
						'</a>'
					)
				);
			echo '</p>';
		}

		if ( ! empty( $invalid_gateway_collections ) ) {
			echo '<p class="description" style="color: red;">';
			esc_html_e( 'The following payment methods are missing income account codes:', 'flo-sync' );
			echo '</p>';
			echo '<ul class="description" style="color: red;">';

			foreach ( $invalid_gateway_collections as $invalid_gateway_collection ) {
				$payment_gateway    = $invalid_gateway_collection->payment_gateway;
				$backoffice_account = $invalid_gateway_collection->backoffice_account;
				$iac_pay_type       = $invalid_gateway_collection->iac_pay_type;

				echo wp_kses_post(
					'<li>' .
					'<i>' .
					$payment_gateway->method_title . ' - ' . Helpers::truncate_string( $payment_gateway->description ?? '' ) .
					' (' . $iac_pay_type . ') ' .
					'</i>' .
					__( 'in Hub account', 'flo-sync' ) . ' ' .
					'<i>' .
					$backoffice_account->get_display_name() .
					'</i>' .
					'</li>'
				);
			}
			echo '</ul>';
		}

		if ( ! empty( $invalid_vat_bo_accounts ) ) {
			echo '<p class="description" style="color: red;">';
			esc_html_e( 'The following Hub accounts are missing are missing VAT account codes. Account codes are required for 0%, 6%, 12% and 25%.', 'flo-sync' );
			echo '</p>';
			echo '<ul class="description" style="color: red;">';
			foreach ( $invalid_vat_bo_accounts as $bo_accounts ) {
				echo wp_kses_post(
					'<li>' . $bo_accounts->get_display_name() . '</li>'
				);
			}
			echo '</ul>';
		}

		return ob_get_clean();
	}

	/**
	 * Backwards compatibility for old settings.
	 * If no value is set in the API, try to get it from the old setting.
	 * Delete old setting after getting it the first time.
	 *
	 * @param string $setting_name_without_env Setting name with env part replaced with %s for sprintf().
	 * @param mixed  $api_value                Value from API.
	 *
	 * @return mixed Value from API or from old setting.
	 */
	public static function maybe_get_old_setting_value( string $setting_name_without_env, mixed $api_value ): mixed {
		$current_env = Envs::get_current_env();

		$old_setting_name = sprintf( $setting_name_without_env, $current_env );

		$old_value = get_option( $old_setting_name, null );

		if ( is_null( $old_value ) ) {
			return $api_value;
		}

		if ( empty( $api_value ) && $old_value ) {
			$api_value = $old_value;
		}

		delete_option( $old_setting_name );

		return $api_value;
	}
}
