<?php
/**
 * Contains API calls available for both main and stock accounts.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\API;

use NMFS\Sync\Items\Orders\Receipts\Models\Receipt_Model;

/**
 * Class Flo_API
 */
class API extends Abstract_API_Base {

	/**
	 * Get store settings from API.
	 *
	 * @return object|null
	 */
	public function get_store_settings(): ?object {
		return $this->request_object( 'GET', 'settings' );
	}

	/**
	 * Get bookkeeping account codes.
	 *
	 * @param array $args Query parameters.
	 *
	 * @return object|null Decoded JSON object of account codes, or null if request failed.
	 */
	public function get_account_codes( array $args ): ?object {
		return $this->request_object( 'GET', 'accountcodes', $args );
	}

	/**
	 * Get VAT accounts.
	 *
	 * @param array $args Query parameters.
	 *
	 * @return array|null    Decoded JSON array of VAT accounts, or null if request failed.
	 */
	public function get_vat_accounts( array $args ): ?array {
		return $this->request_array( 'GET', 'settings/accounting/vat', $args );
	}

	/**
	 * Get income account codes.
	 *
	 * @param array $args Query parameters.
	 *
	 * @return array|null    Decoded JSON array of income account codes, or null if request failed.
	 */
	public function get_income_account_codes( array $args ): ?object {
		return $this->request_object( 'GET', 'settings/accounting/income', $args );
	}

	/**
	 * Get other account codes.
	 *
	 * @param array $args Query parameters.
	 *
	 * @return array|null    Decoded JSON array of income account codes, or null if request failed.
	 */
	public function get_other_account_codes( array $args ): ?object {
		return $this->request_object( 'GET', 'settings/accounting/other', $args );
	}

	/**
	 * Get article.
	 *
	 * @param string $article_no Article number.
	 *
	 * @return array|null Decoded JSON object of article, or null if request failed.
	 */
	public function get_article( string $article_no ): ?array {
		$article_data = $this->request_array( 'GET', 'articles/' . $article_no );

		return $article_data['Article'] ?? null;
	}

	/**
	 * Get articles from API.
	 *
	 * @param array $args API request parameters.
	 *
	 * @return array|null Decoded JSON object of articles, or null if request failed.
	 */
	public function get_articles( array $args ): ?array {
		return $this->request_array( 'GET', NMF_ARTICLES_ENDPOINT, $args );
	}

	/**
	 * Get orders.
	 *
	 * @param null|array $params API parameters.
	 *
	 * @return array|null
	 */
	public function get_orders( ?array $params ): ?array {
		return $this->request_array( 'GET', 'orders', $params, 'v3' );
	}

	/**
	 * Post order to API.
	 *
	 * @param array $order Order data.
	 *
	 * @return object|null JSON encoded api response, or null if failed.
	 */
	public function post_order( array $order ): ?object {
		$body = wp_json_encode( array( $order ) );

		return $this->request_object( 'POST', 'orders', $body, 'v3' );
	}

	/**
	 * Get customer from API
	 *
	 * @param array $params Query parameters.
	 *
	 * @return array|null Decoded JSON array of customers, or null if request failed.
	 */
	public function get_customer( array $params ): ?array {
		// Search type can be one of the following: 'AllFields', 'CustomerNumber', 'Name', 'IdentityNumber', 'Email', 'Mobile'.
		$params = array_merge(
			array(
				'SearchPhrase'         => '',
				'SearchType'           => 'AllFields',
				'PageSize'             => 0,
				'ExactMatch'           => true,
				'OnlyInternalCustomer' => false,
				'OnlyExternalCustomer' => false,
			),
			$params,
		);

		$body = wp_json_encode( $params );
		$body = $body ? $body : '';

		$response = $this->request( 'POST', 'customers/find', $body );

		if ( $response ) {
			$data = json_decode( $response, true );

			if ( is_array( $data ) ) {
				return $data;
			}
		}

		return null;
	}

	/**
	 * Create receipt in API.
	 *
	 * @param Receipt_Model $receipt Receipt data.
	 *
	 * @return false|string JSON encoded api response, or false if failed.
	 */
	public function post_receipt( Receipt_Model $receipt ): false|string {
		$body = wp_json_encode( $receipt );
		$body = $body ? $body : '';

		return $this->request( 'POST', 'receipts', $body );
	}
}
