<?php
/**
 * API core functions.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\API;

use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;
use NMFS\Logger;

/**
 * Class Flo_API_Core
 */
abstract class Abstract_API_Base {

	/**
	 * API account.
	 *
	 * @var Backoffice_Account_Model
	 */
	public Backoffice_Account_Model $account;

	/**
	 * API URL.
	 *
	 * @var string $url
	 */
	protected string $url;

	/**
	 * API account username.
	 *
	 * @var string $user
	 */
	protected string $user;

	/**
	 * API account password.
	 *
	 * @var string $pass
	 */
	protected string $pass;

	/**
	 * Construct the API object.
	 *
	 * @param Backoffice_Account_Model $account The account to use.
	 */
	public function __construct( Backoffice_Account_Model $account ) {
		$this->account = $account;
		$this->url     = $account->url;
		$this->user    = $account->user;
		$this->pass    = $account->pass;

		if ( ! $this->url || ! $this->user || ! $this->pass ) {
			Logger::error( 'Invalid credentials on API construction' );
			Logger::error( $this->account );
		}
	}

	/**
	 * Make API request.
	 *
	 * @param string            $method   HTTP Method. One of: GET, POST, PUT, PATCH, DELETE.
	 * @param string            $endpoint API endpoint. E.g. 'articles'.
	 * @param string|array|null $body     Request body. Strings are passed in request body as is, arrays are converted to query string.
	 * @param string            $version  API version. E.g. 'v2'.
	 * @param array             $headers  Request headers.
	 *
	 * @return false|string     Response body, or false on failure.
	 */
	protected function request(
		string $method,
		string $endpoint,
		null|string|array $body = null,
		string $version = 'v2',
		array $headers = array()
	): false|string {
		if ( empty( $this->url ) || empty( $this->user ) || empty( $this->pass ) ) {
			Logger::error( 'Invalid API credentials' );
			Logger::error( 'URL: ' . $this->url );
			Logger::error( 'User: ' . $this->user );
			Logger::error( 'Pass: ' . $this->pass );
			Logger::error( 'Method: ' . $method );
			Logger::error( 'Endpoint: ' . $endpoint );

			return false;
		}

		$method   = strtoupper( $method );
		$endpoint = strtolower( $endpoint );
		$version  = strtolower( $version );

		if ( $version ) {
			$version .= '/';
		}
		$request_url = sprintf( '%s%s%s/', $this->url, $version, $endpoint );

		$headers['Authorization'] = 'Basic ' . base64_encode( "$this->user:$this->pass" ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_encode
		$headers['Content-Type']  = 'application/json; charset=utf-8';

		$params = array(
			'method'  => $method,
			'headers' => $headers,
			'timeout' => 30,
		);

		$body = $this->parse_request_body( $body, $method, $endpoint, $version, $headers );

		if ( ! empty( $body ) ) {
			if ( is_string( $body ) ) {
				$params['body'] = $body;
			} else { // Is array.
				$request_url .= '?' . http_build_query( $body );
			}
		}

		Logger::debug( 'Request start - ' . $method . ' ' . $endpoint . ' from Hub account ID: ' . $this->account->wp_id . '. Body:' );
		Logger::debug( $body );

		usleep( 200 * 1000 ); // Quiet down api calls, 200 ms sleep.
		$response = $this->get_request_response( $request_url, $params );

		Logger::debug( 'Request end - ' . $method . ' ' . $endpoint );

		return $response;
	}

	/**
	 * Get the response from the API request, decoded as JSON to array.
	 *
	 * @param string            $method   HTTP Method. One of: GET, POST, PUT, PATCH, DELETE.
	 * @param string            $endpoint API endpoint. E.g. 'articles'.
	 * @param null|string|array $body     Request body.
	 * @param string            $version  API version. E.g. 'v2'.
	 * @param array             $headers  Request headers.
	 *
	 * @return ?array
	 */
	protected function request_array(
		string $method,
		string $endpoint,
		null|string|array $body = '',
		string $version = 'v2',
		array $headers = array()
	): ?array {
		$response = $this->request( $method, $endpoint, $body, $version, $headers );

		if ( $response ) {
			$data = json_decode( $response, true );

			if ( is_array( $data ) ) {
				return $data;
			}
		}

		return null;
	}

	/**
	 * Get the response from the API request, decoded as JSON to array.
	 *
	 * @param string            $method   HTTP Method. One of: GET, POST, PUT, PATCH, DELETE.
	 * @param string            $endpoint API endpoint. E.g. 'articles'.
	 * @param null|string|array $body     Request body.
	 * @param string            $version  API version. E.g. 'v2'.
	 * @param array             $headers  Request headers.
	 *
	 * @return ?object
	 */
	protected function request_object(
		string $method,
		string $endpoint,
		null|string|array $body = '',
		string $version = 'v2',
		array $headers = array()
	): ?object {
		$response = $this->request( $method, $endpoint, $body, $version, $headers );

		if ( $response ) {
			$data = json_decode( $response );

			if ( is_object( $data ) ) {
				return $data;
			}
		}

		return null;
	}

	/**
	 * Get API request response.
	 *
	 * @param string $request_url Request URL.
	 * @param array  $params      Request parameters.
	 * @param int    $retry_count Retry count.
	 *
	 * @return false|string Response body, or false on failure.
	 */
	private function get_request_response( string $request_url, array $params, int $retry_count = 0 ): false|string {

		$request     = wp_remote_request( esc_url_raw( $request_url ), $params );
		$status_code = wp_remote_retrieve_response_code( $request );

		if ( is_wp_error( $request ) ) {
			Logger::error( 'START: API Request failed.' );
			Logger::error( $request_url );
			Logger::error( print_r( $params, true ) ); // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r

			$messages = $request->get_error_messages();

			if ( is_array( $messages ) && ! empty( $messages ) ) {
				Logger::error( print_r( $messages, true ) ); // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r
			}

			Logger::error( 'END: API Request failed.' );
		} elseif ( ! empty( $request['body'] ) && is_string( $request['body'] ) && 200 === $status_code ) {
			Logger::debug( 'Response body:' );
			Logger::debug( $request['body'] );

			return $request['body'];
		} else {
			$silent_404 = apply_filters( 'nmf_get_request_response_silent_404', false );

			if ( 404 === $status_code && $silent_404 ) {
				Logger::debug( 'Silent 404' );
			} else {
				Logger::error(
					'START: API Request failed with response code ' . $status_code
				);

				Logger::error( 'Response URL:' );
				Logger::error( $request_url );
				Logger::error( 'Response Params:' );
				Logger::error( $params );
				if ( isset( $request['body'] ) ) {
					Logger::error( 'Body:' );
					Logger::error( $request['body'] );
				}
				Logger::error( 'END: API Request failed.' );
			}
		}

		$retry_limit = 5;

		$retry_codes = array( // Array of request response codes that should trigger retry.
			429 => 'API Rate limit reached',
		);

		if ( ! empty( $retry_codes[ $status_code ] ) ) {
			if ( $retry_count <= $retry_limit ) {
				Logger::error( $retry_codes[ $status_code ] . ', retrying after short pause...' );
				usleep( 1000 * 1000 );
				++$retry_count;

				return $this->get_request_response( $request_url, $params, $retry_count );
			} else {
				Logger::error( 'Request retry limit reached. Aborting request.' );

				return false;
			}
		}

		return false;
	}

	/**
	 * Parses API request body, stripping invalid keys if it is an array.
	 *
	 * @param string|array|null $body     Body of API request.
	 * @param string            $method   HTTP method. Used for filter.
	 * @param string            $endpoint API endpoint. Used for filter.
	 * @param string            $version  API version. Used for filter.
	 * @param array             $headers  Request headers. Used for filter.
	 *
	 * @return string|array|null
	 */
	protected function parse_request_body(
		string|array|null $body,
		string $method,
		string $endpoint,
		string $version,
		array $headers
	): string|array|null {
		if ( ! is_array( $body ) ) { // String or null.
			return $body;
		}

		$valid_keys = apply_filters(
			'nmf_api_request_valid_body_keys',
			array(
				'page',
				'pageSize',
				'updatedSince',
				'modifiedSince',
			),
			$method,
			$endpoint,
			str_replace( '/', '', $version ),
			$headers
		);

		if ( true === $valid_keys ) {
			return $body; // array.
		}

		return filter_var_array(
			$body,
			array_flip( $valid_keys ),
			false
		); // array.
	}
}
