<?php
/**
 * Hub Accounts
 *
 * @package flo-sync
 */

namespace NMFS\Sync\API\Backoffice_Accounts;

use NMFS\Logger;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;
use NMFS\Sync\Sync_Updated_Status;
use Throwable;
use WP_Error;

/**
 * Code for Hub Account Sync.
 *
 * @package flo-sync
 */
class Backoffice_Accounts {

	/**
	 * The prefix for the custom post type meta-fields.
	 *
	 * @var string $cpt_meta_prefix
	 */
	public static string $cpt_meta_prefix = 'nmf_bo_account_';

	/**
	 * Check if functionality for multiple stocks is enabled.
	 *
	 * @param string|null $env Flo API environment. Defaults to environment set in options.
	 *
	 * @return bool
	 */
	public static function is_multiple_stocks_enabled( ?string $env = 'current' ): bool {
		static $is_multiple_stocks_enabled;

		if ( ! isset( $is_multiple_stocks_enabled ) ) {
			$env = Envs::validate_env( $env );

			$is_multiple_stocks_enabled = ! empty( self::get_stock_accounts( $env ) );
		}

		return $is_multiple_stocks_enabled;
	}

	/**
	 * Get the main account.
	 *
	 * @param string $env The environment to get the accounts for. Can be 'prod' or 'test'.
	 *
	 * @return Backoffice_Account_Model|null Returns the main account for the given environment or null if none exists.
	 */
	public static function get_main_account( string $env ): ?Backoffice_Account_Model {
		$env      = Envs::validate_env( $env );
		$accounts = self::get_accounts(
			array(
				'posts_per_page' => 1,
			),
			array(
				'type' => 'main',
				'env'  => $env,
			)
		);

		return ! empty( $accounts ) ? reset( $accounts ) : null;
	}

	/**
	 * Get the current main Hub account.
	 *
	 * @return Backoffice_Account_Model|null Returns the main account for the current environment or null if none exists.
	 */
	public static function get_current_main_account(): ?Backoffice_Account_Model {
		$env = Envs::get_current_env();
		return self::get_main_account( $env );
	}

	/**
	 * Get all stock accounts.
	 *
	 * @param null|string $env The environment to get the accounts for. Can be 'prod' or 'test'. If null, all accounts are returned.
	 * @param bool        $include_disabled Whether to include disabled accounts.
	 *
	 * @return Backoffice_Account_Model[] Array of stock Hub accounts
	 */
	public static function get_stock_accounts( ?string $env = null, bool $include_disabled = false ): array {
		$env = Envs::validate_env( $env );

		return self::get_accounts(
			array(),
			array(
				'type' => 'stock',
				'env'  => $env,
			),
			$include_disabled
		);
	}

	/**
	 * Get all accounts for current environment.
	 *
	 * @return Backoffice_Account_Model[]
	 */
	public static function get_all_current_accounts(): array {
		$accounts = array_merge(
			array(
				self::get_current_main_account(),
			),
			self::get_stock_accounts( 'current' )
		);

		return array_filter( $accounts );
	}

	/**
	 * Get Hub Accounts.
	 *
	 * @return Backoffice_Account_Model[] Array of all Hub accounts
	 */
	public static function get_all(): array {
		return self::get_accounts( array(), array(), true, false );
	}

	/**
	 * Get current stock Hub accounts.
	 *
	 * @param array $get_post_args    Arguments to pass to get_posts().
	 * @param array $custom_args      Custom arguments to pass to get_posts().
	 * @param bool  $include_disabled Include disabled accounts.
	 * @param bool  $only_unique      Only get unique accounts. There should only be unique accounts, but this is a failsafe used to delete duplicates.
	 *
	 * @return Backoffice_Account_Model[] Array of stock Hub accounts
	 */
	private static function get_accounts(
		array $get_post_args = array(),
		array $custom_args = array(),
		bool $include_disabled = false,
		bool $only_unique = true
	): array {

		$get_post_args = array_merge(
			array(
				'post_type'      => 'mf_bo_accounts',
				'posts_per_page' => - 1,
				'post_status'    => 'publish',
				'orderby'        => 'id',
				'order'          => 'ASC',
			),
			$get_post_args
		);

		$custom_args = array_merge(
			array(
				'type' => false,
				'env'  => false,
			),
			$custom_args
		);

		$meta_query = array();

		if ( $custom_args['type'] ) {
			$meta_query[] = array(
				'key'     => self::$cpt_meta_prefix . 'api_type',
				'value'   => $custom_args['type'],
				'compare' => '=',
			);
		}

		if ( $custom_args['env'] ) {
			$meta_query[] = array(
				'key'     => self::$cpt_meta_prefix . 'api_env',
				'value'   => $custom_args['env'],
				'compare' => '=',
			);
		}

		if ( ! empty( $meta_query ) ) {
			$get_post_args['meta_query'] = $meta_query;
		}

		$posts    = get_posts( $get_post_args );
		$accounts = array();

		if ( ! empty( $posts ) ) {
			foreach ( $posts as $post ) {
				$account_model = Factory::create_from_wp_post( $post );
				if ( $include_disabled || $account_model->is_enabled() ) {
					if ( $only_unique ) {
						$accounts[ $account_model->url . $account_model->user ] = $account_model;
					} else {
						$accounts[] = $account_model;
					}
				}
			}
		}

		return $accounts;
	}

	/**
	 * Get a Hub account from a post-ID.
	 *
	 * @param int|string $backoffice_account_post_id Hub account WP post ID.
	 */
	public static function get_account_by_post_id( int|string $backoffice_account_post_id ): ?Backoffice_Account_Model {
		$backoffice_account_post = get_post( (int) $backoffice_account_post_id );
		if ( ! $backoffice_account_post ) {
			return null;
		}

		if ( 'mf_bo_accounts' !== $backoffice_account_post->post_type ) {
			return null;
		}

		return Factory::create_from_wp_post( $backoffice_account_post );
	}

	/**
	 * Create a Hub account from a Hub model, and save it to the database as CPT.
	 *
	 * @param Backoffice_Account_Model $account Hub account to create.
	 *
	 * @return Backoffice_Account_Model|false The created Hub account, or false on failure.
	 */
	public static function create_from_model( Backoffice_Account_Model $account ): false|Backoffice_Account_Model {
		$title = $account->display_name ?? $account->org_name ?? $account->url;

		$post_id = wp_insert_post(
			array(
				'post_title'  => $title,
				'post_type'   => 'mf_bo_accounts',
				'post_status' => 'publish',
			)
		);

		if ( ! $post_id ) {
			return false;
		}

		$account->wp_id = $post_id;

		update_post_meta( $post_id, self::$cpt_meta_prefix . 'display_name', $account->display_name ?? $account->org_name );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'api_url', $account->url );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'api_user', $account->user );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'api_pass', $account->pass );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'api_type', $account->type );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'api_env', $account->env );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'enabled', $account->is_enabled() );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'allowed_shipping_methods', $account->allowed_shipping_methods );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'org_name', $account->org_name );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'org_num', $account->org_num );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'last_sync', $account->last_sync );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'article_account_codes', $account->get_article_account_codes() );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'income_account_codes', $account->get_income_account_codes() );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'vat_account_codes', $account->get_vat_account_codes() );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'shipping_settings', $account->get_shipping_settings() );
		update_post_meta( $post_id, self::$cpt_meta_prefix . 'partial_refund_article_sku', $account->partial_refund_article_sku );

		return $account;
	}

	/**
	 * Update Hub account CPT from a Hub model.
	 *
	 * @param Backoffice_Account_Model $account Hub account to update.
	 *
	 * @return int|WP_Error Post ID on success. The value 0 or WP_Error on failure.
	 */
	public static function update_from_model( Backoffice_Account_Model $account ): WP_Error|int {
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'display_name', $account->display_name ?? $account->org_name );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'api_url', $account->url );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'api_user', $account->user );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'api_pass', $account->pass );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'api_type', $account->type );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'api_env', $account->env );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'enabled', $account->is_enabled() );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'allowed_shipping_methods', $account->allowed_shipping_methods );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'org_name', $account->org_name );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'org_num', $account->org_num );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'last_sync', $account->last_sync );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'article_account_codes', $account->get_article_account_codes() );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'income_account_codes', $account->get_income_account_codes() );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'vat_account_codes', $account->get_vat_account_codes() );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'shipping_settings', $account->get_shipping_settings() );
		update_post_meta( $account->wp_id, self::$cpt_meta_prefix . 'partial_refund_article_sku', $account->partial_refund_article_sku );

		$post = array(
			'ID'         => $account->wp_id,
			'post_title' => $account->display_name ?? $account->org_name,
		);

		return wp_update_post( $post );
	}

	/**
	 * Update specific property on a Hub account.
	 *
	 * @param int    $account_id Hub account WP Post ID.
	 * @param string $property   Property to update.
	 * @param mixed  $value      Value to set.
	 *
	 * @return bool Update status. True on update, false on failure or no change.
	 */
	public static function update_account_property( int $account_id, string $property, mixed $value ): bool {
		if ( ! property_exists( Backoffice_Account_Model::class, $property ) ) {
			Logger::error( 'Trying to update non-existing property ' . $property . ' on Hub account ' . $account_id );
			return false;
		}

		$full_property_name = self::$cpt_meta_prefix . $property;

		try {
			// Try setting the property on a temporary model to see if it's of the correct type.
			$temp_account = new Backoffice_Account_Model( array(), false );

			if ( method_exists( $temp_account, 'set_' . $property ) ) {
				$temp_account->{'set_' . $property}( $value );
			} else {
				$temp_account->{$property} = $value;
			}
		} catch ( Throwable $e ) {
			Logger::error(
				'Trying to set property ' . $property .
					' to value ' . print_r( $value, true ) . // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r
					', which is of incorrect type or not accessible, for Hub account ' . $account_id
			);
			Logger::error( $e->getMessage() );
			return false;
		}

		// TODO: Implement check for non-updated value?

		return (bool) update_post_meta( $account_id, $full_property_name, $value );
	}

	/**
	 * Disconnect/delete Hub account.
	 *
	 * @param Backoffice_Account_Model $account Hub account to disconnect.
	 *
	 * @return void
	 */
	public static function disconnect_account( Backoffice_Account_Model $account ): void {
		// Delete CPT.
		wp_delete_post( $account->wp_id );
	}

	/**
	 * Disconnect duplicate Hub accounts.
	 *
	 * @param Backoffice_Account_Model[]|null $existing_accounts Existing accounts to check for duplicates. Null to fetch all.
	 *
	 * @return Backoffice_Account_Model[] With duplicates removed, keyed by URL and user.
	 */
	private static function disconnect_duplicates( ?array $existing_accounts = null ): array {
		if ( null === $existing_accounts ) {
			$existing_accounts = self::get_all();
		}

		if ( empty( $existing_accounts ) ) {
			return array();
		}

		$unique_accounts = array();

		foreach ( $existing_accounts as $account ) {
			if ( ! isset( $unique_accounts[ $account->url . $account->user ] ) ) {
				$unique_accounts[ $account->url . $account->user ] = $account;
			} else {
				self::disconnect_account( $account );
			}
		}

		return $unique_accounts;
	}

	/**
	 * Update Hub accounts from the plugin settings page.
	 *
	 * @param array $posted_accounts Accounts from settings POST.
	 *
	 * @return void
	 */
	public static function update_from_settings( array $posted_accounts ): void {

		$existing_accounts        = self::get_all();
		$unique_existing_accounts = self::disconnect_duplicates( $existing_accounts );

		$sanitized_accounts = self::sanitize_post_input( $posted_accounts );

		self::clear_invalid_accounts();

		$is_accounts_updated = false;

		foreach ( $sanitized_accounts as $account_key => $account ) {

			// Account exists, update.
			if ( ! empty( $unique_existing_accounts[ $account['url'] . $account['user'] ] ) ) {
				$action           = 'update';
				$existing_account = $unique_existing_accounts[ $account['url'] . $account['user'] ];
				unset( $unique_existing_accounts[ $account['url'] . $account['user'] ] );

				if (
					$existing_account->env !== $account['env'] ||
					$existing_account->type !== $account['type']
				) {
					// Relevant data of the account has been changed, trigger stock sync.
					$is_accounts_updated = true;
				}

				$existing_account->pass                     = $account['pass'];
				$existing_account->env                      = $account['env'];
				$existing_account->type                     = $account['type'];
				$existing_account->display_name             = $account['display_name'];
				$existing_account->enabled                  = $account['enabled'];
				$existing_account->allowed_shipping_methods = $account['allowed_shipping_methods'];

				$sanitized_accounts[ $account_key ] = $existing_account;

			} else { // Create a new account.
				$action = 'create';

				// New account, trigger stock sync.
				$is_accounts_updated = true;

				$sanitized_accounts[ $account_key ] = Factory::create(
					array(
						'url'                      => $account['url'],
						'user'                     => $account['user'],
						'pass'                     => $account['pass'],
						'env'                      => $account['env'],
						'type'                     => $account['type'],
						'display_name'             => $account['display_name'],
						'enabled'                  => $account['enabled'],
						'shipping_type'            => $account['shipping_type'],
						'allowed_shipping_methods' => $account['allowed_shipping_methods'],
					)
				);
			}

			Sync::sync( $sanitized_accounts[ $account_key ], $action );
		}

		if ( ! empty( $existing_account ) ) {
			foreach ( $unique_existing_accounts as $account ) {
				self::disconnect_account( $account );
			}

			// Account has been deleted, trigger stock sync.
			$is_accounts_updated = true;
		}

		if ( $is_accounts_updated ) {
			// Trigger re-sync of all article stocks.
			delete_option( Sync_Updated_Status::get_option_name( 'stock' ) );
		}
	}

	/**
	 * Sanitize POST data from settings page.
	 *
	 * @param array $posted_accounts POST data.
	 *
	 * @return array Sanitized accounts.
	 */
	private static function sanitize_post_input( array $posted_accounts ): array {
		$sanitized_accounts = array();

		$envs = Envs::get_valid_envs();

		foreach ( $posted_accounts as $env => $accounts ) {
			if ( ! isset( $envs[ $env ] ) ) {
				continue;
			}

			if ( ! is_array( $accounts ) ) {
				continue;
			}

			foreach ( $accounts as $key => $account ) {
				if ( empty( $account['api_url'] ) ) {
					continue;
				}

				$type = 'main' === $key ? 'main' : 'stock';

				$api_display_name =
					! empty( $account['api_display_name'] ) ?
					trim( sanitize_text_field( wp_unslash( $account['api_display_name'] ) ) ) :
					'';

				$allowed_shipping_methods = array();

				if ( ! empty( $account['allowed_shipping_methods'] ) ) {
					$allowed_shipping_methods = array_map(
						function ( $method ) {
							return trim( sanitize_text_field( wp_unslash( $method ) ) );
						},
						$account['allowed_shipping_methods']
					);
				}

				$new_account = array(
					'url'                      => trim( sanitize_text_field( wp_unslash( $account['api_url'] ) ) ),
					'user'                     => trim( sanitize_text_field( wp_unslash( $account['api_user'] ) ) ),
					'pass'                     => trim( sanitize_text_field( wp_unslash( $account['api_pass'] ) ) ),
					'display_name'             => $api_display_name,
					'env'                      => $env,
					'type'                     => $type,
					'enabled'                  => 'disabled' !== ( $account['enabled'] ?? 'enabled' ),
					'allowed_shipping_methods' => $allowed_shipping_methods,
				);

				if ( ! str_ends_with( $new_account['url'], '/' ) ) {
					$new_account['url'] .= '/';
				}

				$sanitized_accounts[] = $new_account;
			}
		}
		return $sanitized_accounts;
	}

	/**
	 * Get all accounts stored as invalid after saving settings.
	 *
	 * @return array
	 */
	public static function get_invalid_accounts(): array {
		$invalid_accounts = false;
		try {
			$invalid_accounts = get_option( 'nmf_invalid_bo_accounts', array() );
		} catch ( Throwable $e ) {
			Logger::error( 'Failed to get invalid BO accounts with error: ' . $e->getMessage() );
		}

		if ( is_array( $invalid_accounts ) ) {
			return $invalid_accounts;
		}

		Logger::error( 'Failed to get invalid BO accounts, not array. Resetting.' );
		self::clear_invalid_accounts();

		return array();
	}

	/**
	 * Update invalid accounts.
	 *
	 * @param array $invalid_accounts Invalid accounts.
	 *
	 * @return void
	 */
	public static function update_invalid_accounts( array $invalid_accounts ): void {
		self::clear_invalid_accounts();

		update_option( 'nmf_invalid_bo_accounts', $invalid_accounts );
	}

	/**
	 * Clear invalid accounts.
	 */
	public static function clear_invalid_accounts(): void {
		delete_option( 'nmf_invalid_bo_accounts' );
	}
}
