<?php
/**
 * Handles API environments.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\API\Backoffice_Accounts;

use NMFS\Logger;
use NMFS\Settings;

defined( 'ABSPATH' ) || exit;

/**
 * Class Envs
 */
class Envs {

	/**
	 * Validate passed API environment.
	 *
	 * @param null|string $env Environment to validate. Can be 'current' to get the current environment.
	 *
	 * @return null|string Valid environment. If $env is invalid, the default environment is returned.
	 */
	public static function validate_env( ?string $env ): ?string {
		if ( is_null( $env ) ) {
			return null;
		} elseif ( 'current' === $env ) {
			return self::get_current_env();
		} elseif ( self::is_env_valid( $env ) ) {
				return $env;
		} else {
			Logger::error(
				'Fell back to default environment from invalid environment: ' . $env
			);

			return self::get_default_env();
		}
	}

	/**
	 * Get current environment.
	 *
	 * @return string
	 */
	public static function get_current_env(): string {
		$env = Settings::get( NMF_SYNC_ENV, '' );

		/**
		 * This is a fallback for when the environment was set to 'current' in settings.
		 * It should never happen, but if it does, it would cause a loop.
		 * So we'll just set it to the default environment.
		 */
		if ( 'current' === $env ) {
			$env = self::get_default_env();
		}

		return self::validate_env( $env );
	}

	/**
	 * Get all valid environment.
	 *
	 * @return array Array of valid environments.
	 */
	public static function get_valid_envs(): array {
		return array(
			'prod' => __( 'Production', 'flo-sync' ),
			'test' => __( 'Test', 'flo-sync' ),
		);
	}

	/**
	 * Get key of the default API environment.
	 *
	 * @return string
	 */
	public static function get_default_env(): string {
		return 'test';
	}

	/**
	 * Check if an environment is valid.
	 *
	 * @param string $env The environment to check.
	 *
	 * @return bool
	 */
	public static function is_env_valid( string $env ): bool {
		$envs = self::get_valid_envs();

		return isset( $envs[ $env ] );
	}
}
