<?php
/**
 * Code for Hub Account Model
 *
 * @package flo-sync
 */

namespace NMFS\Sync\API\Backoffice_Accounts\Models;

use NMFS\Logger;
use NMFS\Models\Base_Model;
use NMFS\Sync\API\API;
use NMFS\Sync\API\Main_API;
use NMFS\Sync\API\Stock_API;
use NMFS\Sync\Items\Bookkeeping\Article_Account_Codes\Models\Article_Account_Code_Model;
use NMFS\Sync\Items\Bookkeeping\Income_Account_Codes\Models\Income_Account_Code_Model;
use NMFS\Sync\Items\Bookkeeping\Vat_Account_Codes\Models\Vat_Account_Code_Model;
use NMFS\Sync\Items\Bookkeeping\Other_Account_Codes\Models\Shipping_Model;
use Throwable;

/**
 * Hub model.
 */
class Backoffice_Account_Model extends Base_Model {

	/**
	 * The WordPress ID of the Hub account.
	 *
	 * @var int
	 */
	public int $wp_id;

	/**
	 * Allows disabling of Hub accounts, without deleting them.
	 *
	 * @var bool
	 */
	public bool $enabled;

	/**
	 * The name of the Hub account to be displayed in frontend.
	 *
	 * @var string
	 */
	public string $display_name;

	/**
	 * URL to the Hub account API.
	 *
	 * @var string
	 */
	public string $url;

	/**
	 * Username for the Hub account.
	 *
	 * @var string
	 */
	public string $user;

	/**
	 * Password for the Hub account.
	 *
	 * @var string
	 */
	public string $pass;

	/**
	 * The type of the Hub account. Can be 'main' or 'stock'.
	 *
	 * @var string
	 */
	public string $type;

	/**
	 * The environment of the Hub account. Can be 'prod' or 'test'.
	 *
	 * @var string
	 */
	public string $env;

	/**
	 * Allowed shipping methods for the Hub account.
	 * Strings of "[zone_id]:[method_id]:[method_instance_id]".
	 *
	 * @var string[]
	 */
	public array $allowed_shipping_methods;

	/**
	 * The organization number of the Hub account.
	 *
	 * @var string
	 */
	public string $org_num;

	/**
	 * The organization name of the Hub account.
	 *
	 * @var string
	 */
	public string $org_name;

	/**
	 * The last time the Hub account was synced.
	 *
	 * @var string
	 */
	public string $last_sync;

	/**
	 * Article account codes for the Hub account.
	 *
	 * @var Article_Account_Code_Model[]
	 */
	private array $article_account_codes;

	/**
	 * Article account codes for the Hub account.
	 *
	 * @var Income_Account_Code_Model[]
	 */
	private array $income_account_codes;

	/**
	 * VAT account codes for the Hub account.
	 *
	 * @var Vat_Account_Code_Model[]
	 */
	private array $vat_account_codes;

	/**
	 * Shipping settings.
	 *
	 * @var Shipping_Model
	 */
	private Shipping_Model $shipping_settings;

	/**
	 * The SKU of the article used for partial refunds.
	 *
	 * @var string
	 */
	public string $partial_refund_article_sku;

	/**
	 * The inherited class of API for the Hub account.
	 *
	 * @var API
	 */
	public API $api;

	/**
	 * Constructs the Hub account model.
	 *
	 * @param array $args            Model properties.
	 * @param bool  $instantiate_api Whether to instantiate the API class.
	 */
	public function __construct( array $args = array(), bool $instantiate_api = true ) {
		parent::__construct( $args );

		if ( $instantiate_api ) {
			if ( $this->is_main() ) {
				$this->api = new Main_API( null, $this );
			} elseif ( $this->is_stock() ) {
				$this->api = new Stock_API( $this );
			} else {
				Logger::error( 'Invalid Hub account type: ' . $this->type . 'Could not instantiate API.' );
			}
		}
	}

	/**
	 * Get display name property.
	 *
	 * @return string
	 */
	public function get_display_name(): string {
		return ! empty( $this->display_name ) ? $this->display_name : $this->org_name ?? '';
	}

	/**
	 * Check if the account is enabled.
	 * Main accounts are always enabled.
	 *
	 * @return bool Whether the account is enabled. Defaults to true.
	 */
	public function is_enabled(): bool {
		return $this->is_main() || ( $this->enabled ?? true );
	}

	/**
	 * Check if the account is a test account.
	 *
	 * @return bool
	 */
	public function is_test(): bool {
		return 'test' === $this->env;
	}

	/**
	 * Check if the account is a main account.
	 *
	 * @return bool
	 */
	public function is_main(): bool {
		return 'main' === $this->type;
	}

	/**
	 * Check if the account is a stock account.
	 *
	 * @return bool
	 */
	public function is_stock(): bool {
		return 'stock' === $this->type;
	}

	/**
	 * Get article account codes property.
	 *
	 * @return Article_Account_Code_Model[]
	 */
	public function get_article_account_codes(): array {
		return $this->article_account_codes ?? array();
	}

	/**
	 * Set article account codes property.
	 *
	 * @param mixed $article_account_codes Article account codes.
	 */
	public function set_article_account_codes( mixed $article_account_codes ): void {
		$this->article_account_codes = $this->get_account_codes_value(
			Article_Account_Code_Model::class,
			$article_account_codes
		);
	}

	/**
	 * Get income account codes property.
	 *
	 * @return Income_Account_Code_Model[]
	 */
	public function get_income_account_codes(): array {
		return $this->income_account_codes ?? array();
	}

	/**
	 * Set income account codes property.
	 *
	 * @param mixed $income_account_codes Article account codes.
	 */
	public function set_income_account_codes( mixed $income_account_codes ): void {
		$this->income_account_codes = $this->get_account_codes_value(
			Income_Account_Code_Model::class,
			$income_account_codes
		);
	}

	/**
	 * Get vat account codes property.
	 *
	 * @return Vat_Account_Code_Model[]
	 */
	public function get_vat_account_codes(): array {
		return $this->vat_account_codes ?? array();
	}

	/**
	 * Set vat account codes property.
	 *
	 * @param mixed $vat_account_codes Article account codes.
	 */
	public function set_vat_account_codes( mixed $vat_account_codes ): void {
		$this->vat_account_codes = $this->get_account_codes_value(
			Vat_Account_Code_Model::class,
			$vat_account_codes
		);
	}

	/**
	 * Get validated article account codes value of the specified type.
	 *
	 * @param string $account_code_model_name Name of the account code model class.
	 * @param mixed  $value                   Value to validate.
	 *
	 * @return array
	 */
	private function get_account_codes_value( string $account_code_model_name, mixed $value ): array {
		if ( is_array( $value ) ) {
			foreach ( $value as $key => $article_account_code ) {
				if ( ! is_a( $article_account_code, $account_code_model_name ) ) {
					unset( $value[ $key ] );
				}
			}
		} else {
			$value = array();
		}
		return $value;
	}

	/**
	 * Get shipping settings property.
	 *
	 * @return Shipping_Model
	 */
	public function get_shipping_settings(): Shipping_Model {
		return $this->shipping_settings ?? new Shipping_Model();
	}

	/**
	 * Set shipping settings property.
	 *
	 * @param mixed $shipping_settings Shipping settings.
	 *
	 * @return void
	 */
	public function set_shipping_settings( mixed $shipping_settings ): void {
		if ( is_array( $shipping_settings ) ) {
			try {
				$shipping_settings = new Shipping_Model( $shipping_settings );
			} catch ( Throwable $e ) {
				Logger::error( $e->getMessage() );
			}
		}

		if ( is_a( $shipping_settings, Shipping_Model::class ) ) {
			$this->shipping_settings = $shipping_settings;
		}
	}
}
