<?php
/**
 * API requests for the main API account (rather than stock accounts).
 *
 * @package flo-sync
 */

namespace NMFS\Sync\API;

use NMFS\Logger;
use NMFS\Settings;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;

/**
 * Class Flo_Main_API
 */
class Main_API extends API {

	/**
	 * Set up API.
	 *
	 * @param string|null                   $env     Flo API environment. Defaults to environment set in options.
	 *                                               Ignored if $account is set.
	 * @param Backoffice_Account_Model|null $account Hub account to use. Defaults to the main account in env.
	 */
	public function __construct( ?string $env = null, ?Backoffice_Account_Model $account = null ) {
		if ( ! $account ) {
			$account = $env ?
				Backoffice_Accounts::get_main_account( $env ) :
				Backoffice_Accounts::get_current_main_account();
		}

		if ( ! $account ) {
			Logger::error( 'Could not find main account in env: ' . $env );

			return;
		} elseif ( ! $account->is_main() ) {
			Logger::error( 'Account is not main account in env: ' . $env );

			return;
		}

		parent::__construct( $account );
	}

	/**
	 * Get article sets.
	 *
	 * @param array $args Query parameters.
	 *
	 * @return array|null    Decoded JSON array of article sets, or null if request failed.
	 */
	public function get_article_sets( array $args ): ?array {
		return $this->request_array( 'GET', 'articlesets', $args );
	}

	/**
	 * Update article in API
	 *
	 * @param array $article Article data.
	 *
	 * @return false|string
	 */
	public function put_articles( array $article ): false|string {

		if ( ! Settings::is_setting_enabled( NMF_IS_PRICE_ACTIVE ) ) {
			Logger::debug( 'Price sync disabled.' );
			unset( $article[ NMF_RECOMMENDED_PRICE ] );
			unset( $article[ NMF_RETAIL_PRICE ] );
		}

		if ( isset( $article[ NMF_QUANTITY ] ) ) {
			unset( $article[ NMF_QUANTITY ] );
		}

		$body = wp_json_encode( array( $article ) );
		$body = $body ? $body : '';

		$response = $this->request( 'PUT', NMF_ARTICLES_ENDPOINT, $body );

		if ( ! $response ) {
			return false;
		}

		return $this->is_edited_article_successful( $response ) ? $response : false;
	}

	/**
	 * Create article in API
	 *
	 * @param array $article Article data.
	 *
	 * @return false|string
	 */
	public function post_articles( array $article ): false|string {

		$body = wp_json_encode( array( $article ) );
		$body = $body ? $body : '';

		$response = $this->request( 'POST', NMF_ARTICLES_ENDPOINT, $body );

		if ( ! $response ) {
			return false;
		}

		return $this->is_edited_article_successful( $response ) ? $response : false;
	}

	/**
	 * Validate API response for created/updated article.
	 *
	 * @param string $response API response.
	 *
	 * @return bool
	 */
	private function is_edited_article_successful( string $response ): bool {
		$response_object = json_decode( $response, true );

		if ( ! isset( $response_object['list'] ) || ! is_array( $response_object['list'] ) ) {
			return false;
		}

		foreach ( $response_object['list'] as $response ) {
			if ( empty( $response['Success'] ) ) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Get article groups.
	 *
	 * @param array $args Query parameters.
	 *
	 * @return array|null    Decoded JSON array of groups, or null if request failed.
	 */
	public function get_groups( array $args ): ?array {
		return $this->request_array( 'GET', 'groups', $args );
	}

	/**
	 * Get articles.
	 *
	 * @param array $args Query parameters.
	 *
	 * @return array|null    Decoded JSON array of articles, or null if request failed.
	 */
	public function get_campaigns( array $args ): ?array {
		return $this->request_array( 'GET', 'campaigns', $args );
	}

	/**
	 * Get image file from API.
	 *
	 * @param string $image_url Image URL.
	 * @param string $image_type Image type.
	 *
	 * @return false|string
	 */
	public function get_image_file( string $image_url, string $image_type ): false|string {
		if ( empty( $this->user ) || empty( $this->pass ) ) {
			Logger::error( 'No valid API credentials' );

			return false;
		}

		$context = stream_context_create(
			array(
				'http' => array(
					// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_encode
					'header'       => 'Authorization: Basic ' . base64_encode( "$this->user:$this->pass" ),
					'Content-Type' => $image_type,
				),
			)
		);

		// phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents
		return file_get_contents( $image_url, false, $context );
	}
}
