<?php
/**
 * Abstract syncer class.
 *
 * @package flo-sync
 */

namespace NMFS\Sync;

use NMFS\Logger;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;

defined( 'ABSPATH' ) || exit;

/**
 * Abstract syncer class.
 */
abstract class Abstract_Syncer {
	/**
	 * Syncer type.
	 * Can be 'cron' or 'ajax'.
	 *
	 * @var string $type
	 */
	private string $type;

	/**
	 * Number of items to get in each API request.
	 * Used only for cron syncs. This number for ajax syncs is set in settings.
	 *
	 * @var int
	 */
	protected int $page_size = 100;

	/**
	 * If this is set to true, the sync page size will be forced to the value set for the class.
	 * This is useful for syncers that need to get all items in one request.
	 * For instance, the value set in settings for ajax syncs will be ignored.
	 *
	 * @var bool
	 */
	protected bool $force_page_size = false;

	/**
	 * Hub accounts to sync.
	 *
	 * @var Backoffice_Account_Model[]
	 */
	private array $backoffice_accounts;

	/**
	 * Hub account in the current iteration of sync loop.
	 *
	 * @var Backoffice_Account_Model
	 */
	protected Backoffice_Account_Model $current_backoffice_account;

	/**
	 * Array of sync statuses.
	 *
	 * @var array|int[] $status
	 */
	protected array $status = array(
		'add'    => 0,
		'new'    => 0,
		'update' => 0,
		'delete' => 0,
		'error'  => 0,
		'ignore' => 0,
		'page'   => 0,
		'total'  => 0,
	);

	/**
	 * Array of parameters sent with API request.
	 *
	 * @var array $api_params
	 */
	protected array $api_params;

	/**
	 * Constructs syncer. If no Hub accounts are given, the current main account is used.
	 *
	 * @param Backoffice_Account_Model[] $backoffice_accounts Hub accounts to sync.
	 * @param string                     $type Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function __construct( array $backoffice_accounts = array(), string $type = 'cron' ) {
		$this->type = $type;

		if ( empty( $backoffice_accounts ) ) {
			$backoffice_accounts[] = Backoffice_Accounts::get_current_main_account();
		}

		$backoffice_accounts = array_filter( $backoffice_accounts );

		$this->backoffice_accounts = $backoffice_accounts;
		$this->set_current_backoffice_account( reset( $backoffice_accounts ) );
	}

	/**
	 * Syncs objects from Apps to WooCommerce.
	 *
	 * @param array|null $args Arguments to pass to API, or null if no arguments.
	 *
	 *                                @key int      'page_number'      Starting at 0.
	 *                                @key int      'page_size'        Must be greater than 0.
	 *                                @key string   'updated_since'    DateTime string if it should only do partial update.
	 *                                @key callable 'custom_callback'  Custom callback for handling request response.
	 *
	 * @return bool Whether sync was successful.
	 */
	abstract public function sync( ?array $args = null ): bool;

	/**
	 * Get syncer Hub accounts.
	 *
	 * @return Backoffice_Account_Model[]
	 */
	public function get_backoffice_accounts(): array {
		return $this->backoffice_accounts;
	}

	/**
	 * Get the syncer type. Defaults to 'cron'.
	 *
	 * @return string
	 */
	public function get_type(): string {
		return $this->type ?? 'cron';
	}

	/**
	 * Set syncer type.
	 *
	 * @param string $type Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function set_type( string $type ): void {
		$allowed_types = array( 'cron', 'ajax' );

		if ( ! in_array( $type, $allowed_types, true ) ) {
			Logger::error( 'Invalid syncer type: ' . $type );
			wp_die( esc_html( 'Invalid syncer type: ' . $type ) );
		}

		$this->type = $type;
	}

	/**
	 * Check if the current sync is of type 'cron'.
	 *
	 * @return bool
	 */
	public function is_cron(): bool {
		return 'cron' === $this->get_type();
	}

	/**
	 * Check if the current sync is of type 'ajax'.
	 *
	 * @return bool
	 */
	public function is_ajax(): bool {
		return 'ajax' === $this->get_type();
	}

	/**
	 * Get API request page size.
	 *
	 * @return int
	 */
	public function get_page_size(): int {
		return $this->page_size;
	}

	/**
	 * Set API request page size.
	 *
	 * @param int $page_size Page size.
	 *
	 * @return int New page size.
	 */
	public function set_page_size( int $page_size ): int {
		if ( ! $this->force_page_size ) {
			$this->page_size = $page_size;
		}
		return $this->page_size;
	}

	/**
	 * Check if syncer is active.
	 *
	 * @return bool
	 */
	public function is_enabled(): bool {
		return true;
	}

	/**
	 * Get syncer status.
	 *
	 * @param string|null $key Key to get status for. If null, the entire status array is returned.
	 */
	public function get_status( ?string $key = null ) {
		if ( $key ) {
			return $this->status[ $key ] ?? null;
		} else {
			return $this->status;
		}
	}

	/**
	 * Update syncer status.
	 *
	 * @param string     $key    Key to set status for. If null, $status is the entire status array.
	 * @param int|string $status Value to set. If $key is set, this is the value to set for that key. Otherwise, it is the entire status array.
	 *
	 * @return array Updated status array.
	 */
	public function update_status( string $key, int|string $status ): array {
		if ( ! is_numeric( $status ) ) {
			$status = 0;
		}
		$this->status[ $key ] = (int) $status;
		return $this->status;
	}

	/**
	 * Set syncer status.
	 *
	 * @param array $status Status array to replace current status with.
	 *
	 * @return array|int[]
	 */
	public function set_status( array $status ): array {
		$this->status = $status;
		return $this->status;
	}

	/**
	 * Increase syncer status.
	 *
	 * @param string $key   Key to increase status for.
	 * @param int    $value Value to increase status by.
	 *
	 * @return void
	 */
	public function increase_status( string $key, int $value = 1 ): void {
		$this->status[ $key ] += $value;
	}

	/**
	 * Sets and returns skip status
	 */
	public function set_skip_status(): array {
		$this->status = array(
			'total'   => 0,
			'skipped' => 1,
		);
		return $this->get_status();
	}

	/**
	 * Sets and returns skip status
	 */
	public function set_cleaned_status(): array {
		$this->status = array(
			'total'   => 0,
			'cleaned' => 1,
		);
		return $this->get_status();
	}

	/**
	 * Loop for sync.
	 *
	 * @param array|null $params       API params.
	 */
	public function sync_loop( ?array $params ): bool {
		$this->before_sync_loop( $params );
		$synced = $this->sync_loop_inner( $params );
		$this->after_sync_loop();

		return $synced;
	}

	/**
	 * Function to run before sync loop.
	 *
	 * @param array|null $args API params passed by reference.
	 */
	public function before_sync_loop( ?array &$args ) {}

	/**
	 * Loop for sync.
	 *
	 * @param ?array $params API params.
	 *
	 * @return bool Whether all syncs were successful.
	 */
	public function sync_loop_inner( ?array $params = array() ): bool {

		foreach ( $this->backoffice_accounts as $backoffice_account ) {
			$this->set_current_backoffice_account( $backoffice_account );

			$this->parse_api_params( $params );
			$synced = $this->sync( $this->api_params );
			if ( ! $synced ) {
				Logger::error( 'Sync error in ' . get_class( $this ) );
				return false;
			}

			$status = $this->get_status();

			if ( ! empty( $status['total'] ) && $status['total'] > $this->api_params['pageSize'] ) {
				$this->api_params['page'] = 1;

				$total_pages = ceil( $status['total'] / $this->api_params['pageSize'] );
				$this->update_status( 'total_pages', $total_pages );

				while ( $this->api_params['page'] < $total_pages ) {
					$synced = $this->sync( $this->api_params );
					if ( ! $synced ) {
						Logger::error( 'Sync error in ' . get_class( $this ) );
						return false;
					}

					++$this->api_params['page'];
				}
			}
		}

		return true;
	}

	/**
	 * Parses API params, adding default values if not present.
	 *
	 * @param null|array $params API params.
	 *
	 * @return array
	 */
	public function parse_api_params( ?array $params = array() ): array {

		if ( empty( $params['page'] ) ) {
			if ( ! empty( $params['page_number'] ) ) {
				$params['page'] = (int) $params['page_number'];
			} else {
				$params['page'] = 0;
			}
		}

		if ( ! empty( $params['updated_since'] ) ) {
			$params['UpdatedSince'] = $params['updated_since'];
		} elseif ( ! empty( $params['last_updated_at'] ) ) {
			$params['UpdatedSince'] = $params['last_updated_at'];
		}

		if ( ! empty( $params['modified_since'] ) ) {
			$params['modifiedSince'] = $params['modified_since'];
		}

		$params['pageSize'] = $this->page_size;

		$this->api_params = $params;

		return $params;
	}

	/**
	 * Function to run after sync loop.
	 * This will be run even if the sync loop is interrupted.
	 *
	 * @return void
	 */
	public function after_sync_loop() {}

	/**
	 * Gets the current Hub account.
	 *
	 * @return null|Backoffice_Account_Model
	 */
	public function get_current_backoffice_account(): ?Backoffice_Account_Model {
		if ( isset( $this->current_backoffice_account ) ) {
			return $this->current_backoffice_account;
		} else {
			Logger::error( 'No Hub account set for syncer.' );
			return null;
		}
	}

	/**
	 * Sets the current Hub account.
	 *
	 * @param mixed $current_backoffice_account Hub account to sync.
	 */
	public function set_current_backoffice_account( mixed $current_backoffice_account ): void {
		if ( is_a( $current_backoffice_account, Backoffice_Account_Model::class ) ) {
			$this->current_backoffice_account = $current_backoffice_account;
		} else {
			Logger::error( 'Invalid Hub account given to syncer:' );
			Logger::error( $current_backoffice_account );
		}
	}

	/**
	 * Get the name of option storing last updated time.
	 *
	 * @return string
	 */
	public function get_last_updated_option_name(): string {
		return Sync_Updated_Status::get_option_name_by_syncer_class( get_class( $this ) );
	}

	/**
	 * Get the name of API parameter used when getting only data updated since the last request to API.
	 *
	 * @return string
	 */
	public function get_last_updated_api_parameter(): string {
		return Sync_Updated_Status::get_api_parameter_by_syncer_class( get_class( $this ) );
	}
}
