<?php
/**
 * Ajax request.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Ajax\Models;

use NMFS\Logger;
use NMFS\Models\Base_Model;

/**
 * Class Request
 */
class Request_Model extends Base_Model {

	/**
	 * Ajax request action (without wp_ajax_ prefix).
	 *
	 * @var string
	 */
	protected string $action;

	/**
	 * Requested function (usually a sync), callable.
	 * Has to be publicly accessible.
	 *
	 * @var array
	 */
	protected array $request_function;

	/**
	 * Is the request public? Available when not logged in.
	 *
	 * @var bool $is_public
	 */
	protected bool $is_public = false;

	/**
	 * Request constructor.
	 *
	 * @param array $args Request arguments.
	 */
	public function __construct( array $args = array() ) {
		parent::__construct( $args );

		add_action(
			'wp_ajax_' . $this->action,
			array( $this, 'ajax_request' )
		);

		if ( $this->is_public ) {
			add_action(
				'wp_ajax_nopriv_' . $this->action,
				array( $this, 'ajax_request' )
			);
		}
	}

	/**
	 * Ajax request.
	 *
	 * @return void
	 */
	public function ajax_request(): void {
		$this->verify_request( $params );
		$status = call_user_func( $this->request_function, $params );
		$this->respond( $status );
	}

	/**
	 * Verify ajax request validity.
	 *
	 * @param array|null $params         API request params.
	 */
	protected function verify_request( ?array &$params = null ): void {
		$this->verify_nonce();
		$this->get_post_data( $params );
	}

	/**
	 * Verify request nonce.
	 */
	private function verify_nonce(): void {
		if ( ! check_ajax_referer( 'mf_ajax_sync_nonce', 'security' ) ) {
			exit( 'Nonce failed' );
		}
	}

	/**
	 * Verify request page data.
	 *
	 * @param null|array $params API params.
	 */
	protected function get_post_data( ?array &$params ): void {}

	/**
	 * Respond to ajax request.
	 *
	 * @param mixed $response Response data.
	 *
	 * @return void
	 */
	protected function respond( mixed $response ): void {
		if (
			! empty( $_SERVER['HTTP_X_REQUESTED_WITH'] ) &&
			strtolower( $_SERVER['HTTP_X_REQUESTED_WITH'] ) === 'xmlhttprequest'
		) {
			Logger::debug( 'Ajax response' );
			Logger::debug( $response );
			wp_send_json_success( $response );
		} else {
			Logger::debug( 'Not an ajax request' );
			wp_send_json_error( 'fail' );
		}
	}
}
