<?php
/**
 * Ajax sync request model.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Ajax\Models;

use NMFS\Helpers;
use NMFS\Logger;
use NMFS\Sync\Abstract_Syncer;
use NMFS\Sync\Abstract_Syncer_And_Cleaner;
use NMFS\Sync\Sync_Updated_Status;
use Throwable;

defined( 'ABSPATH' ) || exit;

/**
 * Class Sync_Request_Model
 */
class Sync_Request_Model extends Request_Model {

	/**
	 * Syncer class name.
	 *
	 * @var string $syncer_class_name
	 */
	protected string $syncer_class_name;

	/**
	 * Syncer instance.
	 *
	 * @var Abstract_Syncer $syncer
	 */
	protected Abstract_Syncer $syncer;

	/**
	 * Contains status whether this is a request for cleanup rather than sync.
	 *
	 * @var bool $is_cleaner
	 */
	protected bool $is_cleaner = false;

	/**
	 * Number of Hub account to sync.
	 * 0 for first, 1 for second, etc.
	 *
	 * @var int $account
	 */
	private int $account;

	/**
	 * Page number.
	 *
	 * @var int $page
	 */
	private int $page;

	/**
	 * Page size (number of items to get in each API request).
	 *
	 * @var int $page_size
	 */
	private int $page_size;

	/**
	 * Get all data or just data that has been updated since last sync.
	 *
	 * @var bool $get_all_data
	 */
	private bool $get_all_data = true;

	/**
	 * Total number of Hub accounts.
	 *
	 * @var int $total_accounts
	 */
	private int $total_accounts;

	/**
	 * Name of the option to store last sync time.
	 *
	 * @var string $last_updated_option_name
	 */
	private string $last_updated_option_name;

	/**
	 * Sync_Request_Model constructor.
	 *
	 * @param array $args Arguments.
	 */
	public function __construct( array $args = array() ) {
		parent::__construct( $args );

		if ( empty( $this->syncer_class_name ) ) {
			exit( 'Syncer not supplied.' );
		}
	}
	/**
	 * Ajax request handler function.
	 *
	 * @return void
	 */
	public function ajax_request(): void {
		$this->verify_request( $params );

		$this->load_syncer();
		$this->check_if_sync_is_enabled();

		$this->maybe_run_cleaner();

		$this->maybe_update_updated_time_transient();
		$this->sync_inner_loop();
		$this->maybe_update_sync_updated_time();

		$this->respond( $this->syncer->get_status() );
	}

	/**
	 * Verify request page data.
	 *
	 * @param null|array $params API params.
	 *
	 * @phpcs:disable WordPress.Security.NonceVerification.Missing
	 */
	protected function get_post_data( ?array &$params ): void {
		$this->page      = ! empty( $_POST['page'] ) ? intval( $_POST['page'] ) : 0;
		$this->account   = ! empty( $_POST['account'] ) ? intval( $_POST['account'] ) : 0;
		$this->page_size = ! empty( $_POST['page_size'] ) ? intval( $_POST['page_size'] ) : 20;

		$this->get_all_data = ! empty( $_POST['get_all_data'] ) && 'false' !== $_POST['get_all_data'];
	}
	// phpcs:enable

	/**
	 * Load syncer.
	 *
	 * @return void
	 */
	private function load_syncer(): void {
		try {
			$this->syncer = new $this->syncer_class_name( array(), 'ajax' );
		} catch ( Throwable $e ) {
			Logger::error( $e->getMessage() );
			wp_send_json_error( 'Could not load syncer' );
		}

		$this->syncer->set_page_size( $this->page_size );

		$backoffice_accounts = $this->syncer->get_backoffice_accounts();
		$backoffice_account  = $backoffice_accounts[ array_keys( $backoffice_accounts )[ $this->account ] ];
		$this->syncer->set_current_backoffice_account( $backoffice_account );

		$this->total_accounts = count( $backoffice_accounts );

		$this->last_updated_option_name = $this->syncer->get_last_updated_option_name();
	}

	/**
	 * Check if specific sync is enabled.
	 */
	private function check_if_sync_is_enabled(): void {
		if ( $this->syncer->is_enabled() ) {
			return; // Resume sync.
		} else {
			$this->respond( $this->syncer->set_skip_status() ); // Skip sync.
		}
	}

	/**
	 * Maybe run cleaner instead of syncer
	 */
	private function maybe_run_cleaner(): void {
		if ( $this->is_cleaner ) {
			if ( is_a( $this->syncer, Abstract_Syncer_And_Cleaner::class ) ) {
				$this->syncer->clean();
				$this->respond( $this->syncer->set_cleaned_status() );
			} else {
				exit( 'Cleaner not available for this syncer' );
			}
		}
	}

	/**
	 * Maybe update updated time transient. For use when sync is finished.
	 *
	 * @return void
	 */
	private function maybe_update_updated_time_transient(): void {
		if (
			0 === $this->account &&
			0 === $this->page
		) {
			set_transient(
				'ajax_' . $this->last_updated_option_name,
				Helpers::get_current_time()
			);
		}
	}

	/**
	 * Sync inner loop. Will be called once for each ajax request.
	 */
	private function sync_inner_loop(): void {
		$args = array(
			'page' => $this->page,
			// Page size is added in syncer.
		);

		if ( ! $this->get_all_data ) {
			$last_update_option_name   = $this->syncer->get_last_updated_option_name();
			$last_update_api_parameter = $this->syncer->get_last_updated_api_parameter();

			if ( ! empty( $last_update_api_parameter ) ) {
				$args[ $last_update_api_parameter ] = Sync_Updated_Status::get_wp_option( $last_update_option_name );
				if ( empty( $args[ $last_update_api_parameter ] ) ) {
					unset( $args[ $last_update_api_parameter ] );
				}
			}
		}

		$this->syncer->before_sync_loop( $args );
		$args   = $this->syncer->parse_api_params( $args );
		$synced = $this->syncer->sync( $args );
		$this->syncer->after_sync_loop();

		if ( ! $synced ) {
			Logger::error( 'Sync error in ' . get_class( $this->syncer ) );
			wp_send_json_error( 'Sync failed' );
		}

		$status = $this->syncer->get_status();
		$this->syncer->update_status( 'total_accounts', $this->total_accounts );
		$this->syncer->update_status( 'account', $this->account );
		$this->syncer->update_status(
			'total_pages',
			$status['total'] > 1 ?
				ceil( $status['total'] / $this->syncer->get_page_size() ) :
				1
		);
	}

	/**
	 * Maybe update last updated time. For use when sync is finished.
	 *
	 * @return void
	 */
	private function maybe_update_sync_updated_time(): void {
		$status     = $this->syncer->get_status();
		$start_time = get_transient( 'ajax_' . $this->last_updated_option_name );
		if (
			(int) $status['total_accounts'] === $this->account + 1 &&
			(int) $status['total_pages'] === $this->page + 1 &&
			$start_time
		) {
			Sync_Updated_Status::set_wp_option( $this->last_updated_option_name, $start_time, 'ajax' );
			delete_transient( 'ajax_' . $this->last_updated_option_name );
		}
	}
}
