<?php
/**
 * Setup cron for sync.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Cron;

use NMFS\Settings;
use NMFS\Sync\Cron\Models\Cron_Event;

/**
 * Cron class.
 */
class Cron {

	/**
	 * All constructed cron events.
	 *
	 * @var Cron_Event[]
	 */
	private static array $cron_events = array();

	/**
	 * Setup cron jobs.
	 */
	public static function setup(): void {
		// phpcs:ignore WordPress.WP.CronInterval.CronSchedulesInterval
		add_filter( 'cron_schedules', array( __CLASS__, 'add_custom_sync_times' ) );

		self::$cron_events['sync'] = new Cron_Event(
			'nmf_cron_sync',
			Settings::get( NMF_CRON_SYNC_EVENT_INTERVAL, 'everyfifteenminutes' ),
			Settings::is_setting_enabled( NMF_SYNC_STATUS )
		);

		self::$cron_events['sync_checker'] = new Cron_Event(
			'nmf_cron_sync_checker',
			'everyfifteenminutes',
			true,
			array(
				array( __CLASS__, 'check_event_statuses' ),
			)
		);
	}

	/**
	 * Retrieve all cron events.
	 *
	 * @return Cron_Event[] An array of cron events.
	 */
	public static function get_all_cron_events(): array {
		return self::$cron_events;
	}

	/**
	 * Get cron event by name.
	 *
	 * @param string $name Cron event name.
	 *
	 * @return Cron_Event|null
	 */
	public static function get_cron_event( string $name ): ?Cron_Event {
		return self::$cron_events[ $name ] ?? null;
	}

	/**
	 * Filter cron schedules.
	 *
	 * @param array $new_schedules Previous cron schedules.
	 *
	 * @return array Modified cron schedules.
	 */
	public static function add_custom_sync_times( array $new_schedules ): array {
		$new_schedules['everyminute'] = array(
			'interval' => MINUTE_IN_SECONDS,
			'display'  => __( 'Every minute (Northmill Sync)', 'flo-sync' ),
		);

		$new_schedules['everyfiveminutes'] = array(
			'interval' => MINUTE_IN_SECONDS * 5,
			'display'  => __( 'Every 5 minutes (Northmill Sync)', 'flo-sync' ),
		);

		$new_schedules['everyfifteenminutes'] = array(
			'interval' => MINUTE_IN_SECONDS * 15,
			'display'  => __( 'Every 15 minutes (Northmill Sync)', 'flo-sync' ),
		);

		return $new_schedules;
	}

	/**
	 * Check if all cron events are running, and try to fix if something is wrong.
	 */
	public static function check_event_statuses(): void {
		foreach ( self::$cron_events as $event ) {
			$event->check_status();
		}
	}

	/**
	 * Clears all scheduled cron events.
	 *
	 * @return void
	 */
	public static function clear_all(): void {
		foreach ( self::$cron_events as $event ) {
			$event->clear();
		}
	}
}
