<?php
/**
 * Sync cron functions.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Cron\Models;

use NMFS\Helpers;
use NMFS\Logger;
use NMFS\Models\Base_Model;
use NMFS\Sync\Abstract_Syncer;
use NMFS\Sync\Cron\Cron;
use NMFS\Sync\Sync_Updated_Status;

defined( 'ABSPATH' ) || exit;

/**
 * Sync cron class.
 */
class Cron_Sync_Model extends Base_Model {
	/**
	 * Lock object.
	 *
	 * @var Lock_Model
	 */
	protected Lock_Model $lock;

	/**
	 * Name of inheriting class.
	 * Used for lock name.
	 *
	 * @var string $cron_name
	 */
	protected string $cron_name;

	/**
	 * Event for this cron sync.
	 *
	 * @var Cron_Event
	 */
	protected Cron_Event $cron_event;

	/**
	 * Cron action priority.
	 *
	 * @var int
	 */
	protected int $cron_priority = 10;

	/**
	 * Subclass of NMFS\Sync\Abstract_Syncer
	 *
	 * @var Abstract_Syncer $syncer
	 */
	protected Abstract_Syncer $syncer;

	/**
	 * Array of parameters to send with API request.
	 *
	 * @var array $api_params
	 */
	protected array $api_params = array();

	/**
	 * Constructor.
	 *
	 * @param array $args Arguments.
	 */
	public function __construct( array $args = array() ) {
		parent::__construct( $args );
		$this->cron_name = get_class( $this->syncer ) . '_cron';

		$this->cron_event = Cron::get_cron_event( 'sync' );

		$this->cron_event->add_function(
			array( $this, 'cron_sync' ),
			$this->cron_priority
		);
	}

	/**
	 * Loop sync function for cron sync.
	 *
	 * @return void
	 */
	public function cron_sync(): void {
		if ( ! $this->syncer->is_enabled() ) {
			Logger::notice( "Cron Skipped - $this->cron_name" );
			return;
		}

		// Logging occurs inside if cron is locked.
		if ( ! $this->is_unlocked() ) {
			return;
		}

		Logger::notice( "Cron Started - $this->cron_name" );

		$sync_start_time = Helpers::get_current_time();

		$last_update_option_name   = $this->syncer->get_last_updated_option_name();
		$last_update_api_parameter = $this->syncer->get_last_updated_api_parameter();

		if ( ! empty( $last_update_api_parameter ) ) {
			$this->api_params[ $last_update_api_parameter ] = Sync_Updated_Status::get_wp_option( $last_update_option_name );
		}

		$all_syncs_successful = $this->syncer->sync_loop( $this->api_params );

		if ( $all_syncs_successful ) {
			Sync_Updated_Status::set_wp_option( $last_update_option_name, $sync_start_time, 'cron' );

			if ( method_exists( $this->syncer, 'clean' ) ) {
				Logger::notice( "Cron Started - Clean $this->cron_name" );

				$this->syncer->clean();

				Logger::notice( "Cron Ended - Clean $this->cron_name" );
			}
		} else {
			Logger::error( "Cron Error - $this->cron_name - All Syncs were not successful" );
		}

		$this->clear_lock();
		Logger::notice( "Cron Ended - $this->cron_name" );
	}

	/**
	 * Check if there is a lock for the current function.
	 *
	 * @return bool
	 */
	protected function is_unlocked(): bool {
		if ( empty( $this->cron_name ) ) {
			Logger::error( 'Lock name is not set' );

			return false;
		}

		$this->lock = new Lock_Model( $this->cron_name );
		if ( ! $this->lock->lock() ) {
			Logger::warning( "Cron lock in place for $this->cron_name, skipping cron" );

			return false;
		}

		return true;
	}

	/**
	 * Clear the current lock if set.
	 */
	protected function clear_lock(): void {
		if ( isset( $this->lock ) && is_a( $this->lock, Lock_Model::class ) ) {
			$this->lock->clear_lock();
		}
	}
}
