<?php
/**
 * Main code for article sync.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Articles;

use NMFS\Settings;
use NMFS\Sync\API\API;
use NMFS\Sync\API\Main_API;
use NMFS\Sync\Admin_Status;
use NMFS\Sync\Items\Articles\Models\Article_Model;
use NMFS\Logger;
use WC_Product;
use WC_Product_Variable;

/**
 * Class for article sync.
 */
class Articles {

	/**
	 * Holds the status of whether the save article hooks are enabled.
	 *
	 * @var bool $save_article_hook_enabled
	 */
	public static bool $save_article_hook_enabled;

	/**
	 * Construct class.
	 */
	public static function setup(): void {
		add_filter(
			'nmf_api_request_valid_body_keys',
			array( self::class, 'api_request_valid_body_keys' ),
			10,
			4
		);

		self::set_hook_save_article( true );
	}

	/**
	 * Add valid keys to API request.
	 *
	 * @param array  $valid_keys Array of valid keys.
	 * @param string $method    GET, POST, PUT, DELETE.
	 * @param string $endpoint API Endpoint.
	 * @param string $version  API request version.
	 *
	 * @return array
	 */
	public static function api_request_valid_body_keys(
		array $valid_keys,
		string $method,
		string $endpoint,
		string $version
	): array {
		if (
			'GET' === $method &&
			NMF_ARTICLES_ENDPOINT === $endpoint &&
			'v2' === $version
		) {
			$valid_keys[] = 'QuantityUpdatedSince';
			$valid_keys[] = 'ArticleInfoUpdatedSince';
		}
		return $valid_keys;
	}

	/**
	 * Adds or removes the hook for saving articles.
	 *
	 * @param bool $add_hook Whether to add or remove the hooks.
	 */
	public static function set_hook_save_article( bool $add_hook ): void {
		if (
			$add_hook &&
			! Settings::is_setting_enabled( 'sync_only_product_stock_amount' )
		) {
			add_action( 'woocommerce_new_product', array( self::class, 'save_product' ), 10, 2 );
			add_action( 'woocommerce_update_product', array( self::class, 'save_product' ), 10, 2 );
		} else {
			remove_action( 'woocommerce_new_product', array( self::class, 'save_product' ) );
			remove_action( 'woocommerce_update_product', array( self::class, 'save_product' ) );
		}

		self::$save_article_hook_enabled = $add_hook;
	}

	/**
	 * Updates product sync status.
	 *
	 * @param int $product_id Woo product ID.
	 */
	public static function update_product_sync_status( int $product_id ): void {
		$product = wc_get_product( $product_id );

		if ( $product instanceof WC_Product ) {
			$article_instance = new Article_Model();
			$article          = $article_instance->product_to_article( $product );
			$new_hash         = self::generate_article_hash( $article );
			update_post_meta( $product_id, NMF_HASH, $new_hash );
			update_post_meta( $product_id, NMF_ARTICLE_SYNC_STATUS, __( 'Synced', 'flo-sync' ) );

			Admin_Status::set_sync_status_ok( $product_id );
		}
	}

	/**
	 * Maybe set product as draft.
	 *
	 * @param int $product_id Woo product ID.
	 */
	public static function maybe_set_as_draft( int $product_id ): void {
		if ( ! Settings::is_setting_enabled( 'sync_publish_products_status' ) ) {
			return;
		}

		if ( ! empty( $product_id ) ) {
			$product = wc_get_product( $product_id );
			if ( 'publish' === $product->get_status( 'edit' ) ) {
				if ( 'simple' === $product->get_type() ) {
					$product->set_status( 'draft' );
					$product->save();
					Logger::debug( 'Product set to draft (id: ' . $product_id . ')' );
				} elseif ( 'variation' === $product->get_type() ) {
					$product->set_status( 'private' );
					$product->save();
					Logger::debug( 'Product variation set to private (id: ' . $product_id . ')' );
				}
			}
		}
	}

	/**
	 * Transforms article as received from API to standardized product.
	 *
	 * @param array $article_item Article item from API.
	 *
	 * @return array|null Standardized product.
	 */
	public static function transform_article_to_product( array $article_item ): ?array {
		$article = array();

		if ( ! empty( $article_item ) ) {

			$article_status = $article_item[ NMF_IS_ACTIVE ];

			if ( $article_status ) {
				$article[ NMF_IS_ACTIVE_ARTICLE ] = NMF_ARTICLE_PUBLISH;
			} else {
				$article[ NMF_IS_ACTIVE_ARTICLE ] = NMF_ARTICLE_DRAFT;
			}

			$article['Sku']                       = $article_item[ NMF_ARTICLE_NUMBER ];
			$article[ NMF_QUANTITY ]              = $article_item[ NMF_QUANTITY ];
			$article['Name']                      = $article_item[ NMF_ARTICLE_NAME ];
			$article[ NMF_IS_STOCK_ITEM ]         = $article_item[ NMF_IS_STOCK_ITEM ];
			$article[ NMF_ARTICLE_DISPLAY_ORDER ] = $article_item[ NMF_ARTICLE_DISPLAY_ORDER ];

			$stk = $article_item[ NMF_IS_STOCK_ITEM ];

			if ( $stk ) {
				$article['manage_stock'] = 'yes';
			} else {
				$article['manage_stock'] = 'no';
			}

			$article['ArticleTextHtml']         = is_null( $article_item['ArticleTextHtml'] ) ? '' : $article_item['ArticleTextHtml'];
			$article['Description']             = $article_item['ArticleDescription'];
			$article['GroupId']                 = $article_item['GroupId'];
			$article[ NMF_RETAIL_PRICE ]        = $article_item[ NMF_RETAIL_PRICE ];
			$article[ NMF_RECOMMENDED_PRICE ]   = $article_item[ NMF_RECOMMENDED_PRICE ];
			$article[ NFM_IS_WEB_SHOP_ARTICLE ] = $article_item[ NFM_IS_WEB_SHOP_ARTICLE ];
			$article[ NMF_ARTICLE_TYPE ]        = $article_item[ NMF_ARTICLE_TYPE ];
			$article[ NMF_ACCOUNT_CODE_ID ]     = $article_item[ NMF_ACCOUNT_CODE_ID ];
			$article[ NMF_BARCODE ]             = $article_item[ NMF_BARCODE ];   // Adding barcode 19-02-2019.
			$article['Files']                   = $article_item['Files'];       // Adding barcode 19-02-2019.

			$article['BarcodeAlternatives']           = $article_item['BarcodeAlternatives'] ?? array();
			$article[ NMF_ARTICLE_ORDER_TICKET_TEXT ] = $article_item[ NMF_ARTICLE_ORDER_TICKET_TEXT ] ?? array();

			if ( 0 === $article_item[ NMF_ARTICLE_TYPE ] ) {
				$article[ NMF_ARTICLE_TYPE ] = NMF_TYPE_GOODS;
			}

			/**
			 * Filters the article data before it is returned as a product.
			 *
			 * @param array $article      The transformed article data. Must be returned as an array, or null.
			 * @param array $article_item The article data as from the API.
			 */
			$article = apply_filters( 'moreflo_sync_transform_article_to_product', $article, $article_item );
		}

		return is_array( $article ) ? $article : null;
	}

	/**
	 * Checks if product contains all required data.
	 *
	 * @param array $article The article data.
	 *
	 * @return bool
	 */
	private static function is_full_product( array $article ): bool {
		if (
			! empty( $article[ NMF_ARTICLE_NUMBER ] ) &&
			! empty( $article[ NMF_ACCOUNT_CODE_ID ] ) &&
			! empty( $article[ NMF_ARTICLE_TYPE ] )
		) {
			return true;
		}

		return false;
	}

	/**
	 * Function hooked to Woo product save.
	 *
	 * @param int        $product_id The product ID.
	 * @param WC_Product $product    The product.
	 */
	public static function save_product( int $product_id, WC_Product $product ): void {
		if ( get_post_meta( $product_id, 'nmf_temp_lock', true ) ) {
			Logger::debug( 'Save product skipping, due to temporary lock' . $product_id );

			return;
		}
		$article_instance = new Article_Model();
		$articles         = array();
		$ignore_status    = array( NMF_ARTICLE_DRAFT, 'auto-draft' );
		$product_status   = $product->get_status();

		if ( $product instanceof WC_Product_Variable && ! in_array( $product_status, $ignore_status, true ) ) {

			$variations = $product->get_children();

			if ( is_array( $variations ) && ! empty( $variations ) ) {

				foreach ( $variations as $variant_id ) {
					$articles[ $variant_id ] = $article_instance->variant_to_article( $product, $variant_id );
				}
			}
		} else {
			$articles[ $product_id ] = $article_instance->product_to_article( $product );
		}

		if ( ! empty( $articles ) ) {

			foreach ( $articles as $id => $article ) {
				Logger::debug( 'Saving article' );
				Logger::debug( $article );

				if ( ! empty( $article ) && self::is_full_product( $article ) ) {

					$old_hash = get_post_meta( $id, NMF_HASH, true );
					$new_hash = self::generate_article_hash( $article );
					if ( $old_hash !== $new_hash ) {
						$main_api = new Main_API();
						if ( $old_hash ) {
							$response = $main_api->put_articles( $article );
						} else {
							$response = $main_api->post_articles( $article );
						}

						if ( $response ) {
							update_post_meta( $id, NMF_HASH, $new_hash );
							update_post_meta( $id, NMF_ARTICLE_SYNC_STATUS, __( 'Synced', 'flo-sync' ) );
							update_post_meta( $id, 'mf_last_sync_response', $response );
							Logger::debug( 'Product pushed and meta data updated' );

							Admin_Status::set_sync_status_from_response( $id, $response );
						}
					} else {
						Logger::debug( 'Product hash verification failed, same as last sync' );
					}
				} else {
					Logger::error( 'Article updated failed due empty article or is not full article' );
				}
			}
		}
	}

	/**
	 * Generate hash used to see if article has changed.
	 *
	 * @param array $article From API.
	 *
	 * @return false|string
	 */
	public static function generate_article_hash( array $article ): false|string {

		$hash_string     = '';
		$complied_string = '';

		if ( ! empty( $article ) ) {

			$hashed_properties = array(
				NMF_ARTICLE_NUMBER,
				NMF_ARTICLE_NAME,
				NMF_ARTICLE_DISPLAY_ORDER,
				NMF_UNIT,
				NMF_IS_ACTIVE,
				NMF_UPDATE_ON_EXISTING,
				// 'Quantity',
				NFM_IS_WEB_SHOP_ARTICLE,
				NMF_IS_STOCK_ITEM,
				NMF_IS_OPEN_ARTICLE,
				NMF_ACCOUNT_CODE_ID,
				NMF_RECOMMENDED_PRICE,
				NMF_RETAIL_PRICE,
				NMF_ARTICLE_TYPE,
			);

			foreach ( $hashed_properties as $key ) {

				if ( isset( $article[ $key ] ) ) {
					$complied_string .= $article[ $key ];
				}
			}
			$hash_string = wp_hash( $complied_string, 'nonce' );
		}

		return $hash_string;
	}


	/**
	 * Get article property from API.
	 *
	 * @param string   $sku      The article SKU.
	 * @param string   $property The article property.
	 * @param API|null $api      The API instance. If null, defaults to current Main API.
	 *
	 * @return mixed|null The article property value, or null if not found.
	 */
	public static function get_article_property( string $sku, string $property, ?API $api = null ): mixed {
		if ( empty( $api ) ) {
			$api = new Main_API();
		}

		$article = $api->get_article( $sku );

		if ( ! empty( $article ) && isset( $article[ $property ] ) ) {
			return $article[ $property ];
		}

		return null;
	}

	/**
	 * Get article bookkeeping account code ID from API.
	 *
	 * @param string   $sku The article SKU.
	 * @param API|null $api The API instance. If null, defaults to current Main API.
	 *
	 * @return string
	 */
	public static function get_article_bookkeeping_account_code_id( string $sku, ?API $api = null ): string {
		$value = self::get_article_property( $sku, NMF_ACCOUNT_CODE_ID, $api );
		return $value ?? '';
	}
}
