<?php
/**
 * NMF Article model.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Articles\Models;

use Exception;
use NMFS\Settings;
use NMFS\Sync\API\Main_API;
use NMFS\Logger;
use NMFS\Sync\Items\Articles\Stock\Stock;
use NMFS\Sync\Items\Bookkeeping\Article_Account_Codes\Article_Account_Codes;
use NMFS\Sync\Items\Groups\Groups as Group_Sync;
use WC_Product;
use WC_Product_Factory;
use WC_Product_Variable;
use WC_Product_Variation;

define( 'PRODUCT_TYPE', 'product_type' );
define( 'SIMPLE', 'simple' );
define( 'MANAGESTOCK', '_manage_stock' );
define( 'STOCKSTATUS', '_stock_status' );
define( 'DOWNLOADABLE', '_downloadable' );
define( 'VIRTUAL', '_virtual' );
define( 'INSTOCK', 'instock' );

/**
 * Model class.
 */
class Article_Model {

	/**
	 * Whether syncing product titles is enabled.
	 *
	 * @var bool $is_sync_product_title_active
	 */
	private bool $is_sync_product_title_active;

	/**
	 * Whether syncing product display order is enabled.
	 *
	 * @var bool $is_sync_display_order_active
	 */
	private bool $is_sync_display_order_active;

	/**
	 * Whether syncing product prices is enabled.
	 *
	 * @var bool $is_sync_price_active
	 */
	private bool $is_sync_price_active;

	/**
	 * Whether syncing product pictures is enabled.
	 *
	 * @var bool $is_sync_pic_active
	 */
	private bool $is_sync_pic_active;

	/**
	 * Whether syncing tax classes is enabled.
	 *
	 * @var bool $is_sync_tax_active
	 */
	private bool $is_sync_tax_active;

	/**
	 * Whether syncing article description is enabled.
	 *
	 * @var bool $is_sync_article_desc
	 */
	private bool $is_sync_article_desc;

	/**
	 * Whether syncing article status is enabled.
	 *
	 * @var bool $is_sync_article_status
	 */
	private bool $is_sync_article_status;

	/**
	 * Class constructor.
	 */
	public function __construct() {
		$this->is_sync_product_title_active = Settings::is_setting_enabled( 'is_product_title_active' );
		$this->is_sync_display_order_active = Settings::is_setting_enabled( 'sync_product_display_order_status' );
		$this->is_sync_price_active         = Settings::is_setting_enabled( NMF_IS_PRICE_ACTIVE );
		$this->is_sync_pic_active           = Settings::is_setting_enabled( 'is_pic_active' );
		$this->is_sync_tax_active           = Settings::is_setting_enabled( 'is_tax_class_active' );
		$this->is_sync_article_desc         = Settings::is_setting_enabled( 'sync_article_description' );
		$this->is_sync_article_status       = Settings::is_setting_enabled( 'sync_publish_products_status' );
	}

	/**
	 * Add new product in WC.
	 *
	 * @param array $product Product data. Usually from API.
	 *
	 * @return int|void
	 */
	public function add( array $product ) {
		try {
			$product_obj = new WC_Product();
			$product_obj->set_name( $product['Name'] );
			$product_obj->set_sku( $product['Sku'] );
			if (
				$this->is_sync_article_status &&
				NMF_TYPE_GOODS === $product[ NMF_ARTICLE_TYPE ]
			) {
				$product_obj->set_status( $product[ NMF_IS_ACTIVE_ARTICLE ] );
			} else {
				$product_obj->set_status( NMF_ARTICLE_DRAFT );
			}
			$product_obj->set_featured( 'no' );
			$product_obj->set_catalog_visibility( 'visible' );

			$mf_wc_tax_rules = Article_Account_Codes::get_all();

			if ( $this->is_sync_display_order_active ) {
				$product_obj->set_menu_order( $product[ NMF_ARTICLE_DISPLAY_ORDER ] );
			}

			$woocommerce_calc_taxes_enable = get_option( 'woocommerce_calc_taxes' );
			if ( $this->is_sync_price_active ) {
				$woocommerce_prices_include_tax = get_option( 'woocommerce_prices_include_tax' );
				if (
					$this->is_sync_tax_active &&
					'no' === $woocommerce_prices_include_tax &&
					'yes' === $woocommerce_calc_taxes_enable
				) {
					// Calculate amount.
					$tax_rate     = $mf_wc_tax_rules[ $product[ NMF_ACCOUNT_CODE_ID ] ]->get_vat_rate();
					$tax_amount   = $product[ NMF_RETAIL_PRICE ] - ( $product[ NMF_RETAIL_PRICE ] / ( ( $tax_rate / 100 ) + 1 ) );
					$re_cal_price = $product[ NMF_RETAIL_PRICE ] - $tax_amount;
					$product_obj->set_regular_price( $re_cal_price );
				} else {
					$product_obj->set_regular_price( $product[ NMF_RETAIL_PRICE ] );
				}
			}

			if ( $this->is_sync_article_desc ) {
				$product_obj->set_description( $product['ArticleTextHtml'] );
			}

			$new_product_id = $product_obj->save();

			update_post_meta( $new_product_id, NMF_ARTICLE_ACCOUNT_CODE, $product[ NMF_ACCOUNT_CODE_ID ] );

			if ( $this->is_sync_tax_active ) {
				if ( isset( $mf_wc_tax_rules[ $product[ NMF_ACCOUNT_CODE_ID ] ] ) ) {
					update_post_meta( $new_product_id, '_tax_status', 'taxable' );
					update_post_meta( $new_product_id, '_tax_class', $mf_wc_tax_rules[ $product[ NMF_ACCOUNT_CODE_ID ] ]->get_slug() );
				} else {
					update_post_meta( $new_product_id, '_tax_status', 'none' );
					Logger::error( 'Tax class not found for account code: ' . $product[ NMF_ACCOUNT_CODE_ID ] );
				}
			}

			if ( isset( $product[ NMF_BARCODE ] ) && $product[ NMF_BARCODE ] ) {
				add_post_meta( $new_product_id, '_custom_product_barcodes_field', $product[ NMF_BARCODE ] );// Adding barcode 19-02-2019.
			}
			if ( isset( $product['BarcodeAlternatives'] ) && $product['BarcodeAlternatives'] ) {
				// Loop through all barcode alternatives and add to the list.
				foreach ( $product['BarcodeAlternatives'] as $barcode ) {

					if ( ! empty( $barcode['Barcode'] ) ) {
						$alt_bar_code = $barcode['Barcode'];
						add_post_meta( $new_product_id, '_custom_product_barcodes_field', $alt_bar_code );
					}
				}
			}
			/**
			 * Handling simple products
			 */
			if ( NMF_TYPE_GOODS === $product[ NMF_ARTICLE_TYPE ] ) {
				wp_set_object_terms( $new_product_id, SIMPLE, PRODUCT_TYPE );

				// We don't update stock here, it's done in separate stock sync.
			}

			Logger::debug( $new_product_id . 'product productype : ' . $product[ NMF_ARTICLE_TYPE ] );
			/**
			 * Image upload integration 06-10-2020
			 */
			if ( ( $this->is_sync_pic_active ) && $product['Files'] ) {
				$this->upload_images( $new_product_id, $product['Files'] );
			}

			/**
			 * Order ticket text.
			 */
			$order_ticket = array_filter( explode( '\r\n', $product[ NMF_ARTICLE_ORDER_TICKET_TEXT ] ) );
			add_post_meta( $new_product_id, 'nmf_order_ticket_text', $order_ticket );

			do_action( 'moreflo_sync_articles_add', $new_product_id, $product );

			/**
			 * Save GroupId
			 */
			$this->update_product_group( $new_product_id, $product['GroupId'] );

			return $new_product_id;

		} catch ( Exception $e ) {
			Logger::error(
				'Exception occurred while adding the product to the site: ' . $product['Sku'] . ' ' . $e->getMessage()
			);
		}
	}

	/**
	 * Update product in WC.
	 *
	 * @param int   $product_id WC Product ID.
	 * @param array $product    Product data. Usually from API.
	 *
	 * @return void
	 */
	public function update( int $product_id, array $product ): void {
		try {

			$product_obj = wc_get_product( $product_id );

			if ( $product_obj instanceof WC_Product ) {

				if ( $this->is_sync_product_title_active ) {
					$product_obj->set_name( $product['Name'] );
				}

				if ( $this->is_sync_display_order_active ) {
					$product_obj->set_menu_order( $product[ NMF_ARTICLE_DISPLAY_ORDER ] );
				}

				if (
					$this->is_sync_article_status &&
					NMF_TYPE_GOODS === $product[ NMF_ARTICLE_TYPE ]
				) {
					$product_obj->set_status( $product[ NMF_IS_ACTIVE_ARTICLE ] );
				}

				$product_type = WC_Product_Factory::get_product_type( $product_id );

				$mf_wc_tax_rules               = Article_Account_Codes::get_all();
				$woocommerce_calc_taxes_enable = get_option( 'woocommerce_calc_taxes' );
				if ( $this->is_sync_price_active ) {
					$product_price                  = $product[ NMF_RETAIL_PRICE ];
					$woocommerce_prices_include_tax = get_option( 'woocommerce_prices_include_tax' );
					if (
						$this->is_sync_tax_active &&
						'no' === $woocommerce_prices_include_tax &&
						'yes' === $woocommerce_calc_taxes_enable
					) {
						// Calculate amount.
						$tax_rate      = $mf_wc_tax_rules[ $product[ NMF_ACCOUNT_CODE_ID ] ]->get_vat_rate();
						$tax_amount    = $product_price - ( $product_price / ( ( $tax_rate / 100 ) + 1 ) );
						$product_price = $product_price - $tax_amount;
					}

					$product_obj->set_regular_price( $product_price );
				}

				if ( $product_obj instanceof WC_Product_Variation ) {

					update_post_meta( $product_obj->get_parent_id(), NMF_ARTICLE_ACCOUNT_CODE, $product[ NMF_ACCOUNT_CODE_ID ] );
					/**
					 * Todo: get main image and add to thumbnail and rest of image save it main products
					 */

				} else {
					update_post_meta( $product_id, NMF_ARTICLE_ACCOUNT_CODE, $product[ NMF_ACCOUNT_CODE_ID ] );

					if ( ( $this->is_sync_pic_active ) && $product['Files'] ) {
						$this->upload_images( $product_id, $product['Files'] );
					}
				}

				// Remove all since we don't know which once are new.
				if ( get_post_meta( $product_id, '_custom_product_barcodes_field' ) ) {
					delete_post_meta( $product_id, '_custom_product_barcodes_field' );
				}

				if ( isset( $product[ NMF_BARCODE ] ) && $product[ NMF_BARCODE ] ) {
					add_post_meta( $product_id, '_custom_product_barcodes_field', $product[ NMF_BARCODE ] );
				}

				if ( isset( $product['BarcodeAlternatives'] ) && $product['BarcodeAlternatives'] ) {
					// Loop through all barcode alternatives and add to the list.
					foreach ( $product['BarcodeAlternatives'] as $barcode ) {

						if ( false === empty( $barcode['Barcode'] ) ) {

							$alt_bar_code = $barcode['Barcode'];

							Logger::debug( 'Alternative barcode: ' . $alt_bar_code );
							add_post_meta( $product_id, '_custom_product_barcodes_field', $alt_bar_code );
						}
					}
				}

				if (
					$this->is_sync_tax_active &&
					isset( $product[ NMF_ACCOUNT_CODE_ID ] ) &&
					isset( $mf_wc_tax_rules[ $product[ NMF_ACCOUNT_CODE_ID ] ] )
				) {
					update_post_meta( $product_id, '_tax_status', 'taxable' );
					update_post_meta( $product_id, '_tax_class', $mf_wc_tax_rules[ $product[ NMF_ACCOUNT_CODE_ID ] ]->get_slug() );
				}

				if ( $this->is_sync_article_desc ) {
					$product_obj->set_description( $product['ArticleTextHtml'] );
				}

				$product_obj->save();

				if (
					NMF_TYPE_GOODS === $product[ NMF_ARTICLE_TYPE ] &&
					! in_array( $product_type, array( 'simple', 'composite', 'variable' ), true )
				) {
					wp_get_object_terms( $product_id, 'product_type' );
					wp_set_object_terms( $product_id, SIMPLE, PRODUCT_TYPE );

					delete_post_meta( $product_id, DOWNLOADABLE, 'no' );
					delete_post_meta( $product_id, VIRTUAL, 'yes' );
					delete_post_meta( $product_id, MANAGESTOCK, 'no' );
					delete_post_meta( $product_id, STOCKSTATUS, INSTOCK );

					Logger::debug( 'Product type change to Simple' );
				}

				/**
				 * Order ticket text.
				 */
				$order_ticket = array_filter( explode( '\r\n', $product[ NMF_ARTICLE_ORDER_TICKET_TEXT ] ) );
				update_post_meta( $product_id, 'nmf_order_ticket_text', $order_ticket );

				// We don't update stock here, it's done in separate stock sync.

				do_action( 'moreflo_sync_articles_update', $product_id, $product );

				/**
				 * Save GroupId
				 */
				$this->update_product_group( $product_id, $product['GroupId'] );

			}
		} catch ( Exception $e ) {
			Logger::error(
				'Exception occurred while updating the product to the site: ' . $product['Sku'] . ' ' . $e->getMessage()
			);
		}
	}

	/**
	 * Updates product group to Apps group if it exists
	 *
	 * @param int    $product_id  WC Product ID.
	 * @param string $mf_group_id Hub Group ID.
	 */
	private function update_product_group( int $product_id, string $mf_group_id ): void {
		if ( ! Group_Sync::is_enabled() ) {
			return;
		}

		update_post_meta( $product_id, 'mf_group_id', $mf_group_id );
		$terms = array();
		if ( ! empty( $mf_group_id ) ) {
			$terms = get_terms(
				array(
					'taxonomy'   => 'product_cat',
					'meta_query' => array(
						array(
							'key'   => 'mf_group_id',
							'value' => $mf_group_id,
						),
					),
					'fields'     => 'ids',
					'number'     => 1,
					'hide_empty' => false,
				)
			);
		}
		wp_set_object_terms( $product_id, $terms, 'product_cat' );
	}

	/**
	 * Upload images for product.
	 *
	 * @param int     $product_id WC Product ID.
	 * @param array[] $images     Array of images.
	 *
	 * @return void
	 */
	private function upload_images( int $product_id, array $images ): void {
		Logger::debug( 'upload images:' );
		/**
		 * Todo
		 * Stop creating multiple images a for products which is stop force sync creating multiple image generated */
		$image_gallery = array();
		foreach ( $images as $image ) {
			$image_url     = $image['URL']; // Define the image URL here.
			$image_name    = $image['Name'];
			$image_type    = $image['ContentType'];
			$feature_image = $image['IsMainFile'];
			$tmp           = get_temp_dir() . $image_name; // temp folder locations and file name.

			// If the file is already imported, skip download.
			$already_exists = get_posts(
				array(
					'post_type'           => 'attachment',
					'posts_per_page'      => 1,
					'ignore_sticky_posts' => true,
					'meta_query'          => array(
						'relation' => 'AND',
						array(
							'key'   => '_mf_sync_image_id',
							'value' => $image['Id'],
						),
						array(
							'key'   => '_mf_sync_image_name',
							'value' => $image['Name'],
						),
					),
					'fields'              => 'ids',
				)
			);

			if ( ! empty( $already_exists ) ) {
				Logger::debug( 'Image already exists, skipping download of image (' . $image['Id'] . ')' );

				if ( $feature_image ) {
					set_post_thumbnail( $product_id, $already_exists[0] );
				} else {
					$image_gallery[] = $already_exists[0];
				}

				wp_update_post(
					array(
						'ID'           => $already_exists[0],
						'post_content' => $image['Description'],
					)
				);
				continue;
			}

			$compatiable_types = array( 'image/jpeg', 'image/png', 'image/gif' );
			if ( ! in_array( $image_type, $compatiable_types, true ) ) {
				Logger::debug( 'Image type invalid, skipping download of image (' . $image['Id'] . ')' );
				continue;
			}
			$api        = new Main_API();
			$image_data = $api->get_image_file( $image_url, $image_type );

			if ( $image_data ) {
				require_once ABSPATH . 'wp-admin/includes/media.php';
				require_once ABSPATH . 'wp-admin/includes/file.php';
				require_once ABSPATH . 'wp-admin/includes/image.php';

				// Download file to temp location.
				$tmp_dl = file_put_contents( $tmp, $image_data ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_system_read_file_put_contents

				// Set variables for storage.
				$file_array['name']     = $image_name;
				$file_array['tmp_name'] = $tmp;

				if ( is_wp_error( $tmp_dl ) ) {
					try {
						unlink( $file_array['tmp_name'] );
					} catch ( Exception $e ) {
						Logger::error( 'Error on delete image from temp folder :' . $file_array['tmp_name'] );
					}
					Logger::error( 'Error on download image from url :' . $image_url );
					continue;
				} else {
					Logger::debug( 'Successfully downloaded image from url :' . $image_url );
				}

				Logger::debug( 'start image upload to WordPress' );
				$image_id = media_handle_sideload(
					$file_array,
					$product_id,
					$image_name,
					array(
						'post_content' => $image['Description'],
					)
				);

				if ( is_wp_error( $image_id ) ) {
					try {
						unlink( $file_array['tmp_name'] );
					} catch ( Exception $e ) {
						Logger::error( 'Error on delete image from temp folder :' . $file_array['tmp_name'] );
					}
					Logger::error( 'Error on upload to WordPress image :' . $image_id->get_error_message() );
					continue;
				} else {
					Logger::debug( 'Successfully upload image to product ' . $product_id );
					update_post_meta( $image_id, '_mf_sync_image_id', $image['Id'] );
					update_post_meta( $image_id, '_mf_sync_image_name', $image['Name'] );
				}

				if ( $feature_image ) {
					set_post_thumbnail( $product_id, $image_id );
				} else {
					$image_gallery[] = $image_id;
				}
				unset( $image_id );
			} else {
				Logger::error( 'Error on download image from url:' . $image_url . '. Might be due article name containing spaces.EX: /articles/Article+test/' );
			}
		}
		if ( ! empty( $image_gallery ) ) {
			update_post_meta( $product_id, '_product_image_gallery', implode( ',', $image_gallery ) );
		}
		Logger::debug( 'upload images End:' );
	}

	/**
	 * Transform product to data for API.
	 *
	 * @param WC_Product $product WC product to transform into API data.
	 *
	 * @return array
	 */
	public function product_to_article( WC_Product $product ): array {

		$product_id = $product->get_id();

		$manage_stock = get_post_meta( $product_id, MANAGESTOCK, true ) === 'yes';

		if ( $product instanceof WC_Product_Variation ) {
			$account_code = get_post_meta( $product->get_parent_id(), NMF_ARTICLE_ACCOUNT_CODE, true );
		} else {
			$account_code = get_post_meta( $product_id, NMF_ARTICLE_ACCOUNT_CODE, true );
		}

		$quantity = Stock::get_stock_quantity_for_main_account( $product );

		$article = array(
			NMF_ARTICLE_NAME          => $product->get_name(),
			NMF_ARTICLE_DISPLAY_ORDER => $product->get_menu_order(),
			NMF_ARTICLE_NUMBER        => $product->get_sku(),
			NMF_UNIT                  => 'St',
			// ISACTIVE => false, // This is commented out to stop post draft product as inactive article to Hub. Request #8698.
			NMF_UPDATE_ON_EXISTING    => true,
			NFM_IS_WEB_SHOP_ARTICLE   => true,
			NMF_ACCOUNT_CODE_ID       => $account_code,
		);

		if ( ! $this->is_sync_product_title_active ) {
			unset( $article[ NMF_ARTICLE_NAME ] );
		}

		if ( ! $this->is_sync_display_order_active ) {
			unset( $article[ NMF_ARTICLE_DISPLAY_ORDER ] );
		}

		$regular_price = $product->get_regular_price();

		if ( $this->is_sync_price_active ) {
			$article[ NMF_RETAIL_PRICE ] = $regular_price;
		}

		if ( $product->get_status() === NMF_ARTICLE_PUBLISH ) {
			$article[ NMF_IS_ACTIVE ] = true;
		}

		if ( $manage_stock && null !== $quantity ) {
			$article[ NMF_QUANTITY ] = $quantity;
		}

		if ( $manage_stock ) {
			$article[ NMF_IS_STOCK_ITEM ] = true;
		} else {
			$article[ NMF_IS_STOCK_ITEM ] = false;
		}

		if ( ! get_post_meta( $product_id, NMF_HASH, true ) ) {
			// Set new products as Accumulative.
			$article[ NMF_ACCUMULATIVE ] = true;
		}

		$article[ NMF_ARTICLE_TYPE ] = NMF_TYPE_GOODS;

		return $article;
	}

	/**
	 * Transform variable product variant to data for API.
	 *
	 * @param WC_Product_Variable $product    Variable WC product.
	 * @param int                 $variant_id Variant ID.
	 *
	 * @return array|null
	 */
	public function variant_to_article( WC_Product_Variable $product, int $variant_id ): ?array {

		$variant = wc_get_product( $variant_id );

		if ( $variant instanceof WC_Product_Variation ) {

			$product_id   = $product->get_id();
			$manage_stock = get_post_meta( $variant_id, MANAGESTOCK, true );
			$account_code = get_post_meta( $product_id, NMF_ARTICLE_ACCOUNT_CODE, true );
			$quantity     = $variant->get_stock_quantity();

			$article = array(
				NMF_ARTICLE_NAME          => $this->get_variation_name( $product, $variant ),
				NMF_ARTICLE_DISPLAY_ORDER => $variant->get_menu_order(),
				NMF_ARTICLE_NUMBER        => $variant->get_sku( 'edit' ),
				NMF_UNIT                  => 'St',
				// ISACTIVE => false, // This is commented out to stop post draft product as inactive article to Hub. Request #8698.
				NMF_UPDATE_ON_EXISTING    => true,
				NFM_IS_WEB_SHOP_ARTICLE   => true,
				NMF_ACCOUNT_CODE_ID       => $account_code,
			);

			if ( ! $this->is_sync_product_title_active ) {
				unset( $article[ NMF_ARTICLE_NAME ] );
			}

			if ( ! $this->is_sync_display_order_active ) {
				unset( $article[ NMF_ARTICLE_DISPLAY_ORDER ] );
			}

			$regular_price = $variant->get_regular_price();

			if ( $this->is_sync_price_active ) {
				$article[ NMF_RECOMMENDED_PRICE ] = $regular_price; // If sale price has not been set in woocommerce, both recommended and retail prices will be equal. // TODO: Check if this is correct. Sale price seem to be ignored completely.
				$article[ NMF_RETAIL_PRICE ]      = $regular_price;
			}

			if ( $variant->get_status() === NMF_ARTICLE_PUBLISH ) {
				$article[ NMF_IS_ACTIVE ] = true;
			}

			if ( ( 'yes' === $manage_stock ) && null !== $quantity ) {
				$article[ NMF_QUANTITY ] = $quantity;
			}

			if ( 'yes' === $manage_stock ) {
				$article[ NMF_IS_STOCK_ITEM ] = true;
			} else {
				$article[ NMF_IS_STOCK_ITEM ] = false;
			}

			$article[ NMF_ARTICLE_TYPE ] = NMF_TYPE_GOODS;

			if ( ! get_post_meta( $variant_id, NMF_HASH, true ) ) {
				// Set new products as Accumulative.
				$article[ NMF_ACCUMULATIVE ] = true;
			}

			return $article;

		} else {
			Logger::error( 'This product is not an instance of WC_Product_Variation' );
		}

		return null;
	}

	/**
	 * Get the variation name. Will cut names longer than 60 characters
	 *
	 * @param WC_Product           $product   Variable product.
	 * @param WC_Product_Variation $variation Product variation to get name of.
	 *
	 * @return string Variation name.
	 */
	private function get_variation_name( WC_Product $product, WC_Product_Variation $variation ): string {
		$full_name = $product->get_name() . ' - ' . wc_get_formatted_variation( $variation, true, false );

		return mb_substr( $full_name, 0, 60 );
	}
}
