<?php
/**
 * Shipping functions for Multi-account sync.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Articles\Stock;

use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;
use WC_Shipping_Rate;
use WC_Shipping_Zone;
use WC_Shipping_Zones;

/**
 * Shipping class.
 */
class Shipping {
	/**
	 * Setup shipping functions.
	 */
	public static function setup(): void {
		add_action( 'woocommerce_init', array( __CLASS__, 'shipping_instance_form_fields_filters' ) );
		add_action( 'woocommerce_after_shipping_rate', array( __CLASS__, 'maybe_show_shipping_meta' ), 10, 2 );

		add_filter( 'woocommerce_shipping_packages', array( __CLASS__, 'shipping_packages' ), 50 ); // Should not be done early.

		add_action( 'wp_ajax_nmf_update_shipping_store', array( __CLASS__, 'ajax_update_shipping_store' ) );
		add_action( 'wp_ajax_nopriv_nmf_update_shipping_store', array( __CLASS__, 'ajax_update_shipping_store' ) );
	}

	/**
	 * Shipping instance form fields.
	 */
	public static function shipping_instance_form_fields_filters(): void {
		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return;
		}

		/**
		 * Magic method okay for WC().
		 *
		 * @noinspection PhpUndefinedFieldInspection
		 */
		$shipping_methods = WC()->shipping->get_shipping_methods();
		foreach ( $shipping_methods as $shipping_method ) {
			add_filter( 'woocommerce_shipping_instance_form_fields_' . $shipping_method->id, array( __CLASS__, 'shipping_instance_form_add_extra_fields' ) );
		}
	}

	/**
	 * Shipping Instance form add extra fields.
	 *
	 * @param array $form_fields Shipping method form fields.
	 *
	 * @return array
	 */
	public static function shipping_instance_form_add_extra_fields( array $form_fields ): array {
		$form_fields['nmf_shipping_type'] = array(
			'title'       => esc_html__( 'Shipping type', 'flo-sync' ),
			'type'        => 'select',
			'options'     => self::get_shipping_method_types(),
			'description' => '',
			'default'     => self::get_default_shipping_type_id(),
		);

		return $form_fields;
	}

	/**
	 * Get the shipping method type of WC_Shipping_Rate object.
	 *
	 * @param WC_Shipping_Rate $method Shipping method object.
	 * @param bool             $fallback_to_default Fallback to the default shipping type if not found.
	 *
	 * @return string
	 */
	public static function get_shipping_method_type( WC_Shipping_Rate $method, bool $fallback_to_default = true ): string {
		$method_fields = get_option(
			'woocommerce_' . $method->get_method_id() . '_' . $method->get_instance_id() . '_settings'
		);

		if (
			! empty( $method_fields['nmf_shipping_type'] ) &&
			self::is_shipping_type_valid( $method_fields['nmf_shipping_type'] )
		) {
			return $method_fields['nmf_shipping_type'];
		}
		return $fallback_to_default ? self::get_default_shipping_type_id() : '';
	}

	/**
	 * Get shipping method types.
	 *
	 * @return array
	 */
	public static function get_shipping_method_types(): array {
		return array(
			'pickup'   => __( 'Pickup', 'flo-sync' ),
			'delivery' => __( 'Delivery', 'flo-sync' ),
		);
	}

	/**
	 * Get default shipping type id.
	 *
	 * @return string
	 */
	public static function get_default_shipping_type_id(): string {
		return 'pickup';
	}

	/**
	 * Check if shipping type is one of the predefined types.
	 *
	 * @param mixed $shipping_type Value to check.
	 *
	 * @return bool
	 */
	public static function is_shipping_type_valid( mixed $shipping_type ): bool {
		if ( ! is_string( $shipping_type ) ) {
			return false;
		}
		$shipping_types = self::get_shipping_method_types();
		return isset( $shipping_types[ $shipping_type ] );
	}

	/**
	 * Check if shipping method is allowed for a Hub account.
	 *
	 * @param Backoffice_Account_Model $backoffice_account Hub account.
	 * @param WC_Shipping_Zone         $package_zone Shipping zone.
	 * @param WC_Shipping_Rate         $shipping_method Shipping method.
	 *
	 * @return bool True if shipping method is allowed for the Hub account.
	 */
	public static function is_shipping_method_allowed_for_backoffice_account(
		Backoffice_Account_Model $backoffice_account,
		WC_Shipping_Zone $package_zone,
		WC_Shipping_Rate $shipping_method
	): bool {
		$package_shipping_method = $package_zone->get_id() . ':' . $shipping_method->get_id();
		return in_array( $package_shipping_method, $backoffice_account->allowed_shipping_methods, true );
	}

	/**
	 * Add store select or other metas to cart shipping methods.
	 *
	 * @param WC_Shipping_Rate $method Shipping method object.
	 * @param int              $index Shipping package index.
	 */
	public static function maybe_show_shipping_meta( WC_Shipping_Rate $method, int $index ): void {
		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return;
		}

		$packages     = WC()->shipping()->get_packages();
		$package      = $packages[ $index ];
		$package_zone = WC_Shipping_Zones::get_zone_matching_package( $package );

		$cart_shipping_type = self::get_shipping_method_type( $method );

		/**
		 * If the shipping method is pickup, show store select.
		 */
		if ( 'pickup' === $cart_shipping_type ) {
			$accounts = Checkout::get_pickup_backoffice_accounts_with_stock( $package_zone, $method );

			$shipping_method_id = $method->get_id();
			$shipping_method_id = str_replace( ':', '_', $shipping_method_id );
			$select_id          = 'nmf_backoffice_account_id_' . $shipping_method_id;

			$chosen_store = WC()->session->get( 'nmf_cart_chosen_store', array() );
			if ( ( $chosen_store['method'] ?? '' ) === $shipping_method_id ) {
				$chosen_store_id = $chosen_store['store'] ?? '';
			} else {
				$chosen_store_id = '';
			}

			require NMF_PLUGIN_DIR . 'views/cart/pickup-shipping-meta.php';

			/**
			 * If the shipping method is delivery, only check if there is any store with stock.
			 */
		} elseif ( 'delivery' === $cart_shipping_type ) {
			$account = Checkout::get_delivery_backoffice_account_with_stock( $package_zone, $method );

			require NMF_PLUGIN_DIR . 'views/cart/delivery-shipping-meta.php';
		}
	}

	/**
	 * Filter shipping packages.
	 *
	 * @param array $packages Shipping packages.
	 *
	 * @return array
	 */
	public static function shipping_packages( array $packages ): array {
		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return $packages;
		}

		if ( ! isset( $packages[0]['rates'] ) ) {
			return $packages;
		}

		$package_zone = WC_Shipping_Zones::get_zone_matching_package( $packages[0] );

		foreach ( $packages[0]['rates'] as $method_key => $method ) {
			$shipping_method_type = self::get_shipping_method_type( $method );

			switch ( $shipping_method_type ) {
				case 'pickup':
					$accounts = Checkout::get_pickup_backoffice_accounts_with_stock( $package_zone, $method );
					if ( empty( $accounts ) ) {
						unset( $packages[0]['rates'][ $method_key ] );
					}
					break;
				case 'delivery':
					$account = Checkout::get_delivery_backoffice_account_with_stock( $package_zone, $method );
					if ( ! $account ) {
						unset( $packages[0]['rates'][ $method_key ] );
					}
					break;
			}
		}

		return $packages;
	}

	/**
	 * Ajax update shipping store.
	 */
	public static function ajax_update_shipping_store(): never {
		check_ajax_referer( 'nmf-update-shipping-store', 'security' );

		$posted_store  = isset( $_POST['shipping_store'] ) ? wc_clean( wp_unslash( $_POST['shipping_store'] ) ) : array();
		$posted_method = isset( $_POST['shipping_method'] ) ? wc_clean( wp_unslash( $_POST['shipping_method'] ) ) : array();

		WC()->session->set(
			'nmf_cart_chosen_store',
			array(
				'store'  => $posted_store,
				'method' => $posted_method,
			)
		);

		wp_send_json_success();
		exit();
	}
}
