<?php
/**
 * Handles article stock changes.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Articles\Stock;

use NMFS\Sync\Items\Articles\Stock\Models\Stock_Data_Model;
use NMFS\Logger;
use NMFS\Sync\Items\Articles\Articles;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use WC_Order;
use WC_Order_Item;
use WC_Order_Item_Product;
use WC_Order_Refund;
use WC_Product;
use WP_Error;

/**
 * Stock class.
 */
class Stock {

	/**
	 * Constructor.
	 */
	public static function setup(): void {

		add_filter(
			'woocommerce_payment_complete_reduce_order_stock',
			array( self::class, 'maybe_reduce_order_stock' ),
			999, // Should be run after WooCommerce core.
			2
		);
	}

	/**
	 * Replaces core function to reduce stock levels, for multi-account sites.
	 * Based on WC core function wc_reduce_stock_levels().
	 *
	 * @param bool $trigger_reduce Whether to reduce stock.
	 * @param int  $order_id       Order object id.
	 *
	 * @return bool                Whether to use WC core function to reduce stock.
	 */
	public static function maybe_reduce_order_stock( bool $trigger_reduce, int $order_id ): bool {

		// Only run this if multi-account is enabled. Else, the core function should be used.
		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return $trigger_reduce;
		}

		// This check is made since the action for this function also is triggered on order status changes.
		// And an order status change is often triggered after 'woocommerce_pre_payment_complete'
		// but before 'woocommerce_payment_complete'.
		if (
			did_action( 'woocommerce_pre_payment_complete' ) &&
			! did_action( 'woocommerce_payment_complete' )
		) {
			// Order hasn't been synced to Hub as paid yet, wait for that, as we use data from Hub to reduce stock.
			Logger::debug( 'Order not synced as paid yet, wait for that.' );
			return false;
		}

		Logger::debug( 'Start multi-account sync stock reduce for order ' . $order_id );

		$order = wc_get_order( $order_id );

		if ( ! $order ) {
			Logger::error( 'Could not get order when trying to reduce stock, for order id: ' . $order_id );
			return false;
		}

		$stock_already_reduced = $order->get_data_store()->get_stock_reduced( $order_id );

		if ( $stock_already_reduced ) {
			Logger::debug( 'Stock already reduced for order ' . $order_id );
			return false; // Stock already reduced.
		}

		// We need a store with stock management to continue.
		if (
			'yes' !== get_option( 'woocommerce_manage_stock' ) ||
			! apply_filters( 'woocommerce_can_reduce_order_stock', true, $order )
		) {
			Logger::debug(
				'Manage stock disabled when doing Multi-account stock reduction for order: ' . $order_id
			);

			return false;
		}

		// Get stock data from API.
		$articles_stock_data = self::get_stock_data_from_order( $order );

		if ( ! empty( $articles_stock_data ) ) {
			foreach ( $articles_stock_data as $article_sku => $stock_data ) {
				$stock_reduced = self::update_article_stock_data(
					$article_sku,
					$stock_data['stock_data']
				);

				if ( $stock_reduced ) {
					$order_item = $stock_data['order_item'] ?? null;
					// Set stock reduced for order item.
					if ( $order_item instanceof WC_Order_Item ) {
						// To save anyone else from my confusion, this is just for the order item, not the actual product.
						$order_item->add_meta_data(
							'_reduced_stock',
							$order_item->get_quantity(),
							true
						);
						$order_item->save();
					}
				}
			}

			$changes = self::get_order_stock_changes( $order );
			wc_trigger_stock_change_notifications( $order, $changes );
		}

		// Ensure stock is marked as "reduced" in case payment complete or other stock actions are called.
		$order->get_data_store()->set_stock_reduced( $order_id, true );

		do_action( 'woocommerce_reduce_order_stock', $order );

		return false; // Aborts core WooCommerce stock reduction.
	}

	/**
	 * Get stock changes from order, for stock change notifications.
	 *
	 * @param WC_Order $order Order to get changes for.
	 *
	 * @return array
	 */
	private static function get_order_stock_changes( WC_Order $order ): array {
		$changes = array();

		foreach ( $order->get_items() as $item ) {
			if ( ! $item instanceof WC_Order_Item_Product ) {
				continue;
			}

			$product = $item->get_product();

			if ( ! $product instanceof WC_Product ) {
				continue;
			}

			if ( ! $product->managing_stock() ) {
				continue;
			}

			/**
			 * Filter order item quantity.
			 *
			 * @param int|float             $quantity Quantity.
			 * @param WC_Order              $order    Order data.
			 * @param WC_Order_Item_Product $item Order item data.
			 */
			$qty       = apply_filters( 'woocommerce_order_item_quantity', $item->get_quantity(), $order, $item );
			$new_stock = $product->get_stock_quantity();

			$change = array(
				'product' => $product,
				'from'    => $new_stock + $qty,
				'to'      => $new_stock,
			);

			$changes[] = $change;

			/**
			 * Fires when stock reduced to a specific line item
			 *
			 * @param WC_Order_Item_Product $item Order item data.
			 * @param array $change  Change Details.
			 * @param WC_Order $order  Order data.
			 */
			do_action( 'woocommerce_reduce_order_item_stock', $item, $change, $order );
		}

		return $changes;
	}

	/**
	 * Get stock data for a single article from API, and update in WC.
	 *
	 * @param string $article_sku Article SKU.
	 * @param array  $apis_to_skip API:s to skip when getting article stock data.
	 *
	 * @return int|false|null|WP_Error New stock level on success, or false on fail.
	 * Null if article is set to not manage stock.
	 * WP_Error if something went wrong when getting stock from API.
	 */
	public static function get_and_update_article_stock_data(
		string $article_sku,
		array $apis_to_skip = array()
	): false|int|null|WP_Error {
		$stock_data = self::get_article_stock_data( $article_sku, $apis_to_skip );

		if ( is_wp_error( $stock_data ) ) {
			return $stock_data;
		}

		return self::update_article_stock_data( $article_sku, $stock_data );
	}

	/**
	 * Update article stock.
	 *
	 * @param string                  $article_sku Article sku.
	 * @param null|Stock_Data_Model[] $stock_data  Stock data. Stock level in all API:s.
	 *
	 * @return int|false|null New stock level on success, or false. Null if article is set to not manage stock.
	 */
	public static function update_article_stock_data( string $article_sku, ?array $stock_data ): false|int|null {
		$article_hook_status = Articles::$save_article_hook_enabled;
		Articles::set_hook_save_article( false );

		$product_id = wc_get_product_id_by_sku( $article_sku );
		$product    = wc_get_product( $product_id );

		// Product shouldn't manage stock.
		if ( is_null( $stock_data ) ) {
			$product->set_manage_stock( false );
			$product->set_stock_status(); // Defaults to 'instock'.
			$product->save();
			update_post_meta( $product_id, NMF_ARTICLE_STOCK_DATA, null );
			Articles::set_hook_save_article( $article_hook_status );
			return null;
		} else {
			$product->set_manage_stock( true );
			$product->save();
		}

		$stock_total = 0;
		foreach ( $stock_data as $key => $stock ) {
			if ( ! is_a( $stock, Stock_Data_Model::class ) ) {
				unset( $stock_data[ $key ] );
				continue;
			}
			$stock_total += max( $stock->quantity, 0 );
		}

		if ( Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			$stored_stock_data = get_post_meta( $product_id, NMF_ARTICLE_STOCK_DATA, true );

			// phpcs:disable WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
			$current_hash = wp_hash( serialize( $stored_stock_data ) );
			$new_hash     = wp_hash( serialize( $stock_data ) );
			// phpcs:enable

			$meta_up_to_date = true;

			if ( $current_hash !== $new_hash ) {
				$meta_up_to_date = update_post_meta( $product_id, NMF_ARTICLE_STOCK_DATA, $stock_data );
			}

			if ( ! $meta_up_to_date ) {
				Logger::error( 'Failed to update article meta stock data for article ' . $article_sku );
				Articles::set_hook_save_article( $article_hook_status );

				return false;
			}
		}

		$new_stock = self::update_article_wc_stock( $product_id, $stock_total );

		if ( is_wp_error( $new_stock ) ) {
			Logger::error( $new_stock->get_error_message() );

			$new_stock = false;
		}

		Articles::set_hook_save_article( $article_hook_status );

		return $new_stock;
	}

	/**
	 * Update WooCommerce stock.
	 *
	 * @param WC_Product|int $product  WC_Product object or product id.
	 * @param float          $quantity Quantity to set.
	 *
	 * @return WP_Error|int|false New stock level on success. False if stock was not updated. WP_Error if something went wrong.
	 */
	public static function update_article_wc_stock( WC_Product|int $product, float $quantity ): WP_Error|false|int {
		if ( ! is_a( $product, 'WC_Product' ) ) {
			$product = wc_get_product( $product );
		}

		if ( ! $product ) {
			return new WP_Error( 'product_not_found', 'Failed to get article to update stock for.' );
		}

		$old_stock = $product->get_stock_quantity();

		if ( $old_stock === $quantity ) {
			Logger::debug(
				'Stock for article ' . $product->get_sku() . ' was not changed. Old stock: ' . $old_stock . ' Value to set: ' . $quantity
			);

			return false;
		}

		$new_stock = wc_update_product_stock( $product, $quantity );

		if ( is_wp_error( $new_stock ) ) {
			return $new_stock;
		} elseif ( false === $new_stock ) {
			return new WP_Error( 'product_stock_update_failed', 'Failed to update stock for article.' );
		}

		return $new_stock;
	}

	/**
	 * Get stock data from order.
	 *
	 * @param WC_Order|WC_Order_Refund $order Order object.
	 *
	 * @return array Array containing stock data (Stock_Data_Model[]) and order item
	 */
	public static function get_stock_data_from_order( WC_Order|WC_Order_Refund $order ): array {

		$order_articles_data = array();

		// Loop over all items.
		foreach ( $order->get_items() as $item ) {
			/**
			 * Order item data.
			 *
			 * @var WC_Order_Item_Product $item
			 */
			if ( ! $item->is_type( 'line_item' ) ) {
				continue;
			}

			$product = $item->get_product();

			if ( ! $product || ! $product->managing_stock() ) {
				continue;
			}

			$product_sku = $product->get_sku();

			if ( isset( $order_articles_data[ $product_sku ] ) ) {
				continue;
			}

			$stock_data = self::get_article_stock_data( $product_sku );

			if ( is_wp_error( $stock_data ) ) {
				$order->add_order_note( $stock_data->get_error_message() );
				Logger::error( 'Failed to get stock data for article ' . $product_sku . ' in order: ' . $order->get_id() );
				continue;
			}

			$order_articles_data[ $product_sku ] = array(
				'stock_data' => $stock_data,
				'order_item' => $item,
			);
		}

		return $order_articles_data;
	}

	/**
	 * Get article stock data from all connected API:s.
	 *
	 * @param string $product_sku  SKU of article to get stock data for.
	 * @param int[]  $apis_to_skip WP Post ID of API accounts not to get stock data for (usually since it has already been retrieved).
	 *
	 * @return null|WP_Error|Stock_Data_Model[] One of the following:
	 *                                          Array of stock data from all requested Backoffice accounts, if found.
	 *                                          Null if article is not a stock item (in the main Backoffice account).
	 *                                          WP_Error 'stock_data_not_found' if article is not found in any of the Backoffice accounts.
	 */
	public static function get_article_stock_data( string $product_sku, array $apis_to_skip = array() ): null|WP_Error|array {
		$backoffice_accounts = Backoffice_Accounts::get_all_current_accounts();

		$stock_data      = array();
		$main_data_found = false;

		// Loop over all Backoffice accounts.
		foreach ( $backoffice_accounts as $backoffice_account ) {
			// TODO: Check below in MAS reduce stock function.
			// Check if we should skip this account.
			if (
				! empty( $apis_to_skip ) &&
				in_array( $backoffice_account->wp_id, $apis_to_skip, true )
			) {
				if ( $backoffice_account->is_main() ) {
					$main_data_found = true;
				}
				continue;
			}

			// Get article from the current Backoffice Account.
			$article = $backoffice_account->api->get_article( $product_sku );

			// If the current Backoffice account is the main account.
			if ( $backoffice_account->is_main() ) {
				// If we got stock data from the main account, set the flag to true.
				if ( ! is_null( $article ) ) {
					$main_data_found = true;

					// If the article isn't a stock item in the main account.
					if ( ! $article[ NMF_IS_STOCK_ITEM ] ) {
						Logger::debug( 'Article isn\'t stock item in Main account: ' . $product_sku );
						return null;
					}
				}
			}

			$stock_data[] = new Stock_Data_Model(
				array(
					'quantity'              => $article[ NMF_QUANTITY ] ?? 0,
					'backoffice_account_id' => $backoffice_account->wp_id,
					'article_sku'           => $product_sku,
				)
			);
		}

		if ( ! $main_data_found ) {
			Logger::error( 'Failed to get stock data for article ' . $product_sku . ' from main account.' );
			return new WP_Error(
				'stock_data_not_found',
				sprintf(
					// translators: %s: Product SKU.
					__(
						'Failed to get stock data for article %s from main account.',
						'flo-sync'
					),
					$product_sku
				)
			);
		}

		return $stock_data;
	}

	/**
	 * Get stock data stored in WooCommerce for a product.
	 *
	 * @param WC_Product|int $product_id                 WC_Product object or product id.
	 * @param string|null    $sort_by                    Sort by field. Can be property of Stock_Data_Model or Backoffice_Account_Model.
	 * @param bool           $include_backoffice_account Include Hub account in stock data.
	 *
	 * @return Stock_Data_Model[] Array of stock data.
	 */
	public static function get_wc_product_stock_data(
		WC_Product|int $product_id,
		?string $sort_by = null,
		bool $include_backoffice_account = true
	): array {

		if ( ! is_int( $product_id ) ) {
			$product_id = $product_id->get_id();
		}

		if ( ! $product_id ) {
			return array();
		}

		$stock_data = get_post_meta( $product_id, NMF_ARTICLE_STOCK_DATA, true );

		if ( is_array( $stock_data ) ) {
			if ( $include_backoffice_account ) {
				foreach ( $stock_data as $stock_data_item ) {
					$backoffice_account = Backoffice_Accounts::get_account_by_post_id(
						$stock_data_item->backoffice_account_id
					);

					if ( ! is_null( $backoffice_account ) ) {
						$stock_data_item->backoffice_account = $backoffice_account;
					}
				}
			}

			if ( ! empty( $sort_by ) ) {
				usort(
					$stock_data,
					function ( $a, $b ) use ( $sort_by ) {
						if ( isset( $a->$sort_by ) && isset( $b->$sort_by ) ) {
							return $a->$sort_by <=> $b->$sort_by;
						} elseif ( isset( $a->backoffice_account->$sort_by ) && isset( $b->backoffice_account->$sort_by ) ) {
							return $a->backoffice_account->$sort_by <=> $b->backoffice_account->$sort_by;
						}
						return 0;
					}
				);
			}

			return $stock_data;
		} else {
			return array();
		}
	}

	/**
	 * Get stock quantity for the main account.
	 *
	 * @param WC_Product $product Product to get stock quantity for.
	 *
	 * @return null|float
	 */
	public static function get_stock_quantity_for_main_account( WC_Product $product ): ?float {
		if (
			! Backoffice_Accounts::is_multiple_stocks_enabled() ||
			! ( Backoffice_Accounts::get_current_main_account()->wp_id ?? false )
		) {
			return $product->get_stock_quantity();
		}

		return self::get_stock_quantity_from_backoffice_account( $product, Backoffice_Accounts::get_current_main_account()->wp_id );
	}

	/**
	 * Get stock quantity for a Hub account.
	 *
	 * @param WC_Product $product          Product to get stock quantity for.
	 * @param int        $backoffice_account_id Hub account id.
	 * @param float|null $return_on_not_found Return this value if stock data is not found.
	 *
	 * @return float|null
	 */
	public static function get_stock_quantity_from_backoffice_account(
		WC_Product $product,
		int $backoffice_account_id,
		?float $return_on_not_found = 0
	): ?float {
		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return $product->get_stock_quantity();
		}

		$stock_data = self::get_wc_product_stock_data( $product->get_id(), null, false );

		if ( empty( $stock_data ) ) {
			return $return_on_not_found;
		}

		$account_stock_data = array_filter(
			$stock_data,
			function ( $stock_data_item ) use ( $backoffice_account_id ) {
				return $backoffice_account_id === $stock_data_item->backoffice_account_id;
			}
		);

		if ( ! empty( $account_stock_data ) ) {
			$found_stock_data = reset( $account_stock_data );
			return $found_stock_data->quantity;
		} else {
			return $return_on_not_found;
		}
	}
}
