<?php
/**
 * Syncs article stock from API.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Articles\Stock;

use NMFS\Logger;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\Items\Articles\Articles;
use NMFS\Sync\Items\Articles\Stock\Models\Stock_Data_Model;
use NMFS\Sync\Items\Articles\Sync as Article_Sync;
use NMFS\Sync\Abstract_Syncer;

/**
 * Sync class for Stock.
 */
class Sync extends Abstract_Syncer {

	/**
	 * Number of items to get in each API request.
	 *
	 * @var int
	 */
	protected int $page_size = 100;

	/**
	 * Array of articles that stock has already been synced for.
	 *
	 * @var array
	 */
	private array $synced_articles = array();

	/**
	 * Construct class.
	 *
	 * @param array  $backoffice_accounts Array of backoffice accounts to sync stock for.
	 * @param string $type                Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function __construct( array $backoffice_accounts = array(), string $type = 'cron' ) {
		if ( empty( $backoffice_accounts ) ) {
			$backoffice_accounts = Backoffice_Accounts::get_all_current_accounts();
		}
		parent::__construct( $backoffice_accounts, $type );
	}

	/**
	 * Pull articles from API, to sync stock.
	 *
	 * @param array|null $args API request arguments.
	 */
	public function sync( ?array $args = null ): bool {
		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}

		$article_sync = new Article_Sync( array(), $this->get_type() );
		$article_sync->set_current_backoffice_account( $current_backoffice_account );

		$args['custom_callback'] = array( $this, 'process_pulled_article' );

		add_filter(
			'nmf_get_request_response_silent_404',
			array( $this, 'silent_404' )
		);

		$synced = $article_sync->sync( $args );

		if ( ! $synced ) {
			return false;
		}

		$current_status      = $this->get_status();
		$article_sync_status = $article_sync->get_status();

		$status = array();
		foreach ( array_keys( $current_status + $article_sync_status ) as $key ) {
			if (
				'total' === $key ||
				'page' === $key
			) {
				$status[ $key ] = $current_status[ $key ] ?? $article_sync_status[ $key ] ?? 0;
			}
			$status[ $key ] =
				( $current_status[ $key ] ?? 0 ) +
				( $article_sync_status[ $key ] ?? 0 );
		}
		$this->set_status( $status );

		remove_filter(
			'nmf_get_request_response_silent_404',
			array( $this, 'silent_404' )
		);

		return true;
	}

	/**
	 * Update stock data for article being synced.
	 *
	 * @param array $article Retrieved article to update stock data for.
	 *
	 * @return string|null Action, or null on failure.
	 */
	public function process_pulled_article( array $article ): ?string {
		$article     = Articles::transform_article_to_product( $article );
		$article_sku = $article['Sku'] ?? null;

		if ( empty( $article_sku ) ) {
			Logger::error( 'SKU not set for article during stock sync.' );
			return null;
		}

		if ( isset( $this->synced_articles[ $article_sku ] ) ) {
			return null; // Article already synced.
		}

		$this->synced_articles[ $article_sku ] = true;

		if ( ! wc_get_product_id_by_sku( $article_sku ) ) {
			return null; // Article isn't found in WC.
		}

		/**
		 * We don't check the article type or whether it is enabled here since we're only updating stock.
		 * Articles of the incorrect type, or that are disabled, shouldn't exist in WC anyway.
		 */

		$stock_data = array();

		// If article is not a stock item, set stock data to null.
		if ( ! ( $article[ NMF_IS_STOCK_ITEM ] ?? false ) ) {
			$stock_data = null;
		} else {
			$stock_data[] = new Stock_Data_Model(
				array(
					'quantity'              => $article[ NMF_QUANTITY ] ?? 0,
					'backoffice_account_id' => $this->current_backoffice_account->wp_id,
					'article_sku'           => $article_sku,
				)
			);

			if ( Backoffice_Accounts::is_multiple_stocks_enabled() ) {
				// Get stock data from other backoffice accounts.
				// If the main account is among the other accounts checked,
				// and the article is not a stock item there, we will get null.
				// If we're not using Multi-account sync, we will always get an empty array.
				$other_stock_data = Stock::get_article_stock_data(
					$article_sku,
					array( $this->current_backoffice_account->wp_id )
				);

				if ( is_wp_error( $other_stock_data ) ) {
					Logger::error( 'Failed to get stock data for article: ' . $article_sku );
					return null;
				} elseif ( is_null( $other_stock_data ) ) {
					$stock_data = null;
				} else {
					// Merge stock data from all Backoffice accounts.
					$stock_data = array_merge(
						$other_stock_data,
						$stock_data
					);
				}
			}
		}

		$status = Stock::update_article_stock_data( $article_sku, $stock_data );

		return is_int( $status ) ? 'update' : null;
	}

	/**
	 * Disables article save action in article sync loop.
	 *
	 * @param array|null $args API params passed by reference.
	 */
	public function before_sync_loop( ?array &$args ): void {
		Logger::debug( 'Disabling article save action in cron article sync loop' );
		Articles::set_hook_save_article( false );
	}

	/**
	 * Used by filter 'nmf_get_request_response_silent_404'.
	 *
	 * @return true
	 */
	public function silent_404(): bool {
		return true;
	}
}
