<?php
/**
 * Handles view/template changes related to stock.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Articles\Stock;

use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\Items\Articles\Stock\Models\Stock_Data_Model;
use WC_Product;
use WP_Post;

/**
 * Stock view class.
 */
class View {

	/**
	 * Constructor.
	 */
	public static function setup(): void {
		add_action(
			'woocommerce_product_options_inventory_product_data',
			array( self::class, 'admin_add_stock_field' )
		);

		add_action(
			'woocommerce_variation_options_inventory',
			array(
				self::class,
				'admin_add_variation_custom_product_meta_inventory',
			),
			10,
			3
		);

		add_filter(
			'woocommerce_get_stock_html',
			array( self::class, 'stock_html' ),
			10,
			2
		);

		add_filter(
			'woocommerce_get_availability',
			array( self::class, 'product_availability' ),
			10,
			2
		);
	}

	/**
	 * Show stock data on product edit page.
	 */
	public static function admin_add_stock_field(): void {
		global $product_object;

		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() || ! $product_object->get_manage_stock() ) {
			return;
		}

		$product_stock_data = Stock::get_wc_product_stock_data( $product_object, 'display_name' );

		if ( empty( $product_stock_data ) ) {
			return;
		}

		include NMF_PLUGIN_DIR . 'views/admin/admin.wc-article-stock-fields.php';
	}

	/**
	 * Display MAS data on variation edit page.
	 *
	 * @param int     $loop Position in the loop.
	 * @param array   $variation_data Variation data.
	 * @param WP_Post $variation Post data.
	 *
	 * @return void
	 * @noinspection PhpUnusedParameterInspection WP action/filter function.
	 */
	public static function admin_add_variation_custom_product_meta_inventory( int $loop, array $variation_data, WP_Post $variation ): void {
		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return;
		}

		$product_stock_data = Stock::get_wc_product_stock_data( $variation->ID, 'display_name' );

		if ( empty( $product_stock_data ) ) {
			return;
		}

		include NMF_PLUGIN_DIR . 'views/admin/admin.wc-article-stock-fields.php';
	}

	/**
	 * Filter stock html.
	 *
	 * @param string     $html Current stock html.
	 * @param WC_Product $product WC_Product object.
	 *
	 * @noinspection PhpUnusedParameterInspection WP action/filter function.
	 */
	public static function stock_html( string $html, WC_Product $product ): string {
		$availability = $product->get_availability();

		return $availability['availability'];
	}

	/**
	 * Filter product availability.
	 *
	 * @param array      $availability Current product availability.
	 * @param WC_Product $product WC_Product object.
	 */
	public static function product_availability( array $availability, WC_Product $product ): array {
		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return $availability;
		}

		$availability['class'] = 'in-stock';

		$availability['availability'] = self::render_mas_product_availability( $product );

		return $availability;
	}

	/**
	 * Render multi-account sync product availability.
	 * TODO: Also modify order emails to include backorder information.
	 *
	 * @param WC_Product $product WC_Product object.
	 *
	 * @return string HTML for rendered availability.
	 */
	public static function render_mas_product_availability( WC_Product $product ): string {

		if ( ! Backoffice_Accounts::is_multiple_stocks_enabled() ) {
			return '';
		}

		$product_stock_data   = Stock::get_wc_product_stock_data( $product, 'display_name' );
		$is_backorder_allowed = $product->get_backorders() === 'yes' || $product->get_backorders() === 'notify';

		if ( empty( $product_stock_data ) ) {
			return '';
		}

		ob_start();
		echo '<h2>' . esc_html__( 'Stock', 'flo-sync' ) . '</h2>';
		echo '<ul class="nmf-single-product-stock">';
		foreach ( $product_stock_data as $stock_data ) {
			if ( ! isset( $stock_data->backoffice_account ) ) {
				continue;
			}
			$out_of_stock = $stock_data->quantity < 1;

			if ( $out_of_stock && ! $is_backorder_allowed ) {
				$class         = 'nmf-out-of-stock';
				$quantity_text = __( 'Out of stock', 'woocommerce' );

			} elseif ( $product->managing_stock() && $out_of_stock ) {
				$quantity_text = $product->backorders_require_notification() ? __( 'Available on backorder', 'woocommerce' ) : __( 'In stock', 'flo-sync' );
				$class         = 'available-on-backorder';
			} elseif ( ! $product->managing_stock() && $out_of_stock ) {
				$quantity_text = __( 'Available on backorder', 'woocommerce' );
				$class         = 'available-on-backorder';
			} elseif ( $product->managing_stock() ) {
				$quantity_text = wc_format_stock_quantity_for_display( $stock_data->quantity, $product );
				$class         = 'in-stock';
			} else {
				$quantity_text = __( 'In stock', 'flo-sync' );
				$class         = 'in-stock';
			}

			echo '<li class="' . esc_attr( $class ) . '">';
			echo esc_html( $stock_data->backoffice_account->get_display_name() ) . ' <span class="availability-text">(' . esc_html( $quantity_text ) . ')</span>';
			echo '</li>';
		}
		echo '</ul>';

		$html = ob_get_clean();

		/**
		 * Filter the multi-account sync product availability html.
		 *
		 * @param string             $html Current stock html.
		 * @param Stock_Data_Model[] $product_stock_data Product stock data.
		 * @param WC_Product         $product WC_Product object.
		 */
		return apply_filters( 'nmf_mas_product_availability', $html, $product_stock_data, $product );
	}
}
