<?php
/**
 * Handles syncing of articles.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Articles;

use NMFS\Logger;
use NMFS\Settings;
use NMFS\Sync\Items\Articles\Models\Article_Model;
use NMFS\Sync\Abstract_Syncer;
use NMFS\Sync\Items\Articles\Articles as Article_Functions;

defined( 'ABSPATH' ) || exit;

/**
 * Class Sync
 */
class Sync extends Abstract_Syncer {

	/**
	 * Pull articles from API.
	 *
	 * @param array|null $args Arguments to pass to API, or null if no arguments.
	 */
	public function sync( ?array $args = null ): bool {
		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}

		$articles = $current_backoffice_account->api->get_articles( $args );

		if (
			! isset( $articles['Total'] ) ||
			! isset( $articles['Page'] ) ||
			! isset( $articles['list'] )
		) {
			return false;
		}

		$this->update_status( 'page', $articles['Page'] );
		$this->update_status( 'total', $articles['Total'] );

		if ( is_array( $articles['list'] ) ) {
			foreach ( $articles['list'] as $article ) {

				if ( ! empty( $article['Article'] ) ) {

					if ( is_callable( $args['custom_callback'] ?? false ) ) {
						$action = call_user_func( $args['custom_callback'], $article['Article'] );
					} else {
						$action = $this->process_pulled_article( $article['Article'] );
					}

					if ( $action ) {
						$this->increase_status( $action );
					}
				}
			}
		}

		return true;
	}

	/**
	 * Process article from API.
	 *
	 * @param array $article Article.
	 *
	 * @return string|null Action, or null on failure.
	 */
	public function process_pulled_article( array $article ): ?string {
		$article = Article_Functions::transform_article_to_product( $article );

		$sku            = $article['Sku'];
		$mf_is_web_shop = $article[ NFM_IS_WEB_SHOP_ARTICLE ];
		$mf_is_active   = $article[ NMF_IS_ACTIVE_ARTICLE ];

		Logger::debug( 'Processing article no: ' . $sku );

		$current_backoffice_account = $this->get_current_backoffice_account();

		if (
			$current_backoffice_account &&
			$sku === $current_backoffice_account->get_shipping_settings()->get_article_sku()
		) {
			Logger::debug( 'Skipping due to is shipping article.' );
			return null;
		}

		$product_id = wc_get_product_id_by_sku( $sku );

		if (
			! $mf_is_web_shop ||
			NMF_TYPE_GOODS !== $article[ NMF_ARTICLE_TYPE ]
		) {
			// Skip product if it isn't a Web shop article, or if it isn't a goods article.
			Logger::debug( 'Skipping due to !webshop/!ArticleType.' );

			// Maybe set product status to draft if it already exists.
			Article_Functions::maybe_set_as_draft( $product_id );

			return null;
		}

		$article_instance = new Article_Model();

		if ( empty( $product_id ) ) {
			if (
				NMF_ARTICLE_PUBLISH !== $mf_is_active
			) {
				// Skip product if it isn't active in Hub.
				Logger::debug( 'Skipping due to !active' );
				Logger::debug( print_r( $article, true ) ); // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r

				// Maybe set product status to draft if it already exists.
				Article_Functions::maybe_set_as_draft( $product_id );

				return null;
			}
			$product_id = $article_instance->add( $article );

			if ( empty( $product_id ) ) {
				Logger::error( 'Failed to create product: ' . $sku );
				return null;
			}

			$return = 'add';
			Logger::debug( 'Product created in WC: ' . $sku );
		} else {
			/**
			 * TODO: if customer requested update product to
			 * draft what ever article are published to draft based on !active
			 */
			update_post_meta( $product_id, 'nmf_temp_lock', 'yes' );
			$article_instance->update( $product_id, $article );
			$return = 'update';
			Logger::debug( wp_json_encode( $article ) );
			Logger::debug( 'Product updated in WC: ' . $sku );
			delete_post_meta( $product_id, 'nmf_temp_lock', 'yes' );
		}

		Articles::update_product_sync_status( $product_id );

		Logger::debug( 'Processed article no: ' . $sku );

		return $return;
	}

	/**
	 * Disables article save action in article sync loop.
	 *
	 * @param array|null $args API params passed by reference.
	 */
	public function before_sync_loop( ?array &$args ): void {
		Logger::debug( 'Disabling article save action in article sync loop' );
		Articles::set_hook_save_article( false );
	}

	/**
	 * Checks if sync is enabled.
	 *
	 * @return bool
	 */
	public function is_enabled(): bool {
		if ( Settings::is_setting_enabled( 'sync_only_product_stock_amount' ) ) {
			Logger::debug( 'Sync only product stock amount is enabled, skipping article sync' );
			return false;
		}
		return true;
	}
}
