<?php
/**
 * Main code for account code sync.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Bookkeeping\Article_Account_Codes;

use NMFS\Logger;
use NMFS\Sync\Items\Bookkeeping\Article_Account_Codes\Models\Article_Account_Code_Model;
use Throwable;

/**
 * Class for Account code sync.
 */
class Article_Account_Codes {

	/**
	 * All account codes.
	 *
	 * @var Article_Account_Code_Model[]
	 */
	private static array $article_account_codes;

	/**
	 * Get all account codes.
	 *
	 * @return Article_Account_Code_Model[]
	 */
	public static function get_all(): array {
		if ( ! isset( self::$article_account_codes ) ) {
			try {
				$options = get_option( 'nmf_article_account_codes', array() );
			} catch ( Throwable $e ) {
				$options = array();
				Logger::error( 'Error getting article account codes from database: ' . $e->getMessage() );
				self::clean();
			}
			self::$article_account_codes = $options;
		}
		return self::$article_account_codes;
	}

	/**
	 * Update account codes in DB.
	 *
	 * @param Article_Account_Code_Model[] $nmf_tax_classes The account codes.
	 * @return void
	 */
	public static function update( array $nmf_tax_classes ): void {
		self::clean();
		update_option( 'nmf_article_account_codes', $nmf_tax_classes );
	}

	/**
	 * Remove all account codes from DB.
	 */
	public static function clean(): void {
		delete_option( 'nmf_article_account_codes' );
	}

	/**
	 * Get account codes, mainly to use in select input.
	 *
	 * @param bool $include_empty_option Whether to include an empty option with select text.
	 *
	 * @return array
	 */
	public static function get_select_options( bool $include_empty_option = true ): array {
		$return                = array();
		$article_account_codes = self::get_all();

		if ( $include_empty_option ) {
			$return['0'] = __( 'Please select an account code.', 'flo-sync' );
		}

		if ( empty( $article_account_codes ) ) {
			return $return;
		}

		foreach ( $article_account_codes as $article_account_code ) {
			if ( ! empty( $article_account_code->get_id() ) && ! empty( $article_account_code->get_name() ) ) {
				$return[ $article_account_code->get_id() ] = $article_account_code->get_name();
			}
		}

		return $return;
	}
}
