<?php
/**
 * Sync account codes from Hub main account to WooCommerce.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Bookkeeping\Article_Account_Codes;

use NMFS\DB_Helpers;
use NMFS\Settings;
use NMFS\Sync\Abstract_Syncer;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Envs;
use NMFS\Sync\Items\Bookkeeping\Article_Account_Codes\Models\Article_Account_Code_Model;

/**
 * Class Sync for account codes.
 */
class Sync extends Abstract_Syncer {

	/**
	 * Page size will not be changed by ajax (or any other) request.
	 *
	 * @var bool $force_page_size
	 */
	protected bool $force_page_size = true;

	/**
	 * Sync request page size.
	 *
	 * @var int $page_size
	 */
	protected int $page_size = 1000;

	/**
	 * Construct class.
	 *
	 * @param array  $backoffice_accounts Array of Hub accounts to sync stock for.
	 * @param string $type                Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function __construct( array $backoffice_accounts = array(), string $type = 'cron' ) {
		if ( empty( $backoffice_accounts ) ) {
			$backoffice_accounts = Backoffice_Accounts::get_all_current_accounts();
		}
		parent::__construct( $backoffice_accounts, $type );
	}

	/**
	 * Sync account codes from Hub WooCommerce.
	 *
	 * @param array|null $args API arguments.
	 */
	public function sync( ?array $args = null ): bool {

		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}

		$data = $current_backoffice_account->api->get_account_codes( $args );

		if ( ! $data || ! isset( $data->list ) || ! is_array( $data->list ) ) {
			return false;
		}

		$this->update_status( 'page', $data->Page );
		$this->update_status( 'total', $data->Total );

		$nmf_tax_classes = Factory::create_array_from_api( $data->list );

		// TODO: Maybe change the 'update' key of $status?
		Backoffice_Accounts::update_account_property(
			$current_backoffice_account->wp_id,
			'article_account_codes',
			$nmf_tax_classes
		);

		if (
			$current_backoffice_account->is_main() &&
			Envs::get_current_env() === $current_backoffice_account->env
		) {
			$this->update_wc_tax_rates( $data, $nmf_tax_classes );
		}

		return true;
	}

	/**
	 * Update WooCommerce tax classes and tax rates from API response.
	 * These should be updated only from the main account.
	 * Used for article/order taxes. NOT for bookkeeping account codes.
	 *
	 * @param object                       $response The response from the main API.
	 * @param Article_Account_Code_Model[] $nmf_tax_classes The account codes.
	 */
	private function update_wc_tax_rates( object $response, array $nmf_tax_classes ): void {

		Article_Account_Codes::update( $nmf_tax_classes );

		global $wpdb;
		$wc_tax_rate_cls = array();
		$wc_tax_rates    = array();
		$mf_wc_tax_rules = array();

		if ( Settings::is_setting_enabled( 'is_tax_class_active' ) ) {
			$option_tax_tables = array(
				'mf_tax_rates_cls' => array( 'tax_rate_class_id', 'wc_tax_rate_classes' ),
				'mf_tax_rates_ids' => array( 'tax_rate_id', 'woocommerce_tax_rates' ),
			);
			foreach ( $option_tax_tables as $wp_option_name => $table_details ) {
				$mf_option_result = get_option( $wp_option_name );
				if ( ! empty( $mf_option_result ) ) {
					foreach ( $mf_option_result as $id ) {
						$wpdb->delete(
							"$wpdb->prefix$table_details[1]",
							array( $table_details[0] => $id ),
							array( '%d' )
						);
					}
				}
			}

			foreach ( $response->list as $list_item ) {
				$tax_class_name      = $list_item->AccountCode->Name;
				$tax_class_rate      = $list_item->AccountCode->VatRate;
				$tax_class_rate_name = $list_item->AccountCode->Description;
				$tax_class_rate_id   = $list_item->AccountCode->Id;
				$tax_class_slug      = sanitize_title( str_replace( ' ', '-', strtolower( $tax_class_name ) ) );

				$tbl_wc_tax_rate_cls  = $wpdb->prefix . 'wc_tax_rate_classes';
				$wc_tax_rate_cls_data = array(
					'name' => 'MF ' . $tax_class_name,
					'slug' => 'mf-' . $tax_class_slug,
				);

				$wc_tax_rate_cls_format = array( '%s', '%s' );

				$tbl_wc_tax_rates    = $wpdb->prefix . 'woocommerce_tax_rates';
				$wc_tax_rates_data   = array(
					'tax_rate_country'  => '',
					'tax_rate_state'    => '',
					'tax_rate'          => $tax_class_rate . '.0000',
					'tax_rate_name'     => $tax_class_rate_name,
					'tax_rate_priority' => 1,
					'tax_rate_compound' => 0,
					'tax_rate_shipping' => 1,
					'tax_rate_order'    => 0,
					'tax_rate_class'    => 'mf-' . $tax_class_slug,
				);
				$wc_tax_rates_format = array( '%s', '%s', '%s', '%s', '%d', '%d', '%d', '%d', '%s' );

				$insert_wc_tax_rate_classes_id = DB_Helpers::insert_row( $tbl_wc_tax_rate_cls, $wc_tax_rate_cls_data, $wc_tax_rate_cls_format );
				if ( $insert_wc_tax_rate_classes_id ) {
					$wc_tax_rate_cls[] = $insert_wc_tax_rate_classes_id;
				}
				$insert_wc_tax_rate_id = DB_Helpers::insert_row( $tbl_wc_tax_rates, $wc_tax_rates_data, $wc_tax_rates_format );
				if ( $insert_wc_tax_rate_id ) {
					$mf_wc_tax_rules[ $tax_class_rate_id ] = array( 'mf-' . $tax_class_slug, $tax_class_rate );
					$wc_tax_rates[]                        = $insert_wc_tax_rate_id;
				}
			}
			if ( ! empty( $wc_tax_rate_cls ) ) {
				update_option( 'mf_tax_rates_cls', $wc_tax_rate_cls );
			}
			if ( ! empty( $wc_tax_rates ) ) {
				update_option( 'mf_tax_rates_ids', $wc_tax_rates );
			}
			if ( ! empty( $mf_wc_tax_rules ) ) {
				update_option( 'mf_wc_tax_rules', $mf_wc_tax_rules );
			}
		}
	}
}
