<?php
/**
 * Syncer class for income account codes.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Bookkeeping\Income_Account_Codes;

use NMFS\Logger;
use NMFS\Settings;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\Items\Bookkeeping\Income_Account_Codes\Models\Income_Account_Code_Model;
use NMFS\Sync\Items\Orders\Receipts\Helpers as Receipt_Helpers;
use NMFS\Sync\Abstract_Syncer;

defined( 'ABSPATH' ) || exit;

/**
 * Syncer class for income account codes.
 */
class Sync extends Abstract_Syncer {

	/**
	 * Page size will not be changed by ajax (or any other) request.
	 *
	 * @var bool $force_page_size
	 */
	protected bool $force_page_size = true;

	/**
	 * Sync request page size.
	 *
	 * @var int $page_size
	 */
	protected int $page_size = 1000;

	/**
	 * PaymentSource for account codes to get for this sync.
	 *
	 * @var string $api_payment_source
	 */
	private static string $api_payment_source = 'MoreFloOnlineSolution';

	/**
	 * PaymentSourceArg1 for account codes to get for this sync.
	 *
	 * @var string $api_payment_source_arg_1
	 */
	private static string $api_payment_source_arg_1 = 'webshop';

	/**
	 * Construct class.
	 *
	 * @param array  $backoffice_accounts Array of Hub accounts to sync stock for.
	 * @param string $type                Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function __construct( array $backoffice_accounts = array(), string $type = 'cron' ) {
		if ( empty( $backoffice_accounts ) ) {
			$backoffice_accounts = Backoffice_Accounts::get_all();
		}
		parent::__construct( $backoffice_accounts, $type );
	}

	/**
	 * Sync income account codes.
	 *
	 * @param array|null $args Arguments to pass to API, or null if no arguments.
	 */
	public function sync( ?array $args = null ): bool {
		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}

		$result_object = $current_backoffice_account->api->get_income_account_codes( $args );

		if ( ! isset( $result_object->list[0] ) ) {
			Logger::notice( 'No income account codes found' );
			Logger::notice( $result_object );

			return true; // This isn't necessarily an error, so we return true.
		}

		$this->update_status( 'page', $result_object->Page );
		$this->update_status( 'total', $result_object->Total );

		$income_account_codes = $result_object->list;

		/**
		 * Filter the API Payment Source to look for.
		 * Should return a string.
		 */
		$api_payment_source = apply_filters( 'nmf_sync_income_account_codes_api_payment_source', self::$api_payment_source );

		/**
		 * Whether to also check include income account codes received for PaymentSource "General"
		 * Should return a boolean.
		 */
		$include_general_payment_source = apply_filters(
			'northmill_booking_income_account_codes_api_payment_source_include_general',
			true
		);

		/**
		 * Filter the API Payment Source Arg 1 to look for.
		 * Should return a string.
		 */
		$api_payment_source_arg_1 = apply_filters( 'nmf_sync_income_account_codes_api_payment_source_arg_1', self::$api_payment_source_arg_1 );

		$payment_gateway_collections = Receipt_Helpers::get_payment_methods_creating_hub_receipts( false );

		// Reset counters.
		$match_count = 0;

		// Step 1: Create an associative array with 'iac_pay_type' as the key and an array of payment methods as the value.
		$method_map = array();
		foreach ( $payment_gateway_collections as $payment_gateway_collection ) {
			$method_map[ $payment_gateway_collection->iac_pay_type ][] = $payment_gateway_collection;
		}

		// Step 2: Filter income account codes based on 'Payment source' and 'Payment argument'.
		$initial_count          = count( $income_account_codes );
		$filtered_account_codes = array_filter(
			$income_account_codes,
			function ( $account_code ) use ( $api_payment_source, $api_payment_source_arg_1, $include_general_payment_source ) {
				return (
					(
						$account_code->IncomeAccountSetting->PaymentSource === $api_payment_source ||
						(
							$include_general_payment_source &&
							'General' === $account_code->IncomeAccountSetting->PaymentSource
						)
					) &&
					$account_code->IncomeAccountSetting->PaymentSourceArg1 === $api_payment_source_arg_1
				);
			}
		);

		// Step 3: Find matching income account codes by 'Payment Type'.
		$matching_account_codes   = array();
		$updated_payment_gateways = array();
		foreach ( $filtered_account_codes as $account_code ) {
			if ( array_key_exists( $account_code->IncomeAccountSetting->PaymentType, $method_map ) ) {
				foreach ( $method_map[ $account_code->IncomeAccountSetting->PaymentType ] as $payment_gateway_collection ) {
					if (
						'General' === $account_code->IncomeAccountSetting->PaymentSource &&
						in_array( $payment_gateway_collection->payment_gateway->id, $updated_payment_gateways, true )
					) {
						continue;
					}

					$account_code_value = Settings::maybe_get_old_setting_value(
						"nmf_income_account_code_{$payment_gateway_collection->payment_gateway->id}_%s_from_pre_3.0.0",
						$account_code->IncomeAccountSetting->AccountNumber
					);

					// Step 4: Add to the result if there's a match.
					$matching_account_codes[ $payment_gateway_collection->payment_gateway->id ] = new Income_Account_Code_Model(
						array(
							'id'                => $account_code->IncomeAccountSetting->Id,
							'payment_method_id' => $payment_gateway_collection->payment_gateway->id,
							'account_code'      => $account_code_value,
						)
					);

					++$match_count;
					$updated_payment_gateways[] = $payment_gateway_collection->payment_gateway->id;
				}
			}
		}

		$filtered_out_count = $initial_count - $match_count;

		$this->update_status( 'update', $match_count );
		$this->update_status( 'ignore', $filtered_out_count );

		Backoffice_Accounts::update_account_property(
			$current_backoffice_account->wp_id,
			'income_account_codes',
			$matching_account_codes
		);

		return true;
	}
}
