<?php
/**
 * Handles syncing of bookkeeping vat accounts.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Bookkeeping\Vat_Account_Codes;

use NMFS\Logger;
use NMFS\Sync\Abstract_Syncer;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\Items\Bookkeeping\Vat_Account_Codes\Models\Vat_Account_Code_Model;

defined( 'ABSPATH' ) || exit;

/**
 * Class Sync
 */
class Sync extends Abstract_Syncer {

	/**
	 * Page size will not be changed by ajax (or any other) request.
	 *
	 * @var bool $force_page_size
	 */
	protected bool $force_page_size = true;

	/**
	 * Sync request page size.
	 *
	 * @var int $page_size
	 */
	protected int $page_size = 1000;

	/**
	 * Construct class.
	 *
	 * @param array  $backoffice_accounts Array of Hub accounts to sync stock for.
	 * @param string $type                Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function __construct( array $backoffice_accounts = array(), string $type = 'cron' ) {
		if ( empty( $backoffice_accounts ) ) {
			$backoffice_accounts = Backoffice_Accounts::get_all();
		}
		parent::__construct( $backoffice_accounts, $type );
	}

	/**
	 * Syncs bookkeeping vat accounts to WooCommerce
	 *
	 * @param array|null $args Arguments to pass to API, or null if no arguments.
	 */
	public function sync( ?array $args = null ): bool {
		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}

		$vat_accounts = $current_backoffice_account->api->get_vat_accounts( $args );

		if (
			! isset( $vat_accounts['list'] ) ||
			! isset( $vat_accounts['Page'] ) ||
			! isset( $vat_accounts['Total'] )
		) {
			Logger::error( 'No income account codes found' );
			Logger::error( $vat_accounts );

			return true; // This isn't necessarily an error, so we return true.
		}

		$this->update_status( 'page', $vat_accounts['Page'] );
		$this->update_status( 'total', $vat_accounts['Total'] );

		$account_codes = array();

		if ( is_array( $vat_accounts['list'] ) ) {
			foreach ( $vat_accounts['list'] as $item ) {
				if ( empty( $item['VatAccountSetting'] ) ) {
					$this->increase_status( 'error' );
					continue;
				}

				$account_code = isset( $item['VatAccountSetting']['AccountNumber'] ) ? intval( $item['VatAccountSetting']['AccountNumber'] ) : null;
				$var_rate     = isset( $item['VatAccountSetting']['VatRate'] ) ? intval( $item['VatAccountSetting']['VatRate'] ) : null;

				$account_codes[ $var_rate ] = new Vat_Account_Code_Model(
					array(
						'id'           => (string) $item['VatAccountSetting']['Id'],
						'account_code' => $account_code,
						'vat_rate'     => $var_rate,
					)
				);

				$this->increase_status( 'update' );
			}

			Backoffice_Accounts::update_account_property(
				$current_backoffice_account->wp_id,
				'vat_account_codes',
				$account_codes
			);

		}

		return true;
	}
}
