<?php

namespace NMFS\Sync\Items\Campaigns\Models;

use Automattic\WooCommerce\Utilities\NumberUtil;
use WC_Coupon;

defined( 'ABSPATH' ) || exit;

class Discounter_Type_2_Model implements Discounter_Interface {
	/**
	 * @param WC_Coupon $coupon
	 * @param string[]  $skus
	 * @param int       $x
	 * @param int       $y
	 * @param int       $priority
	 * @param int       $threshold_total
	 */
	public function __construct( WC_Coupon $coupon, array $skus, int $x, int $y, int $priority, int $threshold_total ) {
		$this->coupon          = $coupon;
		$this->skus            = $skus;
		$this->x               = $x;
		$this->y               = $y;
		$this->priority        = $priority;
		$this->threshold_total = $threshold_total;
	}

	public function get_name(): string {
		return __( 'Buy X amount and pay Y', 'flo-sync' );
	}

	public function get_coupon(): WC_Coupon {
		return $this->coupon;
	}

	public function get_priority(): int {
		return $this->priority;
	}

	public function is_applicable( float $cart_total, array $products ): bool {
		if ( $cart_total < $this->threshold_total ) {
			return false;
		}

		$applicable_products = $this->applicable_products( $products );

		if ( count( $applicable_products ) < $this->x ) {
			return false;
		}

		foreach ( $applicable_products as $applicable_product ) {
			$applicable_product->add_use();
		}

		return true;
	}

	public function get_discounted_products( array $products, int $cart_total ): array {
		return $this->_get_discounted_products(
			$this->applicable_products( $products ),
			$cart_total
		);
	}

	protected function applicable_products( $products ): array {
		$skus = $this->skus;

		return array_filter(
			$products,
			function ( $product ) use ( $skus ) {
				if ( $product->get_uses() > 0 ) {
					return false;
				}

				return in_array( $product->get_sku(), $skus );
			}
		);
	}

	/**
	 * @param Product_Model[] $products
	 * @param int             $maximum_allowed_discount
	 *
	 * @return Product_Model[]
	 */
	protected function _get_discounted_products( array $products, int $maximum_allowed_discount ): array {
		if ( count( $products ) < $this->x ) {
			return array();
		}

		$discounted_products = array_splice( $products, - $this->x, $this->x ); // Remove x most expensive products

		if ( $maximum_allowed_discount >= $this->y ) {
			$discount = $this->y / count( $discounted_products );
			foreach ( $discounted_products as $i ) {
				// Calculate discount per product
				$i->set_discount( NumberUtil::round( $i->get_price() - $discount, wc_get_rounding_precision() ) );
			}
		}

		if ( empty( $products ) ) {
			return array_merge(
				$discounted_products
			);
		}

		return array_merge(
			$discounted_products,
			$this->_get_discounted_products( $products, $maximum_allowed_discount )
		);
	}

	/** @var WC_Coupon WooCommerce coupon for this discount */
	protected WC_Coupon $coupon;
	/** @var string[] All skus valid for this discount */
	protected array $skus;
	/** @var int Value X (amount needed to buy) */
	protected int $x;
	/** @var int Value Y (amount with discount if X is fulfilled) */
	protected int $y;
	/** @var int Priority */
	protected int $priority;
	/** @var int Cart sum needed to activate this discount */
	protected int $threshold_total;
}
