<?php

namespace NMFS\Sync\Items\Campaigns\Models;

use Automattic\WooCommerce\Utilities\NumberUtil;
use WC_Coupon;

defined( 'ABSPATH' ) || exit;

class Discounter_Type_5_Model implements Discounter_Interface {
	/**
	 * @param WC_Coupon $coupon
	 * @param string[]  $skus
	 * @param object    $values
	 * @param int       $priority
	 * @param int       $threshold_total
	 */
	public function __construct( WC_Coupon $coupon, array $skus, object $values, int $priority, int $threshold_total ) {
		$this->coupon          = $coupon;
		$this->skus            = $skus;
		$this->values          = $values;
		$this->priority        = $priority;
		$this->threshold_total = $threshold_total;
	}

	public function get_name(): string {
		return __( 'Price Reduction', 'flo-sync' );
	}

	public function get_coupon(): WC_Coupon {
		return $this->coupon;
	}

	public function get_priority(): int {
		return $this->priority;
	}

	public function is_applicable( float $cart_total, array $products ): bool {
		if ( $cart_total < $this->threshold_total ) {
			return false;
		}

		$applicable_products = $this->applicable_products( $products );

		if ( count( $applicable_products ) < 1 ) {
			return false;
		}

		foreach ( $applicable_products as $applicable_product ) {
			$applicable_product->add_use();
		}

		return true;
	}

	public function get_discounted_products( array $products, int $cart_total ): array {
		return $this->_get_discounted_products(
			$this->applicable_products( $products ),
			$cart_total
		);
	}

	protected function applicable_products( $products ): array {
		$skus = $this->skus;

		return array_filter(
			$products,
			function ( $product ) use ( $skus ) {
				if ( $product->get_uses() > 0 ) {
					return false;
				}

				return in_array( $product->get_sku(), $skus, true );
			}
		);
	}

	/**
	 * @param Product_Model[] $products
	 * @param int             $maximum_allowed_discount
	 *
	 * @return Product_Model[]
	 */
	protected function _get_discounted_products( array $products, int $maximum_allowed_discount ): array {
		if ( empty( $products ) ) {
			return array();
		}

		foreach ( $products as $product ) {
			$discount = NumberUtil::round(
				$this->get_value_of_sku( $product->get_sku() ),
				wc_get_rounding_precision()
			);

			if ( $maximum_allowed_discount >= $discount ) {
				$product->set_discount( $discount );
				$maximum_allowed_discount -= $discount;
			}
		}

		return $products;
	}

	/**
	 * Gets value of sku from values array
	 *
	 * @param string $sku
	 *
	 * @return float
	 */
	protected function get_value_of_sku( string $sku ): float {
		if ( property_exists( $this->values, $sku ) ) {
			return floatval( $this->values->{$sku} );
		}

		return 0.0;
	}

	/** @var WC_Coupon WooCommerce coupon for this discount */
	protected WC_Coupon $coupon;
	/** @var string[] All skus valid for this discount */
	protected array $skus;
	/** @var object SKU values */
	protected object $values;
	/** @var int Priority */
	protected int $priority;
	/** @var int Cart sum needed to activate this discount */
	protected int $threshold_total;
}
