<?php
/**
 * Syncs groups to WooCommerce as product categories
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Groups;

use NMFS\Sync\Abstract_Syncer_And_Cleaner;
use NMFS\Logger;

defined( 'ABSPATH' ) || exit;

/**
 * Class Sync
 */
class Sync extends Abstract_Syncer_And_Cleaner {

	/**
	 * Number of items to get in each API request.
	 *
	 * @var int
	 */
	protected int $page_size = 1000;


	/**
	 * Check if sync is enabled.
	 *
	 * @return bool
	 */
	public function is_enabled(): bool {
		return Groups::is_enabled();
	}

	/**
	 * Syncs groups to WooCommerce as product categories
	 *
	 * @param array|null $args Arguments to pass to API, or null if no arguments.
	 */
	public function sync( ?array $args = null ): bool {
		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}
		$groups = $current_backoffice_account->api->get_groups( $args );

		if (
			! isset( $groups['Total'] ) ||
			! isset( $groups['Page'] ) ||
			! isset( $groups['list'] )
		) {
			return false;
		}

		$this->update_status( 'page', $groups['Page'] );
		$this->update_status( 'total', $groups['Total'] );

		if ( is_array( $groups['list'] ) ) {
			$this->process_pulled_groups( $groups['list'] );
		}

		return true;
	}

	/**
	 * Process groups pulled from API
	 *
	 * @param array $api_list List of groups.
	 *
	 * @return void
	 */
	public function process_pulled_groups( array $api_list ): void {
		$api_list = $this->order_by_hierarchy( $api_list );

		if ( empty( $api_list ) ) {
			return;
		}

		foreach ( $api_list as $group ) {
			if ( empty( $group['Group'] ) || ! is_array( $group['Group'] ) ) {
				continue;
			}

			$action = $this->process_pulled_group( $group['Group'] );

			if ( '' !== $action ) {
				$this->increase_status( $action );
			}
		}

		set_transient( 'nmf_last_group_sync_loop', time(), MINUTE_IN_SECONDS );
		update_option( 'mf_sync_group_last_synced', time() );
	}

	/**
	 * Process a single group pulled from API
	 *
	 * @param array $group Group.
	 *
	 * @return string
	 */
	public function process_pulled_group( array $group ): string {
		Logger::debug( 'Processing group: ' . $group['Id'] );

		$term_id = null;
		$args    = array(
			'parent' => 0,
			'name'   => $group['Name'],
		);

		$current_terms = get_terms(
			array(
				'taxonomy'   => 'product_cat',
				'hide_empty' => false,
				'meta_query' => array(
					array(
						'key'   => 'mf_group_id',
						'value' => $group['Id'],
					),
				),
			)
		);

		if ( ! empty( $current_terms ) ) {
			$term_id = $current_terms[0]->term_id;
		}

		if ( $group['ParentId'] ) {
			$parent_terms = get_terms(
				array(
					'taxonomy'   => 'product_cat',
					'hide_empty' => false,
					'meta_query' => array(
						array(
							'key'   => 'mf_group_id',
							'value' => $group['ParentId'],
						),
					),
				)
			);

			if ( ! empty( $parent_terms ) ) {
				$args['parent'] = $parent_terms[0]->term_id;
			}
		}

		if ( is_null( $term_id ) ) {
			$term_parent = $args['parent'] ?? null;
			$term_id     = term_exists( $group['Name'], 'product_cat', $term_parent );
			if ( ! is_null( $term_id ) ) {
				$term_id = $term_id['term_id'];
				$return  = 'update';
				Logger::debug( 'Re-using WC group ' . $term_id . ' for ' . $group['Id'] );
			} else {
				$insert_result = wp_insert_term( $group['Name'], 'product_cat', $args );
				if ( is_wp_error( $insert_result ) ) {
					Logger::error( 'Error creating WC group (' . $group['Id'] . ') : ' . $insert_result->get_error_message() );

					return '';
				}

				$term_id = $insert_result['term_id'];
				$return  = 'add';
			}

			Logger::debug( 'Group created in WC: ' . $group['Id'] );
		} else {
			wp_update_term( $term_id, 'product_cat', $args );
			$return = 'update';

			Logger::debug( 'Group updated in WC: ' . $group['Id'] );
		}

		update_term_meta( $term_id, 'order', $group['DisplayOrder'] );
		update_term_meta( $term_id, 'mf_group_id', $group['Id'] );
		update_term_meta( $term_id, 'mf_last_updated', time() );

		Logger::debug( 'Processed group no: ' . $group['Id'] );

		return $return;
	}

	/**
	 * Clean old groups.
	 *
	 * @return void
	 */
	public function clean(): void {
		if ( get_transient( 'nmf_last_group_sync_loop' ) ) {
			// Already cleaned.
			Logger::debug( 'Skipped clean groups due to skipped group import' );

			return;
		}

		Logger::debug( 'Processing clean groups' );

		$outdated_terms = get_terms(
			array(
				'taxonomy'   => 'product_cat',
				'hide_empty' => false,
				'fields'     => 'ids',
				'meta_query' => array(
					'relation' => 'AND',
					array(
						'key'     => 'mf_group_id',
						'compare' => 'EXISTS',
					),
					array(
						'key'     => 'mf_last_updated',
						'value'   => time() - 5 * MINUTE_IN_SECONDS,
						'compare' => '<',
					),
				),
			)
		);

		if ( ! empty( $outdated_terms ) ) {
			foreach ( $outdated_terms as $term_id ) {
				$mf_group_id = get_term_meta( $term_id, 'mf_group_id', true );
				wp_delete_term( $term_id, 'product_cat' );
				Logger::debug( 'Deleted product term: ' . $mf_group_id );
			}
		}

		Logger::debug( 'Processed clean groups' );
	}

	/**
	 * Orders a group list with parents first
	 *
	 * @param array $api_list List of groups.
	 *
	 * @return array Ordered list
	 */
	public function order_by_hierarchy( array $api_list ): array {
		$ordered_list = $this->do_order_by_hierarchy( $api_list );

		$ordered_list_ids = array_map(
			function ( $i ) {
				return $i['Group']['Id'];
			},
			$ordered_list
		);

		foreach ( $api_list as $item ) {
			if ( in_array( $item['Group']['Id'], $ordered_list_ids, true ) ) {
				continue;
			}

			$ordered_list[] = $item;
		}

		return $ordered_list;
	}

	/**
	 * Orders a group list with parents first
	 *
	 * @param array       $api_list  List of groups.
	 * @param null|string $parent_id Parent id.
	 *
	 * @return array Ordered list
	 */
	private function do_order_by_hierarchy( array $api_list, ?string $parent_id = null ): array {
		$matching_parent     = array_filter(
			$api_list,
			function ( $item ) use ( $parent_id ) {
				return $item['Group']['ParentId'] === $parent_id;
			}
		);
		$not_matching_parent = array_filter(
			$api_list,
			function ( $item ) use ( $parent_id ) {
				return $item['Group']['ParentId'] !== $parent_id;
			}
		);

		if ( empty( $not_matching_parent ) ) {
			return $matching_parent;
		}

		if ( empty( $matching_parent ) ) {
			return array();
		}

		$ordered_children = array();
		foreach ( $matching_parent as $matching_parent_item ) {
			$ordered_children = array_merge(
				$ordered_children,
				$this->do_order_by_hierarchy( $not_matching_parent, $matching_parent_item['Group']['Id'] )
			);
		}

		return array_merge(
			$matching_parent,
			$ordered_children
		);
	}
}
