<?php
/**
 * Flo Order Model
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Orders\Models;

use Automattic\WooCommerce\Utilities\NumberUtil;
use NMFS\Logger;
use NMFS\Settings;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;
use NMFS\Sync\Items\Articles\Stock\Stock;
use NMFS\Sync\Items\Orders\Orders;
use NMFS\Sync\Items\Orders\Receipts\Models\Receipt_Model;
use NMFS\Sync\Items\Orders\Receipts\Receipts;
use NMFS\Sync\Items\Orders\Receipts\Factory as Receipt_Factory;
use WC_Abstract_Order;
use WC_Order;
use WC_Order_Item_Product;
use WC_Order_Item_Shipping;
use WC_Order_Refund;
use WC_Product;
use WC_Tax;
use WP_Error;

/**
 * Order model class.
 */
class Order_Model {

	/**
	 * WC Order of the model.
	 *
	 * @var WC_Order
	 */
	private WC_Order $order;

	/**
	 * Hub account order was made to.
	 *
	 * @var Backoffice_Account_Model
	 */
	private Backoffice_Account_Model $backoffice_account;

	/**
	 * Construct order class.
	 *
	 * @param WC_Abstract_Order $order WC Order.
	 */
	public function __construct( WC_Abstract_Order $order ) {
		if ( $order instanceof WC_Order ) {
			$this->order = $order;
		}
	}

	/**
	 * Get the order Hub account.
	 *
	 * @return Backoffice_Account_Model
	 */
	public function get_backoffice_account(): Backoffice_Account_Model {
		if ( ! empty( $this->backoffice_account ) ) {
			return $this->backoffice_account;
		}

		$backoffice_account_id = $this->order->get_meta( NMF_ORDER_BACKOFFICE_ACCOUNT_ID );
		$backoffice_account    = Backoffice_Accounts::get_account_by_post_id( $backoffice_account_id );

		if ( $backoffice_account ) {
			$this->backoffice_account = $backoffice_account;
			return $this->backoffice_account;
		}

		$parent_id = $this->order->get_parent_id();

		if ( $parent_id ) {
			$parent_order = wc_get_order( $parent_id );
			if ( $parent_order ) {
				$parent_order_model       = new Order_Model( $parent_order );
				$this->backoffice_account = $parent_order_model->get_backoffice_account();
				return $this->backoffice_account;
			}
		}

		$backoffice_account = Backoffice_Accounts::get_current_main_account();

		if ( $backoffice_account ) {
			if ( Backoffice_Accounts::is_multiple_stocks_enabled() ) {
				Logger::warning(
					"Order {$this->order->get_order_number()} has no Hub account. Using main account."
				);
			}

			$this->backoffice_account = $backoffice_account;
			return $this->backoffice_account;
		}

		Logger::error(
			"Order {$this->order->get_order_number()} has no Hub account. No main account found."
		);
		wp_die( 'No Hub account found.' );
	}

	/**
	 * Transforms order to Northmill Flo order
	 *
	 * @return mixed Probably array Hub order, but filtered by moreflo_sync_order_to_moreflo_order.
	 */
	public function order_to_nmflo_order(): mixed {

		$order_no = $this->order->get_order_number();

		$external_reference_prefix = Settings::get( 'hub_order_number_prefix', '' );

		if ( ! empty( $external_reference_prefix ) ) {
			// Make sure the prefix ends with a dash.
			$external_reference_prefix = rtrim( $external_reference_prefix, '-' ) . '-';
		}

		$mf_order = array(
			'ExternalReference'    => $external_reference_prefix . 'MFS-' . $order_no,
			NMF_UPDATE_ON_EXISTING => true,
			'OurReference'         => $order_no,
			'YourOrderNo'          => $order_no,
		);

		$customer         = $this->get_customer();
		$billing_address  = $this->get_billing_address();
		$shipping_address = $this->get_shipping_address();
		$order_rows       = $this->get_order_rows();
		$order_notes      = $this->order->get_customer_note();
		$order_type       = $this->get_order_type();
		$mf_order_data    = get_post_meta( $order_no, 'mf_order' );

		// Saving the order type as meta.
		update_post_meta( $order_no, NMF_ORDER_TYPE, $order_type );

		if ( ! empty( $mf_order_data['mf_order_no'] ) ) {
			$mf_order['Number'] = $mf_order_data['mf_order_no'];
		}

		if ( ! empty( $customer ) ) {
			if ( empty( $customer['CustomerNumber'] ) ) {
				$customer['CustomerNumber'] = $this->get_customer_number( $customer );
			}
			$mf_order['Customer'] = $customer;
		}

		if ( ! empty( $billing_address ) ) {
			$mf_order['BillingAddress'] = $billing_address;
		}

		if ( ! empty( $shipping_address ) ) {
			$mf_order['ShippingAddress'] = $shipping_address;
		}

		if ( ! empty( $order_notes ) ) {
			$mf_order['Note'] = $order_notes;
		}

		$mf_order['PaymentOption'] = $this->get_order_pay_method();
		$mf_order['PaymentStatus'] = $this->get_order_pay_status();

		$mf_order['PaymentReference'] = match ( $this->get_pay_method() ) {
			'moreflo_invoice' => 'Invoice',
			'northmill_bnpl' => 'BNPL:' . $this->order->get_transaction_id(),
			default => $this->get_pay_method() . ': ' . $this->order->get_transaction_id(),
		};

		$mf_order['PaymentOrigin'] = 'WooCommerce';

		$mf_order['State'] = $this->get_order_status( $order_type );

		$mf_order['Rows'] = $order_rows;

		$mf_order['AdjustStockIfOpenState'] =
			Settings::get( 'sync_order_stock_adjustment', 'order_paid' ) === 'order_opened';

		/**
		 * Filter the order before it is sent to Northmill Flo.
		 * Should return an array.
		 *
		 * @param array $mf_order Northmill Flo order.
		 * @param int   $order_id WC order id.
		 */
		return apply_filters( 'moreflo_sync_order_to_moreflo_order', $mf_order, $this->order->get_id() );
	}

	/**
	 * Add refund order data to NM Flo order.
	 *
	 * @param array $mf_order Northmill Flo order.
	 * @param int   $parent_order_id WC order id.
	 *
	 * @return array Updated Northmill Flo order.
	 */
	public function refund_order_to_moreflo_order( array $mf_order, int $parent_order_id ): array {
		$order_no = $this->order->get_order_number();

		$external_reference_prefix = Settings::get( 'hub_order_number_prefix', '' );

		if ( ! empty( $external_reference_prefix ) ) {
			// Make sure the prefix ends with a dash.
			$external_reference_prefix = rtrim( $external_reference_prefix, '-' ) . '-';
		}

		$mf_order['ExternalReference'] = $external_reference_prefix . 'MFS-' . $order_no;
		$mf_order['OurReference']      = $order_no;
		$mf_order['YourOrderNo']       = $order_no;
		$mf_order['Rows']              = $this->get_refund_order_rows( $parent_order_id );
		$mf_order['PaymentOption']     = $this->get_order_pay_method();
		$mf_order['PaymentStatus']     = $this->get_order_pay_status();
		$mf_order['State']             = $this->get_order_status( 'refund' );

		return $mf_order;
	}

	/**
	 * Create a receipt model from order refund.
	 *
	 * @param WC_Order_Refund $order_refund WC order refund object.
	 * @param bool            $restock_items Whether to restock refunded items.
	 *
	 * @return Receipt_Model|WP_Error|bool Receipt model on success, WP_Error on failure, false if aborted.
	 */
	public function order_to_moreflo_receipt_refund( WC_Order_Refund $order_refund, bool $restock_items ): WP_Error|Receipt_Model|bool {

		// Is receipts enabled?
		if ( ! Receipts::is_setup() ) {
			return false;
		}

		// Only create for orders that have sent receipts.
		if ( 'receipt-sent' !== get_post_meta( $this->order->get_id(), '_mf_receipt_status', true ) ) {
			return false;
		}

		if ( 'order_paid' !== Settings::get( 'sync_order_stock_adjustment', 'order_paid' ) ) {
			// Force restocking to false if receipts manage stock is disabled.
			$restock_items = false;
		} else {
			// Check if there are any quantities, else set restock to false.
			$order_items = $order_refund->get_items();
			if ( empty( $order_items ) ) {
				$restock_items = false;
			} else {
				$quantity_order_items = array_filter(
					$order_items,
					function ( $order_item ) {
						return $order_item->get_quantity() !== 0;
					}
				);
				if ( count( $quantity_order_items ) < 1 ) {
					$restock_items = false;
				}
			}
		}

		return Receipt_Factory::create_receipt(
			$this,
			$restock_items
		);
	}

	/**
	 * Create a receipt model from this order.
	 *
	 * @return Receipt_Model|WP_Error|bool Receipt model on success, WP_Error on failure, false if aborted.
	 */
	public function order_to_moreflo_receipt(): WP_Error|Receipt_Model|bool {
		if ( ! Receipts::is_enabled() ) {
			Logger::debug( 'Receipts are not enabled. Receipt not created.' );
			return false;
		}

		if ( ! $this->needs_receipt() ) {
			return false;
		}

		return Receipt_Factory::create_receipt(
			$this,
			Settings::get( 'sync_order_stock_adjustment', 'order_paid' ) === 'order_paid'
		);
	}

	/**
	 * Check if this order needs a receipt.
	 *
	 * @return bool
	 */
	private function needs_receipt(): bool {
		// Is receipts enabled?
		if ( ! Receipts::is_setup() ) {
			return false;
		}

		// Only create for orders that need receipts.
		if ( 'needs-receipt' !== get_post_meta( $this->order->get_id(), '_mf_receipt_status', true ) ) {
			Logger::debug( 'Receipts not needed for order ' . $this->order->get_id() );
			return false;
		}

		// Only create receipts on paid orders.
		if ( is_null( $this->order->get_date_paid() ) ) {
			Logger::debug( 'Receipts not needed since order ' . $this->order->get_id() . ' is not paid.' );
			return false;
		}

		return true;
	}

	/**
	 * Get NMF customer ID of customer.
	 *
	 * @param array $customer Customer data.
	 *
	 * @return false|mixed
	 */
	public function get_customer_number( array $customer ): mixed {
		$nmf_customer_id = false;
		$wc_customer_id  = false;
		if ( get_user_by( 'email', $customer['Email'] ) ) {
			$wc_customer     = get_user_by( 'email', $customer['Email'] );
			$wc_customer_id  = $wc_customer->ID;
			$nmf_customer_id = get_user_meta( $wc_customer_id, 'mf_customer_id', true );
		}

		if ( ! empty( $nmf_customer_id ) ) { // NMF customer ID already stored for this user.
			Logger::debug(
				'Skipping get_customer API call. mf_customer_id#' . $nmf_customer_id .
				' existing on woo side for customer#' . $wc_customer_id
			);
			return $nmf_customer_id; // Skip API call.
		}

		$query = array(
			'SearchPhrase' => $customer['Email'],
			'SearchType'   => 'Email',
		);

		$customer_numbers = $this->get_backoffice_account()->api->get_customer( $query );

		if ( ! empty( $customer_numbers['list'] ) && is_array( $customer_numbers['list'] ) ) {
			// Set the first result from results as customer number.
			$nmf_customer_id = $customer_numbers['list'][0]['Customer']['CustomerNumber'];

			if ( ! empty( $wc_customer_id ) ) {
				// If user use guest account and user email already registered on Woo site, update user meta
				// Or user is already registered user dave mf_customer_id on user meta.
				update_user_meta( $wc_customer_id, 'mf_customer_id', $nmf_customer_id );
				Logger::debug( 'Update User#' . $wc_customer_id . ' with mf_customer_id#' . $nmf_customer_id );
			}
		}
		Logger::debug( 'return customer number #' . $nmf_customer_id . '#' );
		return $nmf_customer_id;
	}

	/**
	 * Get payment method for use in API.
	 *
	 * @return string
	 */
	private function get_order_pay_method(): string {
		if ( 'cod' === $this->get_pay_method() ) {
			return 'Cashier';
		} elseif ( 'moreflo_invoice' === $this->get_pay_method() ) {
			return 'Invoice';
		} else {
			return 'Web';
		}
	}

	/**
	 * Get WC payment method for this order.
	 *
	 * @return string
	 */
	private function get_pay_method(): string {
		return $this->order->get_payment_method();
	}

	/**
	 * Get payment status for use in API.
	 *
	 * @return string
	 */
	private function get_order_pay_status(): string {
		// If the order has been fully paid, it should not revert when set to on-hold (it needs to be credited).
		$order_has_been_fully_paid = get_post_meta( $this->order->get_id(), 'mf_order_has_been_fully_paid', true );
		if ( $order_has_been_fully_paid ) {
			return 'FullyPaid';
		}

		$order_status = $this->order->get_status();
		switch ( $order_status ) {
			case 'pending':
			case 'on-hold':
				$mf_pay_status = 'NotPaid';
				break;
			case 'processing':
				// Orders with payment method 'cod' are set to processing when created (default WC behavior), but are not paid.
				if ( $this->get_pay_method() === 'cod' ) {
					$mf_pay_status = 'NotPaid';
				} elseif ( $this->get_pay_method() === 'moreflo_invoice' ) {
					$mf_pay_status = 'NotPaid';
				} else {
					$mf_pay_status = 'FullyPaid';
					update_post_meta( $this->order->get_id(), 'mf_order_has_been_fully_paid', true );
				}
				break;
			case 'completed':
				$mf_pay_status = 'FullyPaid';
				update_post_meta( $this->order->get_id(), 'mf_order_has_been_fully_paid', true );
				break;
			case 'cancelled':
			case 'refunded':
			case 'failed':
			default:
				$mf_pay_status = 'NotSet';
				break;
		}
		return $mf_pay_status;
	}

	/**
	 * Get order status for use in API.
	 *
	 * @param string $order_type WC order type.
	 *
	 * @return string
	 */
	private function get_order_status( string $order_type ): string {

		$order_status = $this->order->get_status();

		$nmf_order_status = match ( $order_status ) {
			'on-hold' => 'Started',
			'processing', 'completed' => 'Closed',
			'cancelled', 'refunded', 'failed', 'kco_incomplete' => 'Cancelled',
			default => 'Created',
		};

		$payment_completes_status = array( 'completed', 'processing' );
		if (
			in_array( $order_status, $payment_completes_status, true ) &&
			empty( $this->order->get_transaction_id() )
		) {
			$nmf_order_status = 'ReadyForHandOut';
		}

		// Order v3 integration.
		if (
			'NORMAL_ORDER' === $order_type &&
			in_array( $order_status, $payment_completes_status, true ) &&
			$this->get_pay_method() !== 'cod' &&
			$this->get_pay_method() !== 'moreflo_invoice'
		) {
			if ( $this->is_shipping_local_pickup() ) {
				$nmf_order_status = 'ReadyForHandOut';
			} else {
				$nmf_order_status = 'Delivered';
			}
		}

		return $nmf_order_status;
	}

	/**
	 * Create customer data from this order.
	 *
	 * @return array|null Customer data, or null if the customer Name isn't set.
	 */
	public function get_customer(): ?array {

		$nmf_customer_id = get_post_meta( $this->order->get_id(), 'mf_customer_id', true );

		$customer = array(
			'CustomerNumber' => ( isset( $nmf_customer_id ) ) ? $nmf_customer_id : '',
			'Name'           => trim( $this->order->get_billing_first_name() . ' ' . $this->order->get_billing_last_name() ),
			'Address1'       => $this->order->get_billing_address_1(),
			'Address2'       => $this->order->get_billing_address_2(),
			'PostCode'       => $this->order->get_billing_postcode(),
			'City'           => $this->order->get_billing_city(),
			'Country'        => $this->order->get_billing_country(),
			'Email'          => $this->order->get_billing_email(),
			'Mobile'         => $this->order->get_billing_phone(),
		);

		foreach ( $customer as $key => $v ) {
			if ( empty( $v ) && 'CustomerNumber' !== $key ) {
				unset( $customer[ $key ] );
			}
		}

		return ( isset( $customer['Name'] ) ? $customer : null );
	}

	/**
	 * Get the billing address for this order.
	 *
	 * @return array|null Array of billing address data for API, or null if name in billing address is set.
	 */
	private function get_billing_address(): ?array {

		$address = array(
			'Name'     => trim( $this->order->get_billing_first_name() . ' ' . $this->order->get_billing_last_name() ),
			'Address1' => $this->order->get_billing_address_1(),
			'Address2' => $this->order->get_billing_address_2(),
			'PostCode' => $this->order->get_billing_postcode(),
			'City'     => $this->order->get_billing_city(),
			'Country'  => $this->order->get_billing_country(),
			'Email'    => $this->order->get_billing_email(),
			'Mobile'   => $this->order->get_billing_phone(),
		);

		foreach ( $address as $key => $v ) {
			if ( empty( $v ) ) {
				unset( $address[ $key ] );
			}
		}

		return ( isset( $address['Name'] ) ? $address : null );
	}

	/**
	 * Get the shipping address for this order.
	 *
	 * @return array|null Array of shipping address data for API, or null if name in shipping address is set.
	 */
	private function get_shipping_address(): ?array {

		$address = array(
			'Name'     => trim( $this->order->get_shipping_first_name() . ' ' . $this->order->get_billing_last_name() ),
			'Address1' => $this->order->get_shipping_address_1(),
			'Address2' => $this->order->get_shipping_address_2(),
			'PostCode' => $this->order->get_shipping_postcode(),
			'City'     => $this->order->get_shipping_city(),
			'Country'  => $this->order->get_shipping_country(),
		);

		foreach ( $address as $key => $v ) {
			if ( empty( $v ) ) {
				unset( $address[ $key ] );
			}
		}

		return ( isset( $address['Name'] ) ? $address : null );
	}

	/**
	 * Get tax for order row
	 *
	 * @param WC_Product $product Product to get tax for.
	 *
	 * @return float
	 */
	private function get_order_row_tax( WC_Product $product ): float {
		$tax_percentage = 0;
		$wc_tax         = new WC_Tax();

		$tax_class = $product->get_tax_class();

		$tax_data = $wc_tax->get_rates( $tax_class );

		foreach ( $tax_data as $tax_rate ) {
			$tax_percentage = $tax_rate['rate'];
		}

		return is_numeric( $tax_percentage ) ? (float) $tax_percentage : 0;
	}

	/**
	 * Get order rows for this order.
	 *
	 * @return array Array of order rows for API, or null if no order rows are set.
	 */
	private function get_order_rows(): array {

		$order_rows = array();

		/**
		 * Order item products
		 *
		 * @var WC_Order_Item_Product[] $order_items
		 */
		$order_items = $this->order->get_items();

		foreach ( $order_items as $item_data ) {

			// Uses receipt calculations to avoid duplicate code.
			$receipt_article = Receipt_Factory::maybe_create_article(
				$item_data,
				$this,
				0,
				'',
				null,
				false
			);

			if ( is_null( $receipt_article ) ) {
				// Not a valid row.
				continue;
			}

			$product = $item_data->get_product();

			$item = array(
				NMF_ARTICLE_NUMBER => $receipt_article->get_sku(),
				NMF_ARTICLE_NAME   => $receipt_article->get_name(),
				NMF_QUANTITY       => $receipt_article->get_quantity(),
				'Discount'         => NumberUtil::round(
					$receipt_article->get_total_discount(),
					wc_get_rounding_precision()
				),
			);

			$item['UnitAmountInclVatExclDiscount'] = $receipt_article->get_unit_price_without_discounts();
			$item['VatRate']                       = $receipt_article->get_vat_rate();

			if ( ! empty( $item[ NMF_ARTICLE_NUMBER ] ) && ! empty( $item[ NMF_QUANTITY ] ) ) {
				$order_rows[] = $item;

				$order_rows = apply_filters( 'moreflo_sync_add_order_row_after', $order_rows, $item_data, $product );
			}
		}

		$shipping_row = $this->get_shipping_order_row();

		if ( ! is_null( $shipping_row ) ) {
			$order_rows[] = $shipping_row;
		}

		return $order_rows;
	}

	/**
	 * Get order rows for order refund.
	 *
	 * @param int $parent_order_id WC Parent order ID.
	 *
	 * @return array
	 */
	private function get_refund_order_rows( int $parent_order_id ): array {
		$order_rows   = array();
		$order_items  = $this->order->get_items( array( 'line_item', 'shipping' ) );
		$parent_order = wc_get_order( $parent_order_id );

		$shipping_vat         = 25.0; // Default value, is updated below.
		$shipping_product_sku = $this->get_backoffice_account()->get_shipping_settings()->get_article_sku();
		$refund_part_sku      = $this->get_backoffice_account()->partial_refund_article_sku;

		if ( ! empty( $shipping_product_sku ) ) {
			$shipping_product_id = wc_get_product_id_by_sku( $shipping_product_sku );
			$shipping_product    = wc_get_product( $shipping_product_id );

			if ( is_a( $shipping_product, 'WC_Product' ) ) {
				$shipping_vat = $this->get_order_row_tax( $shipping_product );
			}
		}

		foreach ( $order_items as $item_data ) {
			if ( is_a( $item_data, 'WC_Order_Item_Shipping' ) && $shipping_product_sku ) {
				$price        = abs( $item_data->get_total() + $item_data->get_total_tax() );
				$vat          = $shipping_vat;
				$product_sku  = $shipping_product_sku;
				$product_name = __( 'Shipping', 'flo-sync' );
				$quantity     = -1;
				$dis          = 0;
			} elseif ( is_a( $item_data, 'WC_Order_Item_Product' ) ) {
				$product     = $item_data->get_product();
				$product_sku = $product->get_sku();
				$quantity    = $item_data->get_quantity();

				$price        = $product->get_regular_price();
				$vat          = $this->get_order_row_tax( $product );
				$product_name = $product->get_name();
				$dis          = 0;

				if ( 0 === $quantity ) {
					if ( ! $refund_part_sku ) {
						// Refund part sku has not been set, cannot create row for this product.
						Logger::debug( 'Refund part sku has not been set, cannot create row for ' . $product_name . '.' );
						continue;
					}

					$quantity    = -1;
					$product_sku = $refund_part_sku;
					$price       = abs( $item_data->get_total() + $item_data->get_total_tax() );
				} else {
					$original_sales_price = $parent_order->get_item_subtotal( $item_data, true, false );

					if ( $original_sales_price < $price ) {
						$dis = $price * $quantity - $original_sales_price * $quantity;
					}
				}
			} else {
				continue;
			}

			$item = array(
				NMF_ARTICLE_NUMBER => $product_sku,
				NMF_ARTICLE_NAME   => $product_name,
				NMF_QUANTITY       => $quantity,
			);

			$item['UnitAmountInclVatExclDiscount'] = $price;
			$item['VatRate']                       = $vat;
			$item['Discount']                      = $dis;

			if ( ! empty( $item[ NMF_ARTICLE_NUMBER ] ) && ! empty( $item[ NMF_QUANTITY ] ) ) {
				$order_rows[] = $item;
			}
		}

		return $order_rows;
	}

	/**
	 * Get shipping order row for this order.
	 *
	 * @return array|null
	 */
	private function get_shipping_order_row(): ?array {
		$shipping_article_sku = $this->get_backoffice_account()->get_shipping_settings()->get_article_sku();

		if ( ! empty( $shipping_article_sku ) ) {
			$shipping_tax_rate       = 0;
			$shipping_tax            = floatval( $this->order->get_shipping_tax() );
			$shipping_total          = floatval( $this->order->get_shipping_total() );
			$shipping_total_with_vat = round( $shipping_total + $shipping_tax, wc_get_price_decimals() );

			if ( $shipping_tax > 0 ) {
				/**
				 * Shipping items.
				 *
				 * @var WC_Order_Item_Shipping[] $tax_lines
				 */
				$tax_lines = $this->order->get_items( 'shipping' );
				if ( ! empty( $tax_lines ) ) {
					foreach ( $tax_lines as $tl ) {
						$tax_totals         = $tl->get_taxes()['total'];
						$shipping_tax_total = floatval( array_shift( $tax_totals ) );
						$shipping_tax_rate  = ceil( 100 * ( ( ( $shipping_total + $shipping_tax_total ) / $shipping_total ) - 1 ) );
					}
				}
			}

			if ( $shipping_total_with_vat > 0 ) {
				return array(
					NMF_ARTICLE_NUMBER              => $shipping_article_sku,
					NMF_ARTICLE_NAME                => __( 'Shipping', 'flo-sync' ),
					'UnitAmountInclVatExclDiscount' => $shipping_total_with_vat,
					'VatRate'                       => $shipping_tax_rate,
					NMF_QUANTITY                    => 1,
				);
			}
		}

		return null;
	}

	/**
	 * Validate order api response.
	 *
	 * @param string      $type    Order type.
	 * @param string      $action  Executed API action.
	 * @param object|null $order   Order response object. Null if error.
	 *
	 * @return bool
	 */
	public function validate_moreflo_order_response( string $type, string $action, ?object $order ): bool {

		$order_id = $this->order->get_id();

		if ( $order ) {
			$order_data = get_post_meta( $order_id, 'mf_order', true );

			if (
				empty( $order_data[ NMF_ORDER_ID ] ) &&
				NMF_ORDER_TYPE_NORMAL === $type &&
				! empty( $order->list[0]->Success ) &&
				! empty( $order->list[0]->Order->Number )
			) {

				$order_data = array(
					'mf_order_no'  => $order->list[0]->Order->Number,
					NMF_ORDER_ID   => $order->list[0]->Order->Id,
					NMF_ORDER_TYPE => $type,
				);
				update_post_meta( $order_id, 'mf_order', $order_data );
				update_post_meta( $order_id, 'mf_id', $order_data[ NMF_ORDER_ID ] );

			}

			if ( ! empty( $order->list[0]->Success ) ) {
				$backoffice_account = $this->get_backoffice_account();
				if ( 'update' === $action ) {
					$this->add_order_note(
						sprintf(
							// translators: %1$s: Order type, %2$s: Hub order number, %3$s: Hub account name.
							__( 'Hub %1$s order(#%2$s) updated successfully in Hub account %3$s.', 'flo-sync' ),
							$type,
							$order_data['mf_order_no'],
							$backoffice_account->get_display_name()
						)
					);
				} else {
					$this->add_order_note(
						sprintf(
							// translators: %1$s: Order type, %2$s: Hub order number, %3$s: Hub account name.
							__( 'Hub %1$s order(#%2$s) added successfully in Hub account %3$s.', 'flo-sync' ),
							$type,
							$order_data['mf_order_no'],
							$backoffice_account->get_display_name()
						)
					);
				}

				return true;
			}

			$this->add_order_note( $order );
		}

		$this->add_order_note( __( 'Hub order sync failed. Please retry again by saving the order.', 'flo-sync' ) );
		return false;
	}

	/**
	 * Get article from API and update WC article quantity.
	 *
	 * @param WC_Product $product Product to reset stock for.
	 */
	public function reset_product_quantity( WC_Product $product ): void {
		$include_product_types = array( 'simple', 'variation', 'grouped' ); // Product types that can be reset.

		$article_sku = $product->get_sku();
		$product_id  = $product->get_id();

		// Only reset stock for predefined product types.
		if ( ! in_array( $product->get_type(), $include_product_types, true ) ) {
			Logger::error( 'Product type ' . $product->get_type() . ' cannot be reset, for product with SKU: ' . $article_sku );
			$this->order->add_order_note(
				sprintf(
					// translators: %s: Order number.
					__(
						'Cannot reset quantity for product of type "%1$s". Trying to reset product with SKU: %2$s',
						'flo-sync'
					),
					$product->get_type(),
					$article_sku
				)
			);
			return;
		}

		$stock_updated = Stock::get_and_update_article_stock_data( $article_sku );

		if ( is_wp_error( $stock_updated ) ) {
			$this->order->add_order_note( $stock_updated->get_error_message() );
			Logger::error( 'Failed to get stock data for article ' . $article_sku . ' in order: ' . $this->order->get_id() );
			return;
		} elseif ( is_null( $stock_updated ) ) {
			// Product is not stock managed.
			return;
		}

		$new_stock_quantity = $product->get_stock_quantity();

		Logger::debug(
			'Reset stock for product ID ' . $product_id . ', Quantity:' . $new_stock_quantity .
				'. Due to order status ' . $this->order->get_status()
		);
		$this->add_order_note(
			sprintf(
				// translators: %1$s: Product SKU, %2$s: Quantity, %3$s: Order status.
				__( 'Reset stock for product %1$d, Quantity: %2$d. Due to order status %3$d.', 'flo-sync' ),
				$article_sku,
				$new_stock_quantity,
				$this->order->get_status()
			)
		);
	}

	/**
	 * Get the order type for use in API. We currently only support normal orders.
	 *
	 * @return string
	 */
	public function get_order_type(): string {
		return 'NORMAL_ORDER';
	}

	/**
	 * Adds note to order.
	 *
	 * @param mixed $note Note to add. Will be run through print_r if not a string.
	 *
	 * @return void
	 */
	public function add_order_note( mixed $note ): void {
		if ( ! is_string( $note ) ) {
			// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r
			$note = print_r( $note, true );
		}

		$this->order->add_order_note( $note );

		Logger::debug( sprintf( '[Order Note] %s', $note ) );
	}

	/**
	 * Update order from API if the payment status has changed.
	 *
	 * @param string $mf_pay_status Payment status from API.
	 *
	 * @return bool True if order was updated, false if not.
	 */
	public function update_order( string $mf_pay_status ): bool {
		if ( ! isset( $this->order ) ) {
			return false;
		}

		$order_id = $this->order->get_id();

		$current_nmf_order_status = $this->get_current_nmf_order_status();

		if ( $current_nmf_order_status !== $mf_pay_status ) {

			if ( 'FullyPaid' === $mf_pay_status && empty( $this->order->get_transaction_id() ) ) {
				update_post_meta( $order_id, '_payment_method', 'PayedFromCashier' );
				update_post_meta( $order_id, '_payment_method_title', __( 'Payed From Cashier', 'flo-sync' ) );
				update_post_meta( $order_id, '_transaction_id', time() );
				update_post_meta( $order_id, 'mf_order_has_been_fully_paid', true );
				$this->order->payment_complete();
			}

			$mf_order = $this->order_to_nmflo_order();
			$new_hash = Orders::generate_order_hash( $mf_order );
			update_post_meta( $order_id, NMF_HASH, $new_hash );

			return true;
		} else {
			Logger::warning( sprintf( 'Order(%s) update failed in WC. (No mapped MF status/Order payment status not changed)', $order_id ) );
			Logger::debug( sprintf( 'Current Status: %s / New Status: %s', $current_nmf_order_status, $mf_pay_status ) );

			return false;
		}
	}

	/**
	 * Get NFM order status for current order
	 *
	 * @return string
	 */
	private function get_current_nmf_order_status(): string {
		$current_status = $this->order->get_status();
		$new_mf_status  = '';

		switch ( $current_status ) {
			case 'cancelled':
			case 'refunded':
			case 'failed':
				$new_mf_status = 'NotSet';
				break;
			case 'pending':
			case 'on-hold':
				$new_mf_status = 'NotPaid';
				break;
			case 'processing':
			case 'completed':
				$new_mf_status = 'FullyPaid';
				break;
			default:
				break;
		}
		return $new_mf_status;
	}

	/**
	 * Check if the shipping method of this order is local pickup.
	 *
	 * @return bool
	 */
	public function is_shipping_local_pickup(): bool {
		$shipping_methods = $this->order->get_shipping_methods();
		$local_pickup     = false;

		foreach ( $shipping_methods as $item ) {
			$shipping_method_id = $item->get_method_id();
			if ( 'local_pickup' === $shipping_method_id ) {
				$local_pickup = true;
			}
		}

		return $local_pickup;
	}

	/**
	 * Get order total of this order.
	 *
	 * @return float
	 */
	public function get_order_total(): float {
		return $this->order->get_total();
	}

	/**
	 * Get WC order of this model.
	 *
	 * @return WC_Order
	 */
	public function get_order(): WC_Order {
		return $this->order;
	}
}
