<?php
/**
 * Order functions.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Orders;

defined( 'ABSPATH' ) || exit;

use Exception;
use NMFS\Logger;
use NMFS\Settings;
use NMFS\Sync\Cron\Models\Lock_Model;
use NMFS\Sync\Items\Orders\Models\Order_Model;
use NMFS\Sync\Items\Orders\Receipts\Receipts;
use WC_Order;
use WC_Order_Item_Product;
use WC_Order_Refund;

/**
 * Order sync class.
 */
class Orders {

	/**
	 * Constructs order sync.
	 */
	public static function setup(): void {
		add_action( 'woocommerce_new_order', array( self::class, 'save_order' ), 10, 2 ); // Also hooked to 'woocommerce_update_order' in self::set_hook_save_order().
		add_action( 'woocommerce_payment_complete', array( self::class, 'payment_complete_post_order' ) );
		add_action( 'mf_retry_save_order', array( self::class, 'retry_save_order' ) );
		add_action( 'mf_retry_post_receipt', array( self::class, 'retry_post_receipt' ) );

		add_action( 'woocommerce_create_refund', array( self::class, 'validate_refund_order' ) );
		add_action( 'woocommerce_refund_created', array( self::class, 'refund_order' ), 10, 2 );

		add_action( 'northmill_checkout_bnpl_transaction_note', array( self::class, 'add_bnpl_transaction_note' ), 10, 2 );

		self::set_hook_save_order( true );
	}

	/**
	 * Enable or disable save order hook.
	 * This is mainly used for disabling the hook when doing custom order updates.
	 * This is needed because the hook will otherwise trigger when updating order, causing a loop.
	 *
	 * @param bool $on Whether to enable or disable the hook.
	 */
	public static function set_hook_save_order( bool $on ): void {
		if ( $on ) {
			add_action( 'woocommerce_update_order', array( self::class, 'save_order' ), 10, 2 );
		} else {
			remove_action( 'woocommerce_update_order', array( self::class, 'save_order' ) );
		}
	}

	/**
	 * Function to retry saving order after having been postponed due to lock.
	 *
	 * @param int $order_id WC order ID.
	 */
	public static function retry_save_order( int $order_id ): void {
		$order = wc_get_order( $order_id );
		if ( $order ) {
			$order->add_order_note( esc_html__( 'Retrying order sync.', 'flo-sync' ) );
			self::save_order( $order_id, $order );
		}
	}

	/**
	 * Retry posting receipt to the hub for a given order.
	 *
	 * @param int $order_id The WooCommerce order ID.
	 */
	public static function retry_post_receipt( int $order_id ): void {
		$order = wc_get_order( $order_id );
		if ( $order ) {
			$order->add_order_note( esc_html__( 'Retrying receipt sync.', 'flo-sync' ) );
			$order_instance = new Order_Model( $order );
			self::maybe_post_receipt_to_hub( $order_instance, $order_id, $order );
		}
	}

	/**
	 * Sync order after payment has been completed.
	 * This is needed because the order status is not updated until after the payment has been completed.
	 * When doing payment, order sync is postponed until after payment has been completed.
	 *
	 * @param int $order_id WC order ID.
	 *
	 * @return void
	 */
	public static function payment_complete_post_order( int $order_id ): void {
		$order = wc_get_order( $order_id );
		$order->add_order_note( esc_html__( 'Syncing order after payment.', 'flo-sync' ) );
		Logger::debug( 'Syncing order after payment, for order: ' . $order_id );
		Logger::debug( 'Order status: ' . $order->get_status() );
		self::save_order( $order_id, $order );
	}

	/**
	 * Save order to Flo.
	 *
	 * @param int           $order_id WC order ID.
	 * @param WC_Order|null $order    WC order.
	 */
	public static function save_order( int $order_id, ?WC_Order $order = null ): void {
		// Don't sync orders if order sync is disabled.
		if ( ! Settings::is_setting_enabled( 'sync_new_orders_to_apps', true ) ) {
			Logger::notice( 'Order sync is disabled, not syncing order with WP ID: ' . $order_id );
			return;
		}

		if ( doing_action( 'woocommerce_update_order' ) && did_action( 'woocommerce_pre_payment_complete' ) ) {
			// Order is being updated after payment has been completed, but before the status has been changed, do not sync.
			// Wait for woocommerce_payment_complete hook to sync.
			Logger::debug( 'Order sync postponed until after payment has been completed, for order: ' . $order_id );
			return;
		}

		Logger::debug( 'Syncing order with WP ID: ' . $order_id . ', from action ' . current_action() );

		$lock = new Lock_Model( __FUNCTION__ . '-' . $order_id );
		if ( ! $lock->lock() ) {
			// Save order is already running for this order, retry later.
			wp_schedule_single_event( time() + 60, 'mf_retry_save_order', array( $order_id ) );
			$order->add_order_note( esc_html__( 'Aborted syncing order. It is already running.', 'flo-sync' ) );

			return;
		}

		/**
		 * Validate order to sync.
		 */
		$validated_order_data = self::validate_order_to_sync( $order );

		if ( ! $validated_order_data ) {
			$lock->clear_lock();
			return;
		}

		/**
		 * Variables from self::validated_order_data()
		 *
		 * @var string $new_order_hash
		 * @var string $action
		 * @var array $mf_order
		 * @var Order_Model $order_instance
		 */
		$new_order_hash = $validated_order_data['new_order_hash'];
		$action         = $validated_order_data['action'];
		$mf_order       = $validated_order_data['mf_order'];
		$order_instance = $validated_order_data['order_instance'];

		/**
		 * Filter order before posting to Hub.
		 *
		 * @param array     $mf_order Order data formatted for API.
		 * @param WC_Order  $order    WC order.
		 * @param string    $action   Action to be performed for order. Either 'add' or 'update'.
		 */
		$mf_order = apply_filters( 'moreflo_sync_before_order_sync', $mf_order, $order, $action );

		/**
		 * Post order to Hub.
		 */
		$order_post_response = $order_instance->get_backoffice_account()->api->post_order( $mf_order );
		$order_instance->validate_moreflo_order_response( NMF_ORDER_TYPE_NORMAL, $action, $order_post_response );

		/**
		 * Order receipt.
		 */
		self::maybe_post_receipt_to_hub( $order_instance, $order_id, $order );

		if ( $order_post_response ) { // If order was posted successfully.
			update_post_meta( $order_id, NMF_HASH, $new_order_hash );
			update_post_meta( $order_id, NMF_ORDER_SYNC_STATUS, __( 'Synced', 'flo-sync' ) );
			update_post_meta( $order_id, 'mf_last_sync_response', $order_post_response );

			Receipts::maybe_reset_receipt_status( $order_id );

			self::handle_posted_order_stock_changes( $order, $mf_order );

			Logger::debug( 'Order pushed and updated' );
		}

		$lock->clear_lock();
	}

	/**
	 * Validate order to sync.
	 *
	 * @param object|null $order WC Order object.
	 *
	 * @return false|array Returns an array containing the following:
	 *                     - 'new_order_hash' (string): The new order hash.
	 *                     - 'action' (string): The action to perform on the order (add or update).
	 *                     - 'mf_order' (array): The order data to sync with Flo.
	 *                     - 'order_instance' (Order_Model): The Order_Model instance of the order.
	 *                     Returns false if the order is not valid to sync.
	 */
	private static function validate_order_to_sync( ?object $order ): false|array {
		// Only proceed if order is a WC_Order.
		if ( empty( $order ) || ! $order instanceof WC_Order ) {
			Logger::error( 'Order is not a WC_Order object.' );
			return false;
		}

		$order_id       = $order->get_id();
		$order_instance = new Order_Model( $order );
		$mf_order       = $order_instance->order_to_nmflo_order();
		$mf_order_type  = $order_instance->get_order_type();
		$mf_order_total = $order_instance->get_order_total();

		// Only proceed with valid orders.
		if ( empty( $mf_order ) || 'NORMAL_ORDER' !== $mf_order_type ) {
			$order->add_order_note( __( 'Order is invalid and cannot be pushed to Hub.', 'flo-sync' ) );
			Logger::error( 'Order is invalid and cannot be pushed to Hub. (Order ID: ' . $order_id . ')' );
			return false;
		}

		// Don't post orders with a total of 0 or less.
		if ( $mf_order_total <= 0 ) {
			Logger::debug( 'Order total is less than zero. (Order ID: ' . $order_id . ')' );
			return false;
		}

		$old_hash = get_post_meta( $order_id, NMF_HASH, true );
		$new_hash = self::generate_order_hash( $mf_order );

		// Only proceed if the order has changed.
		if ( $old_hash === $new_hash ) {
			Logger::debug( 'Hash verification failed, order has not changed since last sync.' );
			return false;
		}

		return array(
			'new_order_hash' => $new_hash,
			'action'         => $old_hash ? 'update' : 'add',
			'mf_order'       => $mf_order,
			'order_instance' => $order_instance,
		);
	}

	/**
	 * Maybe post receipt to Hub API.
	 *
	 * @param Order_Model $order_instance Order model.
	 * @param int         $order_id       WC order ID.
	 * @param WC_Order    $order          WC order.
	 *
	 * @return void
	 */
	private static function maybe_post_receipt_to_hub( Order_Model $order_instance, int $order_id, WC_Order $order ): void {
		$mf_receipt = $order_instance->order_to_moreflo_receipt();
		if ( is_wp_error( $mf_receipt ) ) { // Preparing the receipt failed.
			update_post_meta( $order_id, '_mf_receipt_status', 'receipt-failed' );

			Logger::error( 'Order receipt failure. (Order ID: ' . $order_id . '; Message: ' . $mf_receipt->get_error_message() . ')' );
			$order->add_order_note( __( 'Preparing receipt for Hub failed. See log for error message.', 'flo-sync' ) );

		} elseif ( $mf_receipt ) { // We can safely send the receipt.
			Logger::debug( 'Send receipt.' );

			$receipt_response = $order_instance->get_backoffice_account()->api->post_receipt( $mf_receipt );
			update_post_meta( $order_id, 'mf_receipt_response', $receipt_response );
			if ( ! $receipt_response ) {
				// Failure.
				self::handle_post_receipt_failure( $order );
			} else {
				update_post_meta( $order_id, '_mf_receipt_status', 'receipt-sent' );
				Logger::debug( 'Order receipt sent successfully. (Order ID: ' . $order_id . ')' );
				$receipt_response_obj = json_decode( $receipt_response );
				// Success.
				$receipt_id = $receipt_response_obj->Receipt->Id;
				$voucher_id = $receipt_response_obj->Receipt->Voucher->Id;
				update_post_meta( $order_id, 'mf_receipt_id', $receipt_id );
				update_post_meta( $order_id, 'mf_receipt_voucher_id', $voucher_id );
				$order->add_order_note(
					sprintf(
					// translators: %1$s: Receipt ID, %2$s: Voucher ID.
						esc_html__(
							'Order receipt sent successfully. (Receipt: %1$s; Voucher: %2$s)',
							'flo-sync'
						),
						$receipt_id,
						$voucher_id
					)
				);
			}
		} else { // Receipt not needed, or receipts are disabled.
			Logger::debug( 'Order receipt not sent. (Order ID: ' . $order_id . ')' );
		}
	}

	/**
	 * Handle post receipt failure.
	 *
	 * @param WC_Order $order WC order.
	 */
	private static function handle_post_receipt_failure( WC_Order $order ): void {
		$reattempt_count = get_post_meta( $order->get_id(), '_mf_receipt_reattempts', true );
		if ( ! $reattempt_count ) {
			$reattempt_count = 0;
		}

		if ( $reattempt_count >= 3 ) {
			// Failure.
			update_post_meta( $order->get_id(), '_mf_receipt_status', 'receipt-failed' );
			Logger::error( 'Order receipt failed. (Order ID: ' . $order->get_id() . ')' );

			$order->add_order_note( __( 'Final receipt sync attempt failed.', 'flo-sync' ) );
			return;
		}

		++$reattempt_count;
		update_post_meta( $order->get_id(), '_mf_receipt_reattempts', $reattempt_count );

		$reattempt_delay_in_minutes = 5 * $reattempt_count;

		Logger::error( 'Failed to send order receipt to Hub. Attempt number ' . $reattempt_count . ' (Order ID: ' . $order->get_id() . ')' );
		$order->add_order_note(
			sprintf(
				// translators: %d: Delay in minutes.
				_n(
					'Failed to send order receipt to Hub. Retrying in %d minute.',
					'Failed to send order receipt to Hub. Retrying in %d minutes.',
					$reattempt_delay_in_minutes,
					'flo-sync'
				),
				$reattempt_delay_in_minutes,
			)
		);
		wp_schedule_single_event(
			time() + ( MINUTE_IN_SECONDS * $reattempt_delay_in_minutes ),
			'mf_retry_post_receipt',
			array( $order->get_id() )
		);
	}

	/**
	 * Handle posted order stock changes.
	 *
	 * @param WC_Order $order    The order object.
	 * @param array    $mf_order Order data as sent to API.
	 *
	 * @return void
	 */
	private static function handle_posted_order_stock_changes( WC_Order $order, array $mf_order ): void {
		if ( did_action( 'woocommerce_pre_payment_complete' ) ) {
			/**
			 * Doing payment (theoretically including processing free orders, but we currently don't support orders with total 0).
			 *
			 * We don't update stock here, since it will be synced and updated when payment is completed.
			 * Function wc_maybe_reduce_stock_levels() is hooked to 'woocommerce_payment_complete'.
			 */
			Logger::notice( 'Skipping stock after sync update because doing payment (Order ID: ' . $order->get_id() . ')' );
		} elseif ( $mf_order['AdjustStockIfOpenState'] ?? false ) {
			/**
			 * Adjust stock on order open.
			 *
			 * Flo Sync is set up to adjust stock when order is opened.
			 * Maybe update stock for order items, if not already done.
			 */
			Logger::notice( 'Maybe adjust stock since order has AdjustStockIfOpenState (Order ID: ' . $order->get_id() . ')' );
			wc_maybe_reduce_stock_levels( $order->get_id() );
		} elseif ( ! doing_action( 'woocommerce_new_order' ) ) {
			/**
			 * Just updating order.
			 *
			 * The order is being updated - rather than created - and a payment is not being processed.
			 * Update stock for order items.
			 */
			self::maybe_reset_order_items_stock_quantity( $order->get_id() );
		} else {
			/**
			 * Creating unpaid order.
			 *
			 * The order is being created (doing action 'woocommerce_new_order'), and is not paid.
			 * Creating orders should be quick, so we don't sync stock here.
			 */
			Logger::notice( 'Skipping stock after sync update, creating unpaid order (Order ID: ' . $order->get_id() . ')' );
		}
	}

	/**
	 * Hook on create refund order.
	 *
	 * @param int   $order_id Order ID.
	 * @param array $args     Refund arguments.
	 */
	public static function refund_order( int $order_id, array $args ): void {
		$refund_order = new WC_Order( $order_id );
		$parent_id    = $refund_order->get_parent_id();
		$parent_order = new WC_Order( $parent_id );

		// Create negative order in Apps
		// Use parent order as template to get all customer info.
		$parent_refund_order_instance = new Order_Model( $parent_order );
		$parent_mf_order              = $parent_refund_order_instance->order_to_nmflo_order();

		// If the order is fully refunded, we will instead sync order as canceled.
		if ( did_action( 'woocommerce_order_fully_refunded' ) ) {
			// We still need to sync refund receipt.
			self::refund_order_receipt( $order_id, $parent_order, $parent_refund_order_instance, $args['restock_items'] );

			return;
		}

		// Create partial refund info.
		$refund_order_instance = new Order_Model( $refund_order );
		$refund_mf_order       = $refund_order_instance->refund_order_to_moreflo_order( $parent_mf_order, $parent_id );

		// Ensure don't have an empty order.
		if ( empty( $refund_mf_order ) || empty( $refund_mf_order['Rows'] ) ) {
			Logger::error( 'Refund order failed, empty. (order id: ' . $order_id . ')' );

			return;
		}

		// Post to Apps.
		/**
		 * Filter order before posting to Hub.
		 * Should return an array.
		 *
		 * @param array     $refund_mf_order Refund order.
		 * @param WC_Order  $refund_order    Refund order.
		 * @param WC_Order  $parent_order    Parent order.
		 */
		$refund_mf_order = apply_filters( 'moreflo_sync_before_order_refund_sync', $refund_mf_order, $refund_order, $parent_order );
		$response        = $parent_refund_order_instance->get_backoffice_account()->api->post_order( $refund_mf_order );
		$refund_order_instance->validate_moreflo_order_response( NMF_ORDER_TYPE_NORMAL, 'add', $response );
		$new_hash = self::generate_order_hash( $refund_mf_order );

		// Check error response.
		if ( ! $response ) {
			Logger::error( 'Refund order failed, no response from Apps. (order id: ' . $order_id . ')' );

			return;
		}

		// Update last sync.
		update_post_meta( $order_id, NMF_HASH, $new_hash );
		update_post_meta( $order_id, NMF_ORDER_SYNC_STATUS, __( 'Synced', 'flo-sync' ) );
		update_post_meta( $order_id, 'mf_last_sync_response', $response );
		self::maybe_reset_order_items_stock_quantity( $order_id );

		self::refund_order_receipt( $order_id, $parent_order, $parent_refund_order_instance, $args['restock_items'] );

		Logger::debug( 'Refund order pushed and updated' );
	}

	/**
	 * Sends refund order receipt if activated in settings
	 *
	 * @param int         $order_id                     WC Order ID.
	 * @param WC_Order    $parent_order                 WC Order.
	 * @param Order_Model $parent_refund_order_instance Order instance.
	 * @param bool        $restock_items                Whether to restock items.
	 */
	private static function refund_order_receipt(
		int $order_id,
		WC_Order $parent_order,
		Order_Model $parent_refund_order_instance,
		bool $restock_items
	): void {
		// Create a new receipt for refunded order.
		$order_refund = wc_get_order( $order_id );
		$mf_receipt   = $parent_refund_order_instance->order_to_moreflo_receipt_refund( $order_refund, $restock_items );
		if ( is_wp_error( $mf_receipt ) ) {
			// Failure.
			update_post_meta( $order_id, '_mf_receipt_status', 'receipt-failed' );

			Logger::error( 'Order receipt failure. (Order ID: ' . $order_id . '; Message: ' . $mf_receipt->get_error_message() . ')' );
		} elseif ( $mf_receipt ) {
			// We can safely send the receipt.
			$receipt_response = $parent_refund_order_instance->get_backoffice_account()->api->post_receipt( $mf_receipt );
			update_post_meta( $order_id, 'mf_receipt_response', $receipt_response );
			if ( ! $receipt_response ) {
				// Failure.
				update_post_meta( $order_id, '_mf_receipt_status', 'receipt-failed' );
				Logger::error( 'Order receipt refund failed. (Order ID: ' . $order_id . ')' );

				$parent_order->add_order_note( 'Order receipt refund failed. See log for error message.' );
			} else {
				// Success.
				update_post_meta( $order_id, '_mf_receipt_status', 'receipt-sent' );
				Logger::debug( 'Order receipt refund sent successfully. (Order ID: ' . $order_id . ')' );
				$receipt_response_obj = json_decode( $receipt_response );
				$receipt_id           = $receipt_response_obj->Receipt->Id;
				$voucher_id           = $receipt_response_obj->Receipt->Voucher->Id;
				update_post_meta( $order_id, 'mf_receipt_id', $receipt_id );
				update_post_meta( $order_id, 'mf_receipt_voucher_id', $voucher_id );
				$parent_order->add_order_note( 'Order receipt refund sent successfully. (Receipt: ' . $receipt_id . '; Voucher: ' . $voucher_id . ')' );
			}
		}
	}

	/**
	 * Checks if refund can be made in a way that is valid for Apps.
	 *
	 * @param WC_Order_Refund $wc_order_refund WC Order Refund.
	 *
	 * @throws Exception If refund is not valid.
	 */
	public static function validate_refund_order( WC_Order_Refund $wc_order_refund ): void {
		$order_refund = wc_get_order( $wc_order_refund );
		$order_items  = $order_refund->get_items();

		$quantity_order_items     = array_filter(
			$order_items,
			function ( $order_item ) {
				return $order_item->get_quantity() !== 0;
			}
		);
		$non_quantity_order_items = array_filter(
			$order_items,
			function ( $order_item ) {
				return $order_item->get_quantity() === 0;
			}
		);
		if ( count( $quantity_order_items ) > 0 && count( $non_quantity_order_items ) > 0 ) {
			// Since row quantity cannot be 0, we cannot have some rows with quantity and some without.
			// If stock management is off, we can set the quantity to 1, but not if it's on.
			// That would mess with the stock in Apps.
			throw new Exception(
				esc_html__(
					'Refunds cannot contain both rows with quantity and rows without. Please make the refund in two steps.',
					'flo-sync'
				)
			);
		}
	}

	/**
	 * Resets stock quantity for updated items.
	 * If not already done in this request.
	 *
	 * @param int $order_id WC Order ID.
	 */
	public static function maybe_reset_order_items_stock_quantity( int $order_id ): void {
		static $already_reset = array();
		Logger::notice( 'Start resetting product stock for order ' . $order_id );

		if ( ! isset( $already_reset[ $order_id ] ) ) {
			$already_reset[ $order_id ] = array();
		}

		$exclude_order_statuses = array( 'failed', 'refunded', 'cancelled', 'completed' );
		$order                  = wc_get_order( $order_id );

		if ( $order instanceof WC_Order_Refund ) {
			$order = wc_get_order( $order->get_parent_id() );
		}

		if ( ! $order instanceof WC_Order ) {
			Logger::error( 'Order is not a woo order. OrderID#' . $order_id );
			return;
		}

		$order_status = $order->get_status();

		if ( in_array( $order_status, $exclude_order_statuses, true ) ) {
			Logger::notice( 'Order status is excluding order status #' . $order_status . ' OrderID#' . $order_id );
			return;
		}

		$order_instance = new Order_Model( $order );

		foreach ( $order->get_items() as $item ) {
			/**
			 * Method get_items defaults to line items.
			 *
			 * @var WC_Order_Item_Product $item
			 */

			if ( $item->get_variation_id() > 0 ) {
				$product = wc_get_product( $item->get_variation_id() );
			} elseif ( ! empty( $item->get_product_id() ) ) {
				$product = wc_get_product( $item->get_product_id() );
			} else {
				$product = null;
			}

			if ( is_a( $product, 'WC_Product' ) ) {

				// Check if this article has already been reset in this request.
				if ( in_array( $product->get_id(), $already_reset[ $order_id ], true ) ) {
					continue;
				}

				// Try resetting stock quantity.
				$order_instance->reset_product_quantity( $product );

				$already_reset[ $order_id ][] = $product->get_id();
			}
		}
		Logger::notice( 'End resetting product stocks' );
	}

	/**
	 * Generate order hash.
	 *
	 * @param array $order Order to generate hash for.
	 *
	 * @return string
	 */
	public static function generate_order_hash( array $order ): string {
		$hash_string = '';

		if ( ! empty( $order ) ) {
			$hash_string = wp_hash( wp_json_encode( $order ), 'nonce' );
		}

		if ( ! is_string( $hash_string ) ) {
			$hash_string = '';
		}

		return $hash_string;
	}

	/**
	 * Gets Apps order number from WooCommerce order if possible
	 *
	 * @param int $order_id WC Order ID.
	 *
	 * @return string|null Apps order number, null on error or not found
	 */
	public static function get_nmf_order_id( int $order_id ): ?string {
		$value              = null;
		$last_sync_response = get_post_meta( $order_id, 'mf_last_sync_response', true );

		if ( ! $last_sync_response ) {
			return null;
		}

		if ( is_string( $last_sync_response ) ) {
			$last_sync_response = json_decode( $last_sync_response );
		}

		if ( $last_sync_response ) {
			if ( ! empty( $last_sync_response->list ) && $last_sync_response->list[0]->Order ) {
				$value = $last_sync_response->list[0]->Order->Number
					? $last_sync_response->list[0]->Order->Number
					: null;
			}
		}

		return (string) $value;
	}

	/**
	 * Add BNPL transaction note to the given note.
	 *
	 * @param string $note     The note to add BNPL transaction information to.
	 * @param int    $order_id The ID of the order.
	 *
	 * @return string The updated note with BNPL transaction information.
	 */
	public static function add_bnpl_transaction_note( string $note, int $order_id ): string {
		$wc_order = wc_get_order( $order_id );
		if ( ! $wc_order ) {
			return $note;
		}

		$hub_order_number = self::get_nmf_order_id( $order_id );

		if ( empty( $hub_order_number ) ) {
			return $note;
		}

		if ( ! empty( $note ) ) {
			$note .= ' ';
		}

		$note .= __( 'Order nr', 'flo-sync' ) . ': ' . $hub_order_number . '. ';
		$note .= __( 'Name', 'flo-sync' ) . ': ' . $wc_order->get_billing_first_name() . ' ' . $wc_order->get_billing_last_name() . '. ';
		$note .= __( 'Phone', 'flo-sync' ) . ': ' . $wc_order->get_billing_phone() . '. ';
		$note .= __( 'Email', 'flo-sync' ) . ': ' . $wc_order->get_billing_email() . '.';

		return $note;
	}
}
