<?php
/**
 * Creates receipts.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Orders\Receipts;

use Automattic\WooCommerce\Utilities\NumberUtil;
use NMFS\Logger;
use NMFS\Sync\Items\Bookkeeping\Vat_Account_Codes\Vat_Accounts;
use NMFS\Sync\Items\Orders\Models\Order_Model;
use NMFS\Sync\Items\Orders\Orders;
use NMFS\Sync\Items\Orders\Receipts\Models\Article_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Customer_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Merchant_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Order_Reference_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Payment_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Receipt_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Vat_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Voucher_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Voucher_Row_Model;
use stdClass;
use WC_Order;
use WC_Order_Item;
use WC_Order_Item_Product;
use WC_Order_Item_Shipping;
use WC_Order_Item_Tax;
use WC_Order_Refund;
use WP_Error;

defined( 'ABSPATH' ) || exit;

/**
 * Class Factory
 */
class Factory {

	/**
	 * The precision to which numbers should be rounded.
	 *
	 * @var int $rounding_precision
	 */
	private static int $rounding_precision = 2;

	/**
	 * Create receipt from WooCommerce order or refund order
	 *
	 * @param Order_Model $order_instance      Order model object.
	 * @param bool        $do_stock_management If the receipt should be in charge of stock.
	 *
	 * @return Receipt_Model|WP_Error Receipt on success, else WP_Error
	 */
	public static function create_receipt( Order_Model $order_instance, bool $do_stock_management ): WP_Error|Receipt_Model {

		if ( wc_get_price_decimals() !== self::$rounding_precision ) {
			return new WP_Error(
				'receipt_price_decimals_error',
				'Price decimals are set to ' . wc_get_price_decimals() . ', it has to be: ' . self::$rounding_precision . '.'
			);
		}

		$text_rows = array();

		$order = $order_instance->get_order();

		$company_corp_no  = $order_instance->get_backoffice_account()->org_num;
		$company_name     = $order_instance->get_backoffice_account()->org_name;
		$shipping_sku     = $order_instance->get_backoffice_account()->get_shipping_settings()->get_article_sku();
		$shipping_account = $order_instance->get_backoffice_account()->get_shipping_settings()->get_article_account_code();

		$customer = $order_instance->get_customer();
		if ( ! empty( $customer ) ) {
			if ( empty( $customer['CustomerNumber'] ) ) {
				$customer['CustomerNumber'] = $order_instance->get_customer_number( $customer );
				if ( ! $customer['CustomerNumber'] ) {
					$customer['CustomerNumber'] = null;
				}
			}
		}

		$customer_number = $customer['CustomerNumber'];

		$vat_account_codes = Vat_Accounts::get_default_account_codes( $order_instance->get_backoffice_account() );

		$order_items = $order->get_items();
		if ( empty( $order_items ) ) {
			return new WP_Error( 'receipt_no_order_items', 'Order contains no items: ' . $order->get_id() );
		}

		$order_references      = array();
		$order_mf_id           = get_post_meta( $order->get_id(), 'mf_id', true );
		$order_reference_index = null;
		if ( $order_mf_id ) {
			$order_reference_index = 0;
			$order_reference       = self::create_order_reference(
				array(
					'Index'            => $order_reference_index,
					'OrderNumber'      => Orders::get_nmf_order_id( $order->get_id() ),
					'Origin'           => 'AppsOrder',
					'OrderChangeState' => is_a( $order, 'WC_Order' )
						? 'Paid' // WC_Order.
						: 'Cancelled', // WC_Order_Refund.
				)
			);
			$order_references[]    = $order_reference;
		}

		$parent_order = 'shop_order_refund' === $order->get_type()
			? wc_get_order( $order->get_parent_id() )
			: null;

		// Make sure we can get apps order id.
		if ( is_null( $parent_order ) ) {
			if ( is_null( Orders::get_nmf_order_id( $order->get_id() ) ) ) {
				return new WP_Error( 'receipt_missing_order_apps_id', 'Order has not correctly been synced to Apps: ' . $order->get_id() );
			}
		} elseif ( is_null( Orders::get_nmf_order_id( $parent_order->get_id() ) ) ) {
			return new WP_Error( 'receipt_missing_order_apps_id_refund', 'Order for refund has not correctly been synced to Apps: ' . $parent_order->get_id() );
		}

		$order_payment_method = is_null( $parent_order )
			? $order->get_payment_method()
			: $parent_order->get_payment_method();

		$payment_method_account =
			apply_filters(
				'flo_sync_receipt_payment_method_account_code',
				Helpers::get_income_account_code( $order_instance->get_backoffice_account(), $order_payment_method ),
				$order_payment_method,
			);

		if ( empty( $payment_method_account ) ) {
			return new WP_Error( 'receipt_no_payment_method_account', 'No bookkeeping account set for payment method: ' . $order_payment_method );
		}

		$woocommerce_calc_taxes_enable = get_option( 'woocommerce_calc_taxes' );
		if ( 'yes' !== $woocommerce_calc_taxes_enable ) {
			return new WP_Error( 'receipt_taxes_off', 'Taxes must be enabled' );
		}

		$coupon_name = __( 'Discount', 'flo-sync' );

		$articles = array();
		foreach ( $order_items as $order_item ) {
			if ( ! is_a( $order_item, 'WC_Order_Item_Product' ) ) {
				return new WP_Error( 'receipt_unknown_order_item', 'Unknown order item: ' . $order_item->get_name() );
			}

			$articles[] = self::maybe_create_article( $order_item, $order_instance, $order_reference_index, $coupon_name, $parent_order );
		}

		$shipping = self::maybe_create_shipping( $order, $shipping_sku, $shipping_account );
		if ( ! is_null( $shipping ) ) {
			$articles[] = $shipping;
		}

		if ( in_array( null, $articles, true ) ) {
			return new WP_Error( 'receipt_articles_missing', __( 'Articles from order are missing', 'flo-sync' ) );
		}

		$vat       = array();
		$tax_items = $order->get_taxes();
		if ( ! empty( $tax_items ) ) {
			foreach ( $tax_items as $tax_item ) {
				if ( ! $tax_item->get_tax_total() && ! $tax_item->get_shipping_tax_total() ) {
					// Ignore empty tax totals.
					continue;
				}

				$vat_row = self::maybe_create_vat(
					$tax_item,
					$articles,
					$vat_account_codes
				);
				if ( is_null( $vat_row ) ) {
					return new WP_Error( 'vat_account_code_missing', __( 'VAT account code missing.', 'flo-sync' ) );
				}

				$vat[] = $vat_row;
			}
		}

		$internal_note = null;
		if ( 'shop_order_refund' === $order->get_type() ) {
			$internal_note = $order->get_reason();
		}

		$receipt_date = is_null( $parent_order )
			? $order->get_date_paid()
			: $order->get_date_created();

		if ( is_null( $receipt_date ) ) {
			return new WP_Error( 'order_not_paid', __( 'Order has not been paid', 'flo-sync' ) );
		}

		$merchant = self::create_merchant(
			array(
				'CorporateIdentityNumber' => $company_corp_no,
				'Name'                    => $company_name,
				'AddressLine1'            => WC()->countries->get_base_address(),
				'AddressLine2'            => WC()->countries->get_base_address_2(),
				'ZipCode'                 => WC()->countries->get_base_postcode(),
				'City'                    => WC()->countries->get_base_city(),
				'Country'                 => WC()->countries->get_base_country(),
				'Phone'                   => null,
				'HomePage'                => get_site_url(),
				'Email'                   => null,
			)
		);

		$customers = array(
			self::create_customer(
				array(
					'CustomerNumber' => $customer_number,
					'CompanyName'    => is_null( $parent_order )
						? $order->get_billing_company()
						: $parent_order->get_billing_company(),
					'Fullname'       => is_null( $parent_order )
						? $order->get_formatted_billing_full_name()
						: $parent_order->get_formatted_billing_full_name(),
					'CustomerType'   => 'Customer',
				)
			),
		);

		$transaction_id = is_null( $parent_order )
			? $order->get_transaction_id()
			: $parent_order->get_transaction_id();

		switch ( strtolower( $order_payment_method ) ) {
			case 'northmill_bnpl':
				$receipt_payment_type = 'NorthmillBNPL';
				$receipt_payment_name = __( 'Pay later', 'flo-sync' );

				$row1         = new stdClass();
				$row1->RowNum = 0;
				$row1->Text   = __( 'Pay later transaction ID', 'flo-sync' );
				$text_rows[]  = $row1;

				$row2         = new stdClass();
				$row2->RowNum = 1;
				$row2->Text   = $transaction_id;

				$text_rows[] = $row2;
				break;

			case 'swish_pay_gateway':
				$receipt_payment_type = 'Swish';
				$receipt_payment_name = __( 'Swish', 'flo-sync' );
				break;

			case 'nmi_pay_gateway':
				$receipt_payment_type = 'CardPayment';
				$receipt_payment_name = __( 'Card Payment', 'flo-sync' );
				break;

			default:
				$receipt_payment_type = 'NamedPayment';
				$receipt_payment_name = __( 'Online Payment', 'flo-sync' );
				break;
		}

		$payments = array(
			self::create_payment(
				array(
					'Name'              => $receipt_payment_name,
					'Amount'            => $order->get_total(),
					'PaymentType'       => $receipt_payment_type,
					'TransactionId'     => $transaction_id,
					'TransactionOrigin' => $order_payment_method,
				)
			),
		);

		$voucher = self::create_voucher( $order, $articles, $vat, $payment_method_account );

		return new Receipt_Model(
			array(
				'ReceiptNumber'    => $order->get_id(),
				'Origin'           => 'WooCommerce - ' . wp_parse_url( get_site_url(), PHP_URL_HOST ),
				'ReceiptType'      => 'Online',
				'InternalNote'     => $internal_note,
				'TransactionDate'  => $receipt_date->format( 'Y-m-d\TH:i:s\.\0\0\0\Z' ),
				'TotalPriceIncVat' => NumberUtil::round( $order->get_total(), self::$rounding_precision ),
				'TotalPriceExVat'  => NumberUtil::round( $order->get_total() - $order->get_total_tax(), self::$rounding_precision ),
				'TotalVat'         => NumberUtil::round( $order->get_total_tax(), self::$rounding_precision ),
				'DoStockMgmt'      => $do_stock_management,
				'Merchant'         => $merchant,
				'Customers'        => $customers,
				'Articles'         => $articles,
				'Payments'         => $payments,
				'Orders'           => $order_references,
				'Discounts'        => array(),
				'Vat'              => $vat,
				'Voucher'          => $voucher,
				'TextRows'         => $text_rows,
			)
		);
	}

	/**
	 * Create article from order item product.
	 *
	 * @param WC_Order_Item_Product|WC_Order_Item $order_item            Order item.
	 * @param Order_Model                         $order_instance        Order that the item refers to.
	 * @param int                                 $order_reference_index Reference to order.
	 * @param string                              $coupon_name           Coupon name is only used if article is discounted.
	 * @param WC_Order|null                       $parent_order          Parent order if refund.
	 * @param bool                                $include_bookkeeping   Whether to generate bookkeeping, which includes API request.
	 *
	 * @return Article_Model|null Article on success, else null.
	 */
	public static function maybe_create_article(
		WC_Order_Item_Product|WC_Order_Item $order_item,
		Order_Model $order_instance,
		int $order_reference_index,
		string $coupon_name,
		?WC_Order $parent_order = null,
		bool $include_bookkeeping = true
	): ?Article_Model {
		$product = $order_item->get_product();
		$order   = $order_instance->get_order();

		if ( ! is_a( $product, 'WC_Product' ) ) {
			return null; // Product cannot be found, abort.
		}

		$vat = is_null( $parent_order )
			? Helpers::get_order_row_tax_rate( $order_item, $order )
			: Helpers::get_order_row_tax_rate( $order_item, $parent_order );

		$price = (float) $product->get_regular_price();
		if ( 'no' === get_option( 'woocommerce_prices_include_tax' ) ) {
			$tax_amount = $price * ( $vat / 100 );
			$price      = $price + $tax_amount;
		}

		$sales_price     = $order->get_item_total( $order_item, true );
		$discount_amount = 0;
		if ( $sales_price < $price ) {
			$discount_amount = abs( $price - $order->get_item_total( $order_item, true, false ) );
		}

		$attributes    = array();
		$serial_number = Helpers::get_serial_number( $product );
		if ( $serial_number ) {
			/**
			 * Misspelling of "Attribute" is from API.
			 *
			 * @noinspection SpellCheckingInspection
			 */
			$attributes[] = array(
				'AttrbuteType' => 'SerialNumber',
				'Value'        => $serial_number,
			);
		}

		$quantity = $order_item->get_quantity();
		if ( $quantity < 0 ) {
			// Refund rows should contain positive quantities and negative unit prices.
			$quantity *= - 1;
			$price    *= - 1;
		}

		$discounts = array();
		if ( $discount_amount ) {
			$discounts[] = array(
				'DiscountType' => 'amount',
				'Amount'       => NumberUtil::round( $discount_amount * $quantity, self::$rounding_precision ),
				'Description'  => $coupon_name,
				'Percentage'   => null, // null during type 'amount'.
			);
		}

		$article_name = $order_item->get_name();
		if ( is_a( $product, 'WC_Product_Variation' ) ) {
			// WooCommerce sometimes doesn't add all the proper attributes to the name of the variation. This will force add them to the title.
			$possible_attributes = array_keys( $product->get_attributes() );
			$title_base          = get_post_field( 'post_title', $product->get_parent_id() );
			$meta_data           = $order_item->get_formatted_meta_data();
			$meta_data           = array_filter(
				$meta_data,
				function ( $v ) use ( $possible_attributes ) {
					return in_array( $v->key, $possible_attributes, true );
				}
			);

			if ( ! empty( $meta_data ) ) {
				$meta_data = array_map(
					function ( $v ) {
						return $v->value;
					},
					$meta_data
				);

				$title_separator     = ' - ';
				$attribute_separator = ', ';
				$title_suffix        = trim( wp_strip_all_tags( implode( $attribute_separator, $meta_data ) ) );
				if ( $title_suffix && $title_base ) {
					if ( $title_base === $article_name ) {
						// Attributes are missing from article name entirely.
						$article_name = $article_name . $title_separator . $title_suffix;
					} else {
						// Some attributes are in the title.
						$article_name = $article_name . $attribute_separator . $title_suffix;
					}
				}
			}
		}

		if ( $include_bookkeeping ) {
			Logger::debug( 'Getting bookkeeping account code for product ' . $product->get_id() );
			$bookkeeping_account = Helpers::get_article_account_code_by_product( $product, $order_instance );
		} else {
			$bookkeeping_account = '';
		}

		return new Article_Model(
			array(
				'OrderIndex'                => $order_reference_index,
				'ArticleNumber'             => $product->get_sku(),
				'Name'                      => $article_name,
				'Quantity'                  => $quantity,
				'UnitPrice'                 => NumberUtil::round(
					$order->get_item_total( $order_item, true, false ),
					self::$rounding_precision
				),
				'UnitPriceWithoutDiscounts' => NumberUtil::round(
					$price,
					self::$rounding_precision
				),
				'VatRate'                   => $vat,
				'TotalPriceIncVat'          => NumberUtil::round(
					$order->get_line_total( $order_item, true ),
					self::$rounding_precision
				),
				'TotalPriceExVat'           => NumberUtil::round(
					$order->get_line_total( $order_item ),
					self::$rounding_precision
				),
				'TotalVat'                  => NumberUtil::round(
					$order_item->get_total_tax(),
					self::$rounding_precision
				),
				'BookkeepingAccount'        => $bookkeeping_account,
				'Discounts'                 => $discounts,
				'Attributes'                => $attributes,
				'type'                      => 'product',
				'taxes_totals'              => $order_item->get_taxes(),
			)
		);
	}

	/**
	 * Create a VAT model.
	 *
	 * @param WC_Order_Item_Tax $tax_item     Tax item to create VAT for.
	 * @param Article_Model[]   $articles     Articles to create VAT for.
	 * @param array             $vat_accounts Account codes for VAT rates.
	 *
	 * @return Vat_Model|null
	 */
	private static function maybe_create_vat( WC_Order_Item_Tax $tax_item, array $articles, array $vat_accounts ): ?Vat_Model {
		$total_amount = 0.0;
		$total_vat    = 0.0;
		foreach ( $articles as $article ) {
			$article_tax_total = $article->get_tax_total( $tax_item->get_rate_id() );
			if ( is_null( $article_tax_total ) ) {
				continue;
			}

			$total_amount += $article->get_total_price_inc_vat();
			$total_vat     = NumberUtil::round( $total_vat + $article_tax_total, self::$rounding_precision ); // Fix binary/decimal conversion.
		}
		if ( ! $total_amount ) {
			return null;
		}

		$vat_rate    = $tax_item->get_rate_percent( 'edit' );
		$vat_account = null;
		if ( 0 === intval( $vat_rate ) ) {
			$vat_account = $vat_accounts[0];
		} elseif ( 6 === intval( $vat_rate ) ) {
			$vat_account = $vat_accounts[6];
		} elseif ( 12 === intval( $vat_rate ) ) {
			$vat_account = $vat_accounts[12];
		} elseif ( 25 === intval( $vat_rate ) ) {
			$vat_account = $vat_accounts[25];
		}

		if ( empty( $vat_account ) ) {
			return null;
		}

		return new Vat_Model(
			array(
				'VatRate'            => $vat_rate,
				'TotalVat'           => $total_vat,
				'TotalAmount'        => NumberUtil::round( $total_amount, self::$rounding_precision ),
				'BookkeepingAccount' => $vat_account,
			)
		);
	}

	/**
	 * Get shipping article.
	 *
	 * @param WC_Order $order            Order to create shipping article for.
	 * @param string   $shipping_sku     Shipping article SKU.
	 * @param int|null $shipping_account Shipping article account code.
	 *
	 * @return Article_Model|null Article on success, null if no shipping.
	 */
	private static function maybe_create_shipping( WC_Order $order, string $shipping_sku, ?int $shipping_account ): ?Article_Model {
		$shipping_tax_rate = 0;
		$shipping_tax      = floatval( $order->get_shipping_tax() );
		$shipping_total    = floatval( $order->get_shipping_total() ) + $shipping_tax;
		$taxes_totals      = null;

		if ( $shipping_tax ) {
			/**
			 * Tax line items.
			 *
			 * @var WC_Order_Item_Shipping $tax_line
			 */
			$tax_lines = $order->get_items( 'shipping' );
			if ( ! empty( $tax_lines ) ) {
				foreach ( $tax_lines as $tax_line ) {
					$taxes_totals       = $tax_line->get_taxes();
					$shipping_tax_total = NumberUtil::round(
						array_reduce(
							$tax_line->get_taxes()['total'],
							function ( $carry, $item ) {
								return $carry + floatval( $item );
							}
						),
						self::$rounding_precision
					);
					$shipping_tax_rate  = ceil( 100 * ( ( ( floatval( $order->get_shipping_total() ) + $shipping_tax_total ) / floatval( $order->get_shipping_total() ) ) - 1 ) );
				}
			}
		}

		if ( ! $shipping_total > 0 ) {
			return null;
		}

		return new Article_Model(
			array(
				'ArticleNumber'      => $shipping_sku,
				'Name'               => __( 'Shipping', 'flo-sync' ),
				'Quantity'           => 1,
				'UnitPrice'          => NumberUtil::round( $shipping_total, self::$rounding_precision ),
				'VatRate'            => NumberUtil::round( $shipping_tax_rate, self::$rounding_precision ),
				'TotalPriceIncVat'   => NumberUtil::round( $shipping_total, self::$rounding_precision ),
				'TotalPriceExVat'    => NumberUtil::round( $order->get_shipping_total(), self::$rounding_precision ),
				'TotalVat'           => NumberUtil::round( $shipping_tax, self::$rounding_precision ),
				'BookkeepingAccount' => $shipping_account,
				'Discounts'          => array(),
				'Attributes'         => array(),
				'type'               => 'shipping',
				'taxes_totals'       => $taxes_totals,
			)
		);
	}

	/**
	 * Creat voucher for order.
	 *
	 * @param WC_Order|WC_Order_Refund $order                  Order to create voucher for.
	 * @param Article_Model[]          $articles               Articles to create voucher for.
	 * @param Vat_Model[]              $vats                   Vats to create voucher for.
	 * @param ?int                     $payment_method_account Payment method account code.
	 *
	 * @return Voucher_Model
	 */
	private static function create_voucher(
		WC_Order|WC_Order_Refund $order,
		array $articles,
		array $vats,
		?int $payment_method_account
	): Voucher_Model {
		$parent_order = 'shop_order_refund' === $order->get_type()
			? wc_get_order( $order->get_parent_id() )
			: null;

		$rows = array(
			array(
				'AccountNumber' => (string) $payment_method_account,
				'DebitAmount'   => NumberUtil::round( $order->get_total(), self::$rounding_precision ),
				'CreditAmount'  => 0,
			),
		);

		foreach ( $articles as $article ) {
			if ( 'product' === $article->get_type() && $article->get_total_price_ex_vat() !== 0.0 ) {
				$rows[] = array(
					'AccountNumber' => $article->get_bookkeeping_account(),
					'DebitAmount'   => 0,
					'CreditAmount'  => NumberUtil::round( $article->get_total_price_ex_vat(), self::$rounding_precision ),
				);
			} elseif ( 'shipping' === $article->get_type() && $article->get_total_price_ex_vat() !== 0.0 ) {
				$rows[] = array(
					'AccountNumber' => $article->get_bookkeeping_account(),
					'DebitAmount'   => 0,
					'CreditAmount'  => NumberUtil::round( $article->get_total_price_ex_vat(), self::$rounding_precision ),
				);
			}
		}

		if ( ! empty( $vats ) ) {
			foreach ( $vats as $vat ) {
				if ( $vat->get_total_vat() !== 0.0 ) {
					$rows[] = array(
						'AccountNumber' => $vat->get_bookkeeping_account(),
						'DebitAmount'   => 0,
						'CreditAmount'  => NumberUtil::round( $vat->get_total_vat(), self::$rounding_precision ),
					);
				}
			}
		}

		// Swap Debit and Credit for negative values.
		$rows = array_map(
			function ( $row ) {
				if ( $row['DebitAmount'] < 0 ) {
					$row['CreditAmount'] += abs( $row['DebitAmount'] );
					$row['DebitAmount']   = 0;
				}
				if ( $row['CreditAmount'] < 0 ) {
					$row['DebitAmount'] += abs( $row['CreditAmount'] );
					$row['CreditAmount'] = 0;
				}

				return $row;
			},
			$rows
		);

		// Transform into voucher rows.
		$rows = array_map(
			function ( $row ) {
				return new Voucher_Row_Model( $row );
			},
			$rows
		);

		$voucher_date = is_null( $parent_order )
			? $order->get_date_paid()
			: $order->get_date_created();

		$voucher_text = is_null( $parent_order )
			// translators: %s is the order number.
			? sprintf( __( 'WooCommerce Order #%s', 'flo-sync' ), Orders::get_nmf_order_id( $order->get_id() ) )
			// translators: %1$d is the refund id, %2$s is the order number.
			: sprintf( __( 'WooCommerce refund (%1$d) of Order #%2$s', 'flo-sync' ), $order->get_id(), Orders::get_nmf_order_id( $parent_order->get_id() ) );

		return new Voucher_Model(
			array(
				'State'           => 'Pending',
				'ReferenceNumber' => is_null( $parent_order )
					? $order->get_order_number()
					: $parent_order->get_order_number() . '-refund-' . $order->get_id(),
				'ReferenceType'   => 'WebOrder',
				'Rows'            => $rows,
				'VoucherDate'     => is_null( $voucher_date )
					? null
					: $voucher_date->format( 'Y-m-d\\TH:i:s' ),
				'VoucherText'     => $voucher_text,
			)
		);
	}

	/**
	 * Create merchant.
	 *
	 * @param array $data Merchant data.
	 *
	 * @return Merchant_Model
	 */
	private static function create_merchant( array $data ): Merchant_Model {
		return new Merchant_Model( $data );
	}

	/**
	 * Create customer.
	 *
	 * @param array $data Customer data.
	 *
	 * @return Customer_Model
	 */
	private static function create_customer( array $data ): Customer_Model {
		return new Customer_Model( $data );
	}

	/**
	 * Create receipt payment model.
	 *
	 * @param array $data Payment data.
	 *
	 * @return Payment_Model
	 */
	private static function create_payment( array $data ): Payment_Model {
		return new Payment_Model( $data );
	}

	/**
	 * Create a receipt order reference model.
	 *
	 * @param array $data Order reference data.
	 *
	 * @return Order_Reference_Model
	 */
	private static function create_order_reference( array $data ): Order_Reference_Model {
		return new Order_Reference_Model( $data );
	}
}
