<?php
/**
 * Helper functions for receipts.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Orders\Receipts;

use NMFS\Logger;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;
use NMFS\Sync\Items\Articles\Articles;
use NMFS\Sync\Items\Bookkeeping\Vat_Account_Codes\Vat_Accounts;
use NMFS\Sync\Items\Orders\Models\Order_Model;
use NMFS\Sync\Items\Orders\Receipts\Models\Payment_Gateway_Collection;
use WC_Cart;
use WC_Order;
use WC_Order_Item_Product;
use WC_Product;
use WC_Tax;

defined( 'ABSPATH' ) || exit;

/**
 * Class Helpers
 */
class Helpers {
	/**
	 * Get bookkeeping article account number from a product.
	 *
	 * @param WC_Product  $product       Product to get account code for.
	 * @param Order_Model $order_instance Order that the product belongs to.
	 *
	 * @return int|false The account code, false if not found.
	 */
	public static function get_article_account_code_by_product( WC_Product $product, Order_Model $order_instance ): false|int {
		$account_codes = $order_instance->get_backoffice_account()->get_article_account_codes();

		if ( empty( $account_codes ) ) {
			return false;
		}

		if ( is_a( $product, 'WC_Product_Variation' ) ) {
			$product_sku = wc_get_product( $product->get_parent_id() )->get_sku();
		} else {
			$product_sku = $product->get_sku();
		}

		$product_account_code_id = Articles::get_article_bookkeeping_account_code_id(
			$product_sku,
			$order_instance->get_backoffice_account()->api
		);

		if ( ! $product_account_code_id ) {
			return false;
		}

		foreach ( $account_codes as $account_code ) {
			if ( $account_code->get_id() !== $product_account_code_id ) {
				continue;
			}

			return $account_code->get_account_code();
		}

		return false;
	}

	/**
	 * Get tax percentage for order item of order.
	 *
	 * @param WC_Order_Item_Product $order_item Order item to get tax percentage for.
	 * @param WC_Order              $order      Order that the item refers to.
	 *
	 * @return int
	 */
	public static function get_order_row_tax_rate( WC_Order_Item_Product $order_item, WC_Order $order ): int {
		$tax_percentage = 0;

		$tax_class = $order_item->get_tax_class();
		$args      = array(
			'country'   => $order->get_billing_country(),
			'state'     => $order->get_billing_state(),
			'postcode'  => $order->get_billing_postcode(),
			'city'      => $order->get_billing_city(),
			'tax_class' => $tax_class,
		);

		if ( empty( $args['country'] ) ) {
			// Default to base.
			$args['country']  = WC()->countries->get_base_country();
			$args['state']    = WC()->countries->get_base_state();
			$args['postcode'] = WC()->countries->get_base_postcode();
			$args['city']     = WC()->countries->get_base_city();
		}
		$tax_data = WC_Tax::find_rates( $args );

		if ( ! empty( $tax_data ) ) {
			foreach ( $tax_data as $tax_rate ) {
				$tax_percentage = $tax_rate['rate'];
			}
		}

		return $tax_percentage;
	}

	/**
	 * Gets tax percentage for product in cart.
	 *
	 * @param WC_Product $product Product to get tax percentage for.
	 * @param WC_Cart    $cart    Cart that the product is in.
	 *
	 * @return int
	 */
	public static function get_cart_product_tax_rate( WC_Product $product, WC_Cart $cart ): int {
		$tax_percentage = 0;
		$wc_tax         = new WC_Tax();

		$tax_class = $product->get_tax_class();

		$tax_data = $wc_tax->get_rates( $tax_class, $cart->get_customer() );

		if ( ! empty( $tax_data ) ) {
			foreach ( $tax_data as $tax_rate ) {
				$tax_percentage = $tax_rate['rate'];
			}
		}

		return $tax_percentage;
	}

	/**
	 * Gets serial number of a product.
	 *
	 * @param WC_Product $product Product to get serial number for.
	 *
	 * @return string Serial number, empty string on not found.
	 */
	public static function get_serial_number( WC_Product $product ): string {
		return ( is_a( $product, 'WC_Product_Variation' ) )
			? get_post_meta( $product->get_parent_id(), '_custom_product_barcodes_field', true )
			: get_post_meta( $product->get_id(), '_custom_product_barcodes_field', true );
	}

	/**
	 * Check if all enabled Checkout payment methods have an income account code set.
	 *
	 * @param string|null                   $payment_gateway             Payment gateway ID. Can be supplied to only check this specific payment gateway.
	 * @param ?Payment_Gateway_Collection[] $invalid_gateway_collections Array filled with invalid payment gateways. Passed by reference.
	 * @param bool                          $log_errors                  Whether to log errors or not.
	 *
	 * @return bool
	 */
	public static function is_required_income_account_codes_set(
		?string $payment_gateway = null,
		?array &$invalid_gateway_collections = array(),
		bool $log_errors = true
	): bool {
		$payment_gateway_collections = self::get_payment_methods_creating_hub_receipts();

		if ( ! is_array( $invalid_gateway_collections ) ) {
			$invalid_gateway_collections = array();
		}

		foreach ( $payment_gateway_collections as $payment_gateway_collection ) {
			$payment_method = $payment_gateway_collection->payment_gateway;

			if ( $payment_gateway && $payment_gateway !== $payment_method->id ) {
				continue;
			}

			$backoffice_accounts = Backoffice_Accounts::get_all_current_accounts();

			foreach ( $backoffice_accounts as $backoffice_account ) {
				if ( ! self::get_income_account_code( $backoffice_account, $payment_method->id ) ) {
					$error = 'Missing income account code for payment method ' . $payment_method->id . ' in Hub account ' . wp_json_encode( $backoffice_account );
					if ( $log_errors ) {
						Logger::error( $error );
					} else {
						Logger::debug( $error );
					}

					$invalid_payment_gateway_collection                     = clone $payment_gateway_collection;
					$invalid_payment_gateway_collection->backoffice_account = $backoffice_account;

					$invalid_gateway_collections[] = $invalid_payment_gateway_collection;
				}
			}
		}

		return empty( $invalid_gateway_collections );
	}

	/**
	 * Check if all VAT account codes are set.
	 * TODO: This could be updated to check which VAT accounts are enabled in Hub and only check those?
	 *
	 * @param ?array $invalid_accounts Array filled with invalid Hub accounts. Passed by reference.
	 * @param bool   $log_errors       Whether to log errors or not.
	 *
	 * @return bool True if all VAT account codes are set, false otherwise.
	 */
	public static function is_required_vat_account_codes_set( ?array &$invalid_accounts = array(), bool $log_errors = true ): bool {
		$backoffice_accounts = Backoffice_Accounts::get_all_current_accounts();
		$invalid_accounts    = array();

		foreach ( $backoffice_accounts as $backoffice_account ) {
			if ( ! is_a( $backoffice_account, Backoffice_Account_Model::class ) ) {
				continue;
			}

			$vat_account_codes = Vat_Accounts::get_default_account_codes( $backoffice_account );
			if (
				empty( $vat_account_codes[0] ) ||
				empty( $vat_account_codes[6] ) ||
				empty( $vat_account_codes[12] ) ||
				empty( $vat_account_codes[25] )
			) {
				$error = 'Missing VAT account code in Hub account: ' . wp_json_encode( $backoffice_account );
				if ( $log_errors ) {
					Logger::error( $error );
				} else {
					Logger::debug( $error );
				}

				$invalid_accounts[] = $backoffice_account;
			}
		}

		return empty( $invalid_accounts );
	}

	/**
	 * Get all payment methods that can be used in the Hub to create Receipts.
	 *
	 * @param bool $only_available Whether to only return payment methods that are available in WooCommerce.
	 *
	 * @return Payment_Gateway_Collection[]
	 */
	public static function get_payment_methods_creating_hub_receipts( bool $only_available = true ): array {

		$payment_method_map = array(
			'NorthmillBNPL'   => array(
				'northmill_bnpl',
			),
			'BankCardPayment' => array(
				'bacs',                 // Woocommerce Core - BACS.
				'nmi_pay_gateway',      // MoreFlo Checkout - NMI Bank Card Payment.
			),
			'Invoice'         => array(
				'moreflo_invoice',      // MoreFlo Checkout - Moreflo Invoice.
			),
			'CashPayment'     => array(
				'cod',                  // Woocommerce Core - Cash on Delivery.
			),
			'Swish'           => array(
				'swish_pay_gateway',    // MoreFlo Checkout - Swish.
			),
			'Klarna'          => array(
				'klarna_payments',
			),
			'Paypal'          => array(
				'ppcp-gateway',
			),
			'ResursBank'      => array(
				'invoice_pays',         // MoreFlo Checkout - Resurs Bank Invoice.
			),
			'Stripe'          => array(
				'stripe',
				'stripe_alipay',
				'stripe_multibanco',
				'stripe_sepa',
				'stripe_giropay',
				'stripe_ideal',
				'stripe_bancontact',
				'stripe_eps',
				'stripe_p24',
				'stripe_boleto',
				'stripe_oxxo',
			),
			'SveaWebPay'      => array(
				'sveawebpay_card',
				'sveawebpay_direct_bank',
				'sveawebpay_invoice',
				'sveawebpay_part_pay',
			),
			'WooPayments'     => array(
				'woocommerce_payments',
			),
			'ExternalPayment' => array( // The "others"-option.
				'cheque',               // Woocommerce Core - Cheque.
			),
		);

		/**
		 * Filter to add/remove payment methods that can be used in the Hub to create Receipts.
		 *
		 * @param array $payment_method_map Payment methods that can be used in the Hub to create Receipts.
		 *                                  The KEYS in the MAIN ARRAY should NOT be changed by the filter.
		 *                                        They correspond to Income account codes in Hub.
		 *                                        PaymentType as received from API.
		 *                                  The filter can change the VALUES in the SUB ARRAYS.
		 *                                        They correspond to payment gateway IDs in WooCommerce.
		 */
		$payment_method_map = apply_filters(
			'flo_sync_receipt_payment_gateway_map',
			$payment_method_map
		);

		$flattened_receipt_gateways = array();

		foreach ( $payment_method_map as $key => $values ) {
			foreach ( $values as $value ) {
				$flattened_receipt_gateways[ $value ] = $key;
			}
		}

		/**
		 * Magic method okay for WC().
		 *
		 * @noinspection PhpUndefinedFieldInspection
		 */
		$payment_gateways = $only_available ?
			WC()->payment_gateways->get_available_payment_gateways() :
			WC()->payment_gateways->payment_gateways();

		$payment_gateway_collections = array();

		foreach ( $payment_gateways as $payment_gateway ) {
			$payment_gateway_collections[] = new Payment_Gateway_Collection(
				array(
					'payment_gateway' => $payment_gateway,
					'iac_pay_type'    => $flattened_receipt_gateways[ $payment_gateway->id ] ?? 'ExternalPayment',
				)
			);
		}

		return $payment_gateway_collections;
	}

	/**
	 * Get income account code for payment gateway.
	 *
	 * @param Backoffice_Account_Model $backoffice_account Hub account to get account code for.
	 * @param string                   $payment_gateway_id Payment gateway.
	 *
	 * @return ?int
	 */
	public static function get_income_account_code(
		Backoffice_Account_Model $backoffice_account,
		string $payment_gateway_id
	): ?int {
		$account_codes = $backoffice_account->get_income_account_codes();

		$account_code = $account_codes[ $payment_gateway_id ] ?? null;

		return ! is_null( $account_code ) ? $account_codes[ $payment_gateway_id ]->get_account_code() : null;
	}
}
