<?php
/**
 * Handles receipts.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Orders\Receipts;

use NMFS\Logger;
use NMFS\Settings;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use WC_Abstract_Order;
use WC_Order;
use WP_Error;

defined( 'ABSPATH' ) || exit;

/**
 * Class Receipts
 */
class Receipts {

	/**
	 * Receipts constructor.
	 */
	public static function setup(): void {
		add_filter( 'moreflo_sync_before_order_sync', array( self::class, 'before_order_sync' ), 10, 2 );
		add_filter( 'moreflo_sync_before_order_refund_sync', array( self::class, 'before_order_refund_sync' ), 10, 3 );
		add_action( 'woocommerce_after_checkout_validation', array( self::class, 'after_checkout_validation' ), 9, 2 );
		add_action( 'woocommerce_new_order', array( self::class, 'maybe_update_order_receipt_status' ), 5 );
	}

	/**
	 * Check if receipts are enabled.
	 *
	 * @return bool Enabled status.
	 */
	public static function is_enabled(): bool {
		return Settings::is_setting_enabled( 'is_receipt_active' );
	}

	/**
	 * Check if receipt function is set up correctly.
	 */
	public static function is_setup(): bool {
		if ( ! self::is_enabled() ) {
			return false;
		}

		$bo_account = Backoffice_Accounts::get_current_main_account();

		if ( ! $bo_account ) {
			return false;
		}

		if ( empty( $bo_account->org_name ) || empty( $bo_account->org_num ) ) {
			return false;
		}

		return true;
	}

	/**
	 * When Receipt creation is on, check if booking accounts are set for payment method and VAT.
	 * Run on woocommerce_after_checkout_validation action.
	 *
	 * @param array    $data Checkout data.
	 * @param WP_Error $errors Checkout errors.
	 *
	 * @return void
	 */
	public static function after_checkout_validation( array $data, WP_Error $errors ): void {
		if ( self::is_enabled() ) {
			// TODO: Make this only check for Hub Account of order.
			if ( ! Helpers::is_required_income_account_codes_set( $data['payment_method'] ) ) {
				Logger::error( 'Abort order on checkout validation, income account code not set for payment method.' );
				$errors->add( 'checkout', __( 'Payment method setup incorrectly, please contact us.', 'flo-sync' ) );
			}

			// TODO: Make this only check for Hub Account of order.
			if ( ! Helpers::is_required_vat_account_codes_set() ) {
				Logger::error( 'Abort order on checkout validation, all required VAT account codes not set.' );
				$errors->add( 'checkout', __( 'VAT accounts setup incorrectly, please contact us.', 'flo-sync' ) );
			}
		}
	}

	/**
	 * Filters payment status based on receipt status.
	 *  - Needs Receipt AND FullyPaid => Set payment status to NotPaid
	 *
	 * @param array             $mf_order Array to send to Apps.
	 * @param WC_Abstract_Order $order    WooCommerce order or refund order.
	 */
	public static function before_order_sync( array $mf_order, WC_Abstract_Order $order ): array {
		// Is receipts enabled?
		if ( ! self::is_enabled() ) {
			return $mf_order;
		}

		// Needs Receipt AND FullyPaid => Set payment status to NotPaid.
		if (
			'needs-receipt' === get_post_meta( $order->get_id(), '_mf_receipt_status', true ) &&
			'FullyPaid' === $mf_order['PaymentStatus']
		) {
			$mf_order['PaymentStatus'] = 'NotPaid';
		}

		return $mf_order;
	}

	/**
	 * Filters payment status based on receipt status.
	 *  - Has not sent receipt AND FullyPaid => Set payment status to NotPaid
	 *
	 * @param array    $mf_order     Array to send to Apps.
	 * @param WC_Order $order WooCommerce order. Not WC_Order_Refund class, but WC_Order.
	 *                                  See refund_order function in Orders.php.
	 * @param WC_Order $parent_order Parent order for refund.
	 *
	 * @return array
	 */
	public static function before_order_refund_sync( array $mf_order, WC_Order $order, WC_Order $parent_order ): array {
		// Is receipts enabled?
		if ( ! self::is_enabled() ) {
			return $mf_order;
		}

		// Make sure the parent order has sent receipt.
		if ( 'receipt-sent' !== get_post_meta( $parent_order->get_id(), '_mf_receipt_status', true ) ) {
			return $mf_order;
		}

		// Has not sent receipt AND FullyPaid => Set payment status to NotPaid.
		if (
			'receipt-sent' !== get_post_meta( $order->get_id(), '_mf_receipt_status', true )
			&& 'FullyPaid' === $mf_order['PaymentStatus']
		) {
			$mf_order['PaymentStatus'] = 'NotPaid';
		}

		return $mf_order;
	}

	/**
	 * Get the receipt status of an order.
	 *
	 * @param int $order_id The ID of the order.
	 *
	 * @return string|false The status of the receipt or false if status is empty.
	 */
	public static function get_order_receipt_status( int $order_id ): string|false {
		$status = get_post_meta( $order_id, '_mf_receipt_status', true );
		return ! empty( $status ) ? $status : false;
	}

	/**
	 * Update the receipt status for a given order.
	 *
	 * @param int    $order_id The ID of the order to update.
	 * @param string $status   The new receipt status to set.
	 *
	 * @return bool True if status was updated or unchanged, false if update failed.
	 */
	public static function update_order_receipt_status( int $order_id, string $status ): bool {
		$current_status = self::get_order_receipt_status( $order_id );
		if ( $current_status === $status ) {
			return true;
		}
		return (bool) update_post_meta( $order_id, '_mf_receipt_status', $status );
	}

	/**
	 * Check if order needs a receipt.
	 *
	 * @param int $order_id ID of order to check.
	 *
	 * @return void
	 */
	public static function maybe_update_order_receipt_status( int $order_id ): void {
		// Is receipts enabled?
		if ( ! self::is_enabled() ) {
			Logger::debug( 'Receipts are not enabled.' );
			return;
		}

		// If this order lacks receipt, set status.
		if ( ! self::get_order_receipt_status( $order_id ) ) {
			self::update_order_receipt_status( $order_id, 'needs-receipt' );
		} else {
			Logger::debug( 'Order already has receipt status.' );
		}
	}

	/**
	 * Reset the receipt status for a given order if it is marked as 'receipt-failed'.
	 *
	 * @param int $order_id The ID of the order.
	 *
	 * @return void
	 */
	public static function maybe_reset_receipt_status( int $order_id ): void {
		// Get current receipt status.
		$current_status = self::get_order_receipt_status( $order_id );
		if ( 'receipt-failed' === $current_status ) {
			// Remove order meta.
			delete_post_meta( $order_id, '_mf_receipt_status' );

			// Reset order status.
			self::maybe_update_order_receipt_status( $order_id );
		}
	}
}
