<?php
/**
 * Handles syncing of orders.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Orders;

use NMFS\Helpers;
use NMFS\Logger;
use NMFS\Sync\Abstract_Syncer;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\Items\Orders\Models\Order_Model;
use NMFS\Sync\Items\Orders\Orders as Order_Functions;
use NMFS\Sync\Sync_Updated_Status;

defined( 'ABSPATH' ) || exit;

/**
 * Class Sync
 */
class Sync extends Abstract_Syncer {

	/**
	 * Number of items to get in each API request.
	 *
	 * @var int
	 */
	protected int $page_size = 10;

	/**
	 * Construct class.
	 *
	 * @param array  $backoffice_accounts Array of Hub accounts to sync stock for.
	 * @param string $type                Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function __construct( array $backoffice_accounts = array(), string $type = 'cron' ) {
		if ( empty( $backoffice_accounts ) ) {
			$backoffice_accounts = Backoffice_Accounts::get_all_current_accounts();
		}
		parent::__construct( $backoffice_accounts, $type );
	}

	/**
	 * Sync orders.
	 *
	 * @param array|null $args Query parameters.
	 *
	 * @return bool
	 */
	public function sync( ?array $args = null ): bool {
		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}

		$orders = $current_backoffice_account->api->get_orders( $args );

		if (
			! isset( $orders['Total'] ) ||
			! isset( $orders['Page'] ) ||
			! isset( $orders['list'] )
		) {
			return false;
		}

		$this->update_status( 'page', $orders['Page'] );
		$this->update_status( 'total', $orders['Total'] );

		if ( is_array( $orders['list'] ) ) {
			foreach ( $orders['list'] as $order ) {
				if ( ! empty( $order['Order'] ) ) {
					$action = $this->process_pulled_order( $order['Order'] );

					if ( $action ) {
						$this->increase_status( $action );
					}
				}
			}
		}

		return true;
	}

	/**
	 * Process pulled order.
	 *
	 * @param array $mf_order Order data.
	 *
	 * @return string
	 */
	private function process_pulled_order( array $mf_order ): string {
		$return = 'ignore';
		$mf_id  = $mf_order['Id'];

		Logger::debug( 'Processing order ' . $mf_id );

		$wc_orders = wc_get_orders(
			array(
				'limit'      => 1,
				'meta_key'   => 'mf_id',
				'meta_value' => $mf_id,
			)
		);

		if ( ! empty( $wc_orders ) && $mf_order['State'] && $mf_order['PaymentStatus'] ) {
			$wc_order = array_shift( $wc_orders );
			Logger::debug( 'Found WC order ' . $wc_order->get_id() );

			$order_instance = new Order_Model( $wc_order );
			$updated        = $order_instance->update_order( $mf_order['PaymentStatus'] );

			if ( $updated ) {
				$return = 'update';
			}
		} elseif ( $this->is_cron() ) {
			Logger::error( 'Order ' . $mf_id . ' not found in WooCommerce' );
		} elseif ( $this->is_ajax() ) {
			Logger::debug( 'Order ' . $mf_id . ' not found in WooCommerce' );
		}

		Logger::debug( 'Processed order ' . $mf_id . ' (' . $return . ')' );

		return $return;
	}

	/**
	 * Disables order save action in order sync loop.
	 *
	 * @param array|null $args API params passed by reference.
	 */
	public function before_sync_loop( ?array &$args ): void {
		Logger::debug( 'Disabling order save action in order sync loop' );
		Order_Functions::set_hook_save_order( false );

		if ( $this->is_ajax() && ! isset( $args['modifiedSince'] ) ) {
			Logger::debug( 'Setting modifiedSince to 1900-01-01T00:00:00, since the parameter is required for the endpoint.' );
			$args['modifiedSince'] = '1900-01-01T00:00:00';
		}
	}

	/**
	 * Checks if order sync is enabled.
	 *
	 * @return bool
	 */
	public function is_enabled(): bool {
		if ( $this->is_cron() ) {
			if ( ! Sync_Updated_Status::get_wp_option( 'orders' ) ) {
				// This is the first time we're syncing orders. Any old orders shouldn't be synced.
				Sync_Updated_Status::set_wp_option( 'orders', Helpers::get_current_time(), 'ajax' );
				Logger::debug(
					'mf_orders_sync_last_updated option not found, setting it to current time and skipping sync'
				);

				return false;
			}
		}
		return true;
	}
}
