<?php
/**
 * Handles syncing of webshop settings.
 *
 * @package flo-sync
 */

namespace NMFS\Sync\Items\Store_Settings;

use NMFS\Logger;
use NMFS\Sync\Abstract_Syncer;
use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;
use NMFS\Sync\API\Backoffice_Accounts\Sync as Backoffice_Account_Sync;

defined( 'ABSPATH' ) || exit;

/**
 * Syncer class for Store settings.
 */
class Sync extends Abstract_Syncer {

	/**
	 * Number of items to get in each API request.
	 *
	 * @var int
	 */
	protected int $page_size = 100;


	/**
	 * Construct class.
	 *
	 * @param Backoffice_Account_Model[] $backoffice_accounts Array of Hub accounts to sync stock for.
	 * @param string                     $type                Type of syncer. Can be 'cron' or 'ajax'.
	 */
	public function __construct( array $backoffice_accounts = array(), string $type = 'cron' ) {
		if ( empty( $backoffice_accounts ) ) {
			$backoffice_accounts = Backoffice_Accounts::get_all_current_accounts();
		}
		parent::__construct( $backoffice_accounts, $type );
	}

	/**
	 * Sync store settings.
	 *
	 * @param null|array $args Arguments.
	 */
	public function sync( ?array $args = null ): bool {
		$current_backoffice_account = $this->get_current_backoffice_account();

		if ( ! $current_backoffice_account ) {
			return false;
		}

		$store_settings = $current_backoffice_account->api->get_store_settings();
		$this->update_status( 'total', 1 );

		if ( ! isset( $store_settings->Settings ) ) {
			Logger::error( 'Store settings not found in API response.' );
			Logger::error( $store_settings );

			return false;
		}

		Backoffice_Account_Sync::process_pulled_backoffice_account(
			$current_backoffice_account,
			$store_settings->Settings,
			'update'
		);

		$partial_refund_article_sku = $store_settings->Settings->PartialRefundArticleNumber ?? 'null';

		if ( $partial_refund_article_sku !== $current_backoffice_account->partial_refund_article_sku ) {
			Backoffice_Accounts::update_account_property(
				$current_backoffice_account->wp_id,
				'partial_refund_article_sku',
				$partial_refund_article_sku
			);
		}

		$shipping_article_sku = (string) $store_settings->Settings->ShippingArticleNumber ?? '';

		$current_shipping_settings = $current_backoffice_account->get_shipping_settings();
		$current_shipping_sku      = $current_shipping_settings->get_article_sku();

		if ( $current_shipping_sku !== $shipping_article_sku ) {
			$current_shipping_settings->set_article_sku( $shipping_article_sku );

			Backoffice_Accounts::update_account_property(
				$current_backoffice_account->wp_id,
				'shipping_settings',
				$current_shipping_settings
			);
		}

		$this->increase_status( 'update' );

		return true;
	}
}
