<?php
/**
 * Sync updated status.
 *
 * @package flo-sync
 */

namespace NMFS\Sync;

use NMFS\Logger;
use NMFS\Sync\Items\Articles\Sync as Article_Sync;
use NMFS\Sync\Items\Articles\Stock\Sync as Stock_Sync;
use NMFS\Sync\Items\Bookkeeping\Article_Account_Codes\Sync as Article_Account_Code_Sync;
use NMFS\Sync\Items\Bookkeeping\Income_Account_Codes\Sync as Income_Account_Code_Sync;
use NMFS\Sync\Items\Bookkeeping\Vat_Account_Codes\Sync as Vat_Account_Code_Sync;
use NMFS\Sync\Items\Bookkeeping\Other_Account_Codes\Sync as Other_Account_Code_Sync;
use NMFS\Sync\Items\Campaigns\Sync as Campaign_Sync;
use NMFS\Sync\Items\Campaigns\Sets\Sync as Campaign_Set_Sync;
use NMFS\Sync\Items\Groups\Sync as Group_Sync;
use NMFS\Sync\Items\Orders\Sync as Order_Sync;
use NMFS\Sync\Items\Store_Settings\Sync as Store_Setting_Sync;

/**
 * Class Sync_Updated_Status
 */
class Sync_Updated_Status {

	/**
	 * Array of syncers that can be updated.
	 *
	 * @var array
	 */
	private static array $updated_since;

	/**
	 * Setup.
	 */
	public static function setup(): void {
		self::$updated_since = array(
			'articles'       => array(
				'option_name'   => 'mf_article_sync_last_updated',
				'syncer'        => Article_Sync::class,
				'api_parameter' => 'ArticleInfoUpdatedSince',
			),
			'stock'          => array(
				'option_name'   => 'nmf_article_stock_sync_last_updated',
				'syncer'        => Stock_Sync::class,
				'api_parameter' => 'QuantityUpdatedSince',
			),
			'aac'            => array(
				'option_name' => 'nmf_sync_article_account_codes_last_updated',
				'syncer'      => Article_Account_Code_Sync::class,
			),
			'iac'            => array(
				'option_name' => 'mf_sync_income_account_codes_last_updated',
				'syncer'      => Income_Account_Code_Sync::class,
			),
			'vac'            => array(
				'option_name' => 'mf_sync_bookkeeping_last_updated',
				'syncer'      => Vat_Account_Code_Sync::class,
			),
			'oac'            => array(
				'option_name' => 'mf_sync_other_account_codes_last_updated',
				'syncer'      => Other_Account_Code_Sync::class,
			),
			'campaigns'      => array(
				'option_name' => 'nmf_sync_campaigns_last_updated', // Unused.
				'syncer'      => Campaign_Sync::class,
			),
			'campaign_sets'  => array(
				'option_name' => 'mf_sync_campaigns_article_sets_last_updated',
				'syncer'      => Campaign_Set_Sync::class,
			),
			'groups'         => array(
				'option_name' => 'mf_sync_groups_last_updated', // Unused.
				'syncer'      => Group_Sync::class,
			),
			'orders'         => array(
				'option_name'   => 'mf_orders_sync_last_updated',
				'syncer'        => Order_Sync::class,
				'api_parameter' => 'modifiedSince',
			),
			'store_settings' => array(
				'option_name' => 'nmf_sync_store_settings_last_updated',
				'syncer'      => Store_Setting_Sync::class,
			),
		);

		add_action( 'wp_ajax_nmf_update_latest_sync', array( __CLASS__, 'ajax_update_latest_sync_time' ) );
	}

	/**
	 * Validate type.
	 * If the type is not valid, return empty string.
	 * Transforms option_name to type.
	 *
	 * @param string $type Type (array key in self::$updated_since), or option_name.
	 *
	 * @return string
	 */
	public static function validate_type( string $type ): string {
		if ( isset( self::$updated_since[ $type ] ) ) {
			return $type;
		}

		foreach ( self::$updated_since as $key => $data ) {
			if ( $data['option_name'] === $type ) {
				return $key;
			}
		}

		Logger::error( 'Invalid type: ' . $type );
		return '';
	}

	/**
	 * Get all updated since date options.
	 *
	 * @return array
	 */
	public static function get_all(): array {
		foreach ( self::$updated_since as $type => $data ) {
			self::get_wp_option( $type );
		}
		return self::$updated_since;
	}

	/**
	 * Get 'updated since' date for the type.
	 *
	 * @param string $type Type (array key in self::$updated_since).
	 * @param string $time_type Time type (main, ajax, cron).
	 *
	 * @return string
	 */
	public static function get_wp_option( string $type, string $time_type = 'main' ): string {
		$type = self::validate_type( $type );
		if ( ! isset( self::$updated_since[ $type ]['time'] ) ) {
			self::$updated_since[ $type ]['times'] = array(
				'main' => get_option( self::get_option_name( $type ), '' ),
				'ajax' => get_option( self::get_option_name( $type ) . '_ajax', '' ),
				'cron' => get_option( self::get_option_name( $type ) . '_cron', '' ),
			);
		}
		return self::$updated_since[ $type ]['times'][ $time_type ];
	}

	/**
	 * Set 'updated since' date for the type.
	 *
	 * @param string      $type Type (array key in self::$updated_since).
	 * @param string      $date Date.
	 * @param null|string $updated_from Source of update (ajax, cron).
	 *
	 * @return void
	 */
	public static function set_wp_option( string $type, string $date, ?string $updated_from ): void {
		$option_name = self::get_option_name( $type );
		if ( ! $option_name ) {
			Logger::error( 'Sync_Updated_Status::set_wp_option Invalid type: ' . $type . '. Date: ' . $date );
			return;
		}

		update_option( self::get_option_name( $type ), $date );

		if ( ! empty( $updated_from ) ) {
			$allowed_sources = array( 'ajax', 'cron' );
			if ( in_array( $updated_from, $allowed_sources, true ) ) {
				update_option( self::get_option_name( $type ) . '_' . $updated_from, $date );
			}
		}
	}

	/**
	 * Get wp option name for the type.
	 *
	 * @param string $type Type (array key in self::$updated_since).
	 *                     Can also be the option name.
	 *
	 * @return null|string
	 */
	public static function get_option_name( string $type ): ?string {
		if ( isset( self::$updated_since[ $type ] ) ) {
			return self::$updated_since[ $type ]['option_name'];
		} else {
			foreach ( self::$updated_since as $data ) {
				if ( $data['option_name'] === $type ) {
					return $type;
				}
			}
		}

		return null;
	}

	/**
	 * Get option name by syncer class.
	 *
	 * @param string $syncer_class Name of syncer class.
	 */
	public static function get_option_name_by_syncer_class( string $syncer_class ): string {
		foreach ( self::$updated_since as $data ) {
			if ( $data['syncer'] === $syncer_class ) {
				return $data['option_name'];
			}
		}
		return '';
	}

	/**
	 * Get api parameter by syncer class.
	 *
	 * @param string $syncer_class Name of syncer class.
	 *
	 * @return string
	 */
	public static function get_api_parameter_by_syncer_class( string $syncer_class ): string {
		foreach ( self::$updated_since as $data ) {
			if ( $data['syncer'] === $syncer_class ) {
				return $data['api_parameter'] ?? '';
			}
		}
		return '';
	}

	/**
	 * Get display name for the type.
	 *
	 * @param string $type Type (array key in self::$updated_since).
	 *
	 * @return string
	 */
	public static function get_display_name( string $type ): string {
		$names = array(
			'articles'       => __( 'Articles', 'flo-sync' ),
			'stock'          => __( 'Stock', 'flo-sync' ),
			'aac'            => __( 'Article Account Codes', 'flo-sync' ),
			'iac'            => __( 'Income Account Codes', 'flo-sync' ),
			'vac'            => __( 'VAT Account Codes', 'flo-sync' ),
			'oac'            => __( 'Other Account Codes', 'flo-sync' ),
			'campaigns'      => __( 'Campaigns', 'flo-sync' ),
			'campaign_sets'  => __( 'Campaigns sets', 'flo-sync' ),
			'groups'         => __( 'Groups', 'flo-sync' ),
			'orders'         => __( 'Orders', 'flo-sync' ),
			'store_settings' => __( 'Store Settings', 'flo-sync' ),
		);

		return $names[ $type ] ?? '';
	}

	/**
	 * Manually update latest sync time via status page ajax.
	 *
	 * @return void
	 */
	public static function ajax_update_latest_sync_time(): void {
		// @phpcs:disable WordPress.Security.NonceVerification.Missing
		$option    = htmlspecialchars( $_POST['latest_sync_option'] );
		$new_value = htmlspecialchars( $_POST['latest_sync'] );
		// @phpcs:enable

		if ( ! $option || ! $new_value ) {
			wp_send_json_error( 'Missing option or new value' );
		}

		self::set_wp_option( $option, $new_value, null );

		wp_send_json_success( 'Updated' );
	}
}
