<?php
/**
 * Handles views.
 *
 * @package flo-sync
 */

namespace NMFS;

use NMFS\Sync\API\Backoffice_Accounts\Backoffice_Accounts;
use NMFS\Sync\API\Backoffice_Accounts\Models\Backoffice_Account_Model;
use NMFS\Sync\Items\Articles\Stock\Shipping_Zones;
use NMFS\Sync\Items\Orders\Orders;
use NMFS\Sync\API\Backoffice_Accounts\Helpers as Backoffice_Account_Helpers;
use NMFS\Sync\API\Backoffice_Accounts\Envs as Backoffice_Accounts_Envs;

/**
 * Class for handling views.
 */
class Views_Admin {

	/**
	 * Setup actions/filters for class.
	 */
	public static function setup(): void {

		add_action( 'admin_menu', array( self::class, 'add_flo_menu_items' ) );

		// Show Apps order number in admin.
		add_filter(
			'manage_shop_order_posts_columns',
			array( self::class, 'manage_shop_order_posts_columns' ),
			20
		);

		add_filter(
			'manage_shop_order_posts_custom_column',
			array( self::class, 'manage_shop_order_posts_custom_column' ),
			10,
			2
		);
	}

	/**
	 * Add Flo menu items to the admin menu.
	 */
	public static function add_flo_menu_items(): void {
		$capability = 'manage_options';

		$page_title = __( 'WP Northmill Sync', 'flo-sync' );
		$menu_title = __( 'Northmill Sync', 'flo-sync' );
		$slug       = 'northmill_flo_sync_admin';
		$callback   = array( self::class, 'render_sync_page' );
		$icon       = NMF_PLUGIN_URL . 'assets/img/nm_logo.png';
		$position   = 100;

		if ( 'inactive' === Sync\Cron\Cron::get_cron_event( 'sync' )->get_status() ) {
			$menu_title .= ' <span class="awaiting-mod">!</span>';
		}

		add_menu_page( $page_title, $menu_title, $capability, $slug, $callback, $icon, $position );

		$parent_slug = 'northmill_flo_sync_admin';
		$page_title  = 'WP Northmill Sync';
		$menu_title  = __( 'Settings', 'flo-sync' );
		$slug        = NMF_SYNC_SETTINGS_SLUG;
		$callback    = array( self::class, 'render_settings_page' );

		add_submenu_page( $parent_slug, $page_title, $menu_title, $capability, $slug, $callback, 1 );

		if ( Debug::is_on() ) {
			$page_title = 'WP Northmill Sync Status';
			$menu_title = __( 'Status', 'flo-sync' );
			$slug       = 'northmill_flo_sync_status';
			$callback   = array( self::class, 'render_status_page' );

			add_submenu_page( $parent_slug, $page_title, $menu_title, $capability, $slug, $callback, 2 );
		}
	}

	/**
	 * Render a view.
	 *
	 * @param string $file_name        File name.
	 * @param array  $args             Array of arguments to pass to view.
	 * @param bool   $use_require_once Whether to use require_once or require.
	 *
	 * @return void
	 */
	public static function render_admin_view( string $file_name, array $args = array(), bool $use_require_once = true ): void {
		/**
		 * $args used in view file.
		 *
		 * @noinspection PhpExpressionResultUnusedInspection
		 */
		$args;
		$full_path = NMF_PLUGIN_VIEWS_DIR . 'admin/' . $file_name;
		if ( file_exists( $full_path ) ) {
			if ( $use_require_once ) {
				require_once $full_path;
			} else {
				require $full_path;
			}
		}
	}

	/**
	 * Render the Flo sync page.
	 */
	public static function render_sync_page(): void {
		$args['available_manual_sync_items_per_page'] = array(
			10  => 10,
			20  => 20,
			30  => 30,
			50  => 50,
			100 => 100,
			200 => 200,
		);

		self::render_admin_view( 'sync-view.php', $args );
	}

	/**
	 * Renders plugin settings page content, and saves settings on form submission.
	 */
	public static function render_settings_page(): void {
		Settings::maybe_update_from_post();

		$mf_settings = Settings::get_all();

		$args = array();

		$args['settings_notices'] = Backoffice_Account_Helpers::get_settings_notices();

		/**
		 * Hub accounts settings.
		 */
		$args['current_env']       = $mf_settings[ NMF_SYNC_ENV ] ?? '';
		$args['valid_envs']        = Backoffice_Accounts_Envs::get_valid_envs();
		$args['stock_accounts']    = Backoffice_Accounts::get_stock_accounts( null, true );
		$args['test_main_account'] = Backoffice_Accounts::get_main_account( 'test' );
		$args['prod_main_account'] = Backoffice_Accounts::get_main_account( 'prod' );

		/**
		 * Sync settings.
		 */
		$args['mf_auto_sync']                      = $mf_settings[ NMF_SYNC_STATUS ] ?? '';
		$args['orders_to_apps_status']             = $mf_settings['sync_new_orders_to_apps'] ?? 'enabled'; // Enabled by default.
		$args['products_publish_status']           = $mf_settings['sync_publish_products_status'] ?? '';
		$args['sync_only_product_stock_amount']    = $mf_settings['sync_only_product_stock_amount'] ?? '';
		$args['sync_price_status']                 = $mf_settings[ NMF_IS_PRICE_ACTIVE ] ?? '';
		$args['sync_article_description']          = $mf_settings['sync_article_description'] ?? '';
		$args['sync_pictures_status']              = $mf_settings['is_pic_active'] ?? '';
		$args['sync_product_title_status']         = $mf_settings['is_product_title_active'] ?? '';
		$args['sync_product_display_order_status'] = $mf_settings['sync_product_display_order_status'] ?? '';
		$args['sync_product_tax_class_status']     = $mf_settings['is_tax_class_active'] ?? 'enabled'; // Enabled by default.
		$args['sync_campaigns_status']             = $mf_settings['is_campaigns_active'] ?? '';
		$args['sync_product_categories_status']    = $mf_settings['is_product_categories_active'] ?? '';

		$args['sync_order_stock_adjustment'] = $mf_settings['sync_order_stock_adjustment'] ?? 'order_paid';
		$args['sync_interval']               = $mf_settings[ NMF_CRON_SYNC_EVENT_INTERVAL ] ?? '';

		$args['available_stock_adjust_options'] = array(
			'order_opened' => __( 'Order is opened', 'flo-sync' ),
			'order_paid'   => __( 'Order is paid', 'flo-sync' ),
		);

		$wp_schedules                = wp_get_schedules();
		$args['available_schedules'] = array();

		if ( is_array( $wp_schedules ) ) {
			foreach ( $wp_schedules as $key => $value ) {
				$args['available_schedules'][ $key ] = $value['display'];
			}
		}

		$invalid_gateway_collections = null;
		$invalid_vat_bo_accounts     = null;

		/**
		 * Receipt settings.
		 */
		$args['create_receipt_in_apps_status'] = $mf_settings['is_receipt_active'] ?? 'disabled';
		$args['receipts_disabled']             = Settings::get_receipt_select_input_disabled_status(
			$invalid_gateway_collections,
			$invalid_vat_bo_accounts
		);
		$args['receipts_select_extra_html']    = Settings::get_receipts_enabled_field_extra_html(
			$invalid_gateway_collections,
			$invalid_vat_bo_accounts
		);

		/**
		 * Misc settings.
		 */
		$args['current_log_level']    = $mf_settings['log_level'] ?? Logger::get_default_log_level();
		$args['available_log_levels'] = Logger::get_log_levels();
		$args['wc_log_a_tag']         = Helpers::get_wc_log_a_tag();

		$args['hub_order_number_prefix'] = $mf_settings['hub_order_number_prefix'] ?? '';

		self::render_admin_view( 'settings-view.php', $args );
	}

	/**
	 * Renders plugin settings page content, and saves settings on form submission.
	 */
	public static function render_status_page(): void {
		self::render_admin_view( 'status-view.php' );
	}

	/**
	 * Add title for Hub ID as column in the order list.
	 *
	 * @param array $columns Array of columns.
	 *
	 * @return array Updated array of columns.
	 */
	public static function manage_shop_order_posts_columns( array $columns ): array {
		return array_merge( $columns, array( 'apps_order_id' => 'Hub ID' ) );
	}

	/**
	 * Add content for Hub ID as column in the order list.
	 *
	 * @param string $column_key Column key.
	 * @param int    $post_id    Order post id.
	 *
	 * @return void
	 */
	public static function manage_shop_order_posts_custom_column( string $column_key, int $post_id ): void {
		if ( 'apps_order_id' === $column_key ) {
			$mf_apps_order_id = Orders::get_nmf_order_id( $post_id );
			if ( ! is_null( $mf_apps_order_id ) ) {
				echo esc_html( '#' . $mf_apps_order_id );
			}
		}
	}

	/**
	 * Render a text field for settings page.
	 *
	 * @param string $id            Field id.
	 * @param string $label         Field label.
	 * @param string $current_value Current value.
	 * @param string $description   Field description.
	 * @param string $extra_html    Extra html.
	 * @param bool   $is_disabled   Whether the select field itself is disabled.
	 *
	 * @return void
	 */
	public static function render_enabled_disabled_field(
		string $id,
		string $label,
		string $current_value,
		string $description = '',
		string $extra_html = '',
		bool $is_disabled = false
	): void {
		$options = array(
			'disabled' => __( 'Disabled', 'flo-sync' ),
			'enabled'  => __( 'Enabled', 'flo-sync' ),
		);

		self::render_setting_select_field(
			$id,
			$label,
			$current_value,
			$options,
			$description,
			$extra_html,
			$is_disabled
		);
	}

	/**
	 * Render a text field for settings page.
	 *
	 * @param string $id            Field id.
	 * @param string $label         Field label.
	 * @param string $current_value Current value.
	 * @param array  $options       Select options.
	 * @param string $description   Field description.
	 * @param string $extra_html    Extra HTML to render after description.
	 * @param bool   $is_disabled   Whether the field is disabled.
	 *
	 * @return void
	 */
	public static function render_setting_select_field(
		string $id,
		string $label,
		string $current_value,
		array $options,
		string $description = '',
		string $extra_html = '',
		bool $is_disabled = false
	): void {
		$args['id']            = $id;
		$args['label']         = $label;
		$args['current_value'] = $current_value;
		$args['options']       = $options;
		$args['description']   = $description;
		$args['extra_html']    = $extra_html;
		$args['is_disabled']   = $is_disabled;

		self::render_admin_view( 'settings/select-field.php', $args, false );
	}

	/**
	 * Render a text field for settings page.
	 *
	 * @param string $id            Field id.
	 * @param string $label         Field label.
	 * @param string $current_value Current value.
	 * @param string $description   Field description.
	 * @param string $extra_html    Extra HTML to render after description.
	 * @param bool   $is_disabled   Whether the field is disabled.
	 *
	 * @return void
	 */
	public static function render_setting_input_field(
		string $id,
		string $label,
		string $current_value,
		string $description = '',
		string $extra_html = '',
		bool $is_disabled = false
	): void {
		$args['id']            = $id;
		$args['label']         = $label;
		$args['current_value'] = $current_value;
		$args['description']   = $description;
		$args['extra_html']    = $extra_html;
		$args['is_disabled']   = $is_disabled;

		self::render_admin_view( 'settings/input-field.php', $args, false );
	}

	/**
	 * Render Hub account fields for settings page.
	 *
	 * @param array|null                    $field_iterators Array of field iterators.
	 * @param Backoffice_Account_Model|null $account         Hub account model to render fields for.
	 * @param string                        $env             Environment to render fields for.
	 * @param bool                          $is_template     Whether to render fields as template.
	 *
	 * @return void
	 */
	public static function render_backoffice_account_fields(
		?array &$field_iterators,
		?Backoffice_Account_Model $account = null,
		string $env = 'test',
		bool $is_template = false
	): void {

		$args =
			array(
				'row_desc'    => '',
				'tbody_id'    => '',
				'tbody_class' => '',
			);

		if ( ! $is_template ) {
			$args['api_url']                  = $account->url ?? '';
			$args['api_user']                 = $account->user ?? '';
			$args['api_pass']                 = $account->pass ?? '';
			$args['api_env']                  = $account->env ?? $env;
			$args['is_main']                  = empty( $account ) || $account->is_main();
			$args['display_name']             = is_a( $account, Backoffice_Account_Model::class ) ? $account->get_display_name() : '';
			$args['account_enabled']          = empty( $account ) || $account->is_enabled();
			$args['allowed_shipping_methods'] = is_a( $account, Backoffice_Account_Model::class ) ? $account->allowed_shipping_methods : array();
		} else {
			$args['tbody_id']                 = 'hub-account-template';
			$args['api_url']                  = '';
			$args['api_user']                 = '';
			$args['api_pass']                 = '';
			$args['api_env']                  = 'template';
			$args['is_main']                  = false;
			$args['display_name']             = '';
			$args['account_enabled']          = true;
			$args['allowed_shipping_methods'] = array();
		}

		$args['connection_status'] = ! empty( $account ) ? Backoffice_Account_Helpers::get_connection_status( $account ) : '';

		if ( $args['is_main'] ) {
			$args['row_desc'] = __( 'Account that data is synced from, as set in sync option below.', 'flo-sync' );

			if ( 'prod' === $args['api_env'] ) {
				$args['row_title'] = __( 'Production main account', 'flo-sync' );
			} else {
				$args['row_title'] = __( 'Test main account', 'flo-sync' );
			}
		} elseif ( 'prod' === $args['api_env'] ) {
			$args['row_title'] = __( 'Production stock account', 'flo-sync' );
		} else {
			$args['row_title'] = __( 'Test stock account', 'flo-sync' );
		}

		$args['tbody_id'] = $args['tbody_id'] ? 'id="' . esc_attr( $args['tbody_id'] ) . '"' : '';

		if ( empty( $field_iterators ) ) {
			$field_iterators = array(
				'test'     => 0,
				'prod'     => 0,
				'template' => 0,
			);
		}
		$current_iterator = $args['is_main'] ? 'main' : $field_iterators[ $args['api_env'] ]++;

		switch ( $args['api_env'] ) {
			case 'test':
				$args['tbody_class'] .= ' hub-account-test';
				break;
			case 'prod':
				$args['tbody_class'] .= ' hub-account-prod';
				break;
		}

		$current_env_iterator_id = '[' . $args['api_env'] . '][' . $current_iterator . ']';

		$args['url_input_id']          = 'bo_accounts' . $current_env_iterator_id . '[api_url]';
		$args['user_input_id']         = 'bo_accounts' . $current_env_iterator_id . '[api_user]';
		$args['pass_input_id']         = 'bo_accounts' . $current_env_iterator_id . '[api_pass]';
		$args['display_name_input_id'] = 'bo_accounts' . $current_env_iterator_id . '[api_display_name]';
		$args['enabled_input_id']      = 'bo_accounts' . $current_env_iterator_id . '[enabled]';
		$args['shipping_methods_id']   = 'bo_accounts' . $current_env_iterator_id . '[allowed_shipping_methods][]';

		$args['make_input_required'] = ! $args['is_main'] && ! $is_template;

		$args['shipping_methods_multi_select'] = Shipping_Zones::get_shipping_methods_multi_select(
			$args['shipping_methods_id'],
			'hub-account-user-allowed-shipping-methods multi-account-field',
			$args['allowed_shipping_methods'],
			$args['make_input_required']
		);

		self::render_admin_view( 'settings/backoffice-account-field.php', $args, false );
	}
}
